<?php

/**
 * Compress helper
 *
 * @package 	PG_Core
 * @subpackage 	Helpers
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('program_compress')) {

    /**
     * Compress content
     *
     * @param string $layout    layout content
     * @param string $file_type file type
     *
     * @return string
     */
    function program_compress($layout, $file_type = 'js')
    {
        $layout = str_replace("\r", "", $layout);
        $literal_strings = array();
        $lines = explode("\n", $layout);
        $clean = "";
        $inComment = false;
        $literal = "";
        $inQuote = false;
        $escaped = false;
        $quoteChar = "";
        $css = ($file_type == 'css') ? true : false;
        for ($i = 0; $i < count($lines); ++$i) {
            $line = $lines[$i];
            $inNormalComment = false;
            for ($j = 0; $j < strlen($line); ++$j) {
                $c = substr($line, $j, 1);
                $d = substr($line, $j, 2);
                if (!$inQuote && !$inComment) {
                    if (($c == "\"" || $c == "'") && !$inComment && !$inNormalComment) {
                        $inQuote = true;
                        $inComment = false;
                        $escaped = false;
                        $quoteChar = $c;
                        $literal = $c;
                    } elseif ($d == "/*" && !$inNormalComment) {
                        $inQuote = false;
                        $inComment = true;
                        $escaped = false;
                        $quoteChar = $d;
                        $literal = $d;
                        ++$j;
                    } elseif ($d == "//" and !$css) {
                        $inNormalComment = true;
                        $clean .= $c;
                    } else {
                        $clean .= $c;
                    }
                } else {
                    if ($c == $quoteChar && !$escaped && !$inComment) {
                        $inQuote = false;
                        $literal .= $c;
                        $clean .= "___" . count($literal_strings) . "___";
                        array_push($literal_strings, $literal);
                    } elseif ($inComment && $d == "*/") {
                        $inComment = false;
                        $literal .= $d;
                        if ($css) {
                            $clean .= '';
                        } else {
                            $clean .= "___" . count($literal_strings) . "___";
                        }

                        array_push($literal_strings, $literal);
                        ++$j;
                    } elseif ($c == "\\" && !$escaped) {
                        $escaped = true;
                    } else {
                        $escaped = false;
                    }
                    $literal .= $c;
                }
            }
            if ($inComment) {
                $literal .= "\n";
            }
            $clean .= "\n";
        }
        $lines = explode("\n", $clean);
        for ($i = 0; $i < count($lines); ++$i) {
            $line = $lines[$i];
            if (!$css) {
                $line = preg_replace("/\/\/(.*)/", "", $line);
            }
            $line = trim($line);
            $line = preg_replace("/\s+/", " ", $line);

//			if ($css)
//			{
//				$line = preg_replace("/\s*([!\}\{;,&=\|\+\/\)\(:])\s*/","\\1",$line);
//			}
//			else
//			{
//				$line = preg_replace("/\s*([!\}\{;,&=\|\-\+\*\/\)\(:])\s*/","\\1",$line);
//			}
            $lines[$i] = $line;
        }
        $layout = implode("\n", $lines);
        $layout = preg_replace("/[\n]+/", "\n", $layout);
        $layout = preg_replace("/;\n/", ";", $layout);
        $layout = preg_replace("/[\n]*\{[\n]*/", "{", $layout);
        for ($i = 0; $i < count($literal_strings); ++$i) {
            $layout = str_replace("___" . $i . "___", $literal_strings[$i], $layout);
        }

        return $layout;
    }
}
