<?php

namespace Pg\Modules\Contact\Controllers;

/**
 * Contact module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Contact user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Contact
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Contact extends \Controller
{

    /**
     * Class constructor
     *
     * @return Contact
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Contact_model");

        /* if ($this->session->userdata("auth_type") == "user") {
          show_404();
          } */
    }

    /**
     * Send message to contact by ajax
     *
     * @return void
     */
    public function ajax_send_message()
    {
        $post_data["user_id"] = intval($this->input->post("user_id"));
        $post_data["sender"] = $this->input->post("sender", true);
        $post_data["phone"] = $this->input->post("phone", true);
        $post_data["email"] = $this->input->post("email", true);
        $post_data["message"] = $this->input->post("message", true);
        $post_data["code"] = $this->input->post("code", true);

        $make_copies = $this->pg_module->get_module_config("contact", "contact_make_copies");
        $need_approve = $this->pg_module->get_module_config("contact", "contact_need_approve");

        $validate_data = $this->Contact_model->validate_contact(null, $post_data);

        $is_captcha_valid = ($this->input->post("code", true) == $this->session->userdata("captcha_word")) ? 1 : 0;
        if (!$is_captcha_valid) {
            $validate_data["errors"][] = l("error_invalid_captcha", "contact");
        }

        if (!empty($validate_data["errors"])) {
            $return["error"] = implode("<br>", $validate_data["errors"]);

            $this->view->assign($return);

            $this->view->render();

            return;
        } else {
            if ($need_approve) {
                $validate_data["data"]["allow_send"] = "1";

                $send_admin_mail = $this->pg_module->get_module_config("contact", "contact_send_mail");
                if ($send_admin_mail) {
                    $email = $this->pg_module->get_module_config("contact", "contact_admin_email");
                    if ($email) {
                        $this->load->model("Notifications_model");
                        $this->Notifications_model->send_notification($email, "auser_guest_contact", array());
                    }
                }
            } else {
                $validate_data["data"]["send"] = "1";

                //get user email
                $this->load->model("Users_model");
                $this->Users_model->set_format_settings('get_safe', false);
                $user = $this->Users_model->get_user_by_id($validate_data["data"]["user_id"]);
                $this->Users_model->set_format_settings('get_safe', true);
                $email = $user["email"];

                $mail_data = $validate_data["data"];
                $mail_data["user"] = $user['output_name'];

                //send mail
                $this->load->model("Notifications_model");
                $this->Notifications_model->send_notification($email, "user_guest_contact", $mail_data, '', $user['lang_id']);
            }

            if ($make_copies) {
                $this->Contact_model->save_contact(null, $validate_data["data"]);
            }

            $this->session->unset_userdata("captcha_word");

            $return["success"] = l("success_contact_save", "contact");

            $this->view->assign($return);

            $this->view->render();

            return;
        }

        return;
    }

    /**
     * Check form control code by ajax
     *
     * @return void
     */
    public function ajax_check_code()
    {
        $is_captcha_valid = $this->input->get("code", true) == $this->session->userdata("captcha_word") ? "true" : "false";
        $this->view->output($is_captcha_valid);
        $this->view->render();
    }

    /**
     * Refresh captcha image
     *
     * @return void
     */
    public function ajax_refresh_captcha()
    {
        $this->load->plugin("captcha");
        $this->config->load("captcha_settings");
        $captcha_settings = $this->config->item("captcha_settings");
        $captcha = create_captcha($captcha_settings);
        $this->session->set_userdata("captcha_word", $captcha["word"]);
        $data["captcha_image"] = $captcha["image"];
        $data["captcha_word_length"] = strlen($captcha["word"]);

        $this->view->assign($data);

        $this->view->render();
    }

}
