<?php

namespace Pg\Modules\Contact_us\Controllers;

/**
 * Contact us module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Contact us admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Countact us
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Contact_us extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Contact_us
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Contact_us_model");

        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'system-items');
    }

    /**
     * Manage contact us reasons
     *
     * @return void
     */
    public function index()
    {
        $attrs = $search_params = array();
        $reasons_count = $this->Contact_us_model->get_reason_count();

        if ($reasons_count > 0) {
            $reasons = $this->Contact_us_model->get_reason_list();
            $this->view->assign('reasons', $reasons);
        }
        $this->load->helper("navigation");
        $url = site_url() . "admin/contact_us";
        $page_data = get_admin_pages_data($url, $reasons_count, ($reasons_count ? $reasons_count : 10), 1, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->Menu_model->set_menu_active_item('admin_contacts_menu', 'reasons_list_item');
        $this->view->setHeader(l('admin_header_reasons_list', 'contact_us'));
        $this->view->render('list');
    }

    /**
     * Edit or create contact us reason
     *
     * @param integer $id reason identifier
     *
     * @return void
     */
    public function edit($id = null)
    {
        if ($id) {
            $data = $this->Contact_us_model->get_reason_by_id($id);
        } else {
            $data = array();
        }

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "mails" => $this->input->post('mails', true),
            );

            $name = $this->input->post('name', true);

            $validate_data = $this->Contact_us_model->validate_reason($id, $post_data, $name);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $flag_add = empty($id) ? true : false;
                $data["id"] = $id = $this->Contact_us_model->save_reason($id, $validate_data["data"], $validate_data["langs"]);

                if (!$flag_add) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_reason', 'contact_us'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_reason', 'contact_us'));
                }

                redirect(site_url() . "admin/contact_us");
            }
            $data = array_merge($data, $validate_data["data"]);
            $this->view->assign('validate_lang', $validate_data["langs"]);
            $temp = $this->Contact_us_model->format_reasons(array($data));
            $data = $temp[0];
        }

        $this->view->assign('data', $data);
        $this->view->assign('languages', $this->pg_language->languages);
        $this->view->assign('languages_count', count($this->pg_language->languages));
        $this->view->assign('cur_lang', $this->pg_language->current_lang_id);

        $this->Menu_model->set_menu_active_item('admin_contacts_menu', 'reasons_list_item');
        $this->view->setHeader(l('admin_header_reasons_list', 'contact_us'));
        $this->view->render('edit');
    }

    /**
     * Remove contact us reason
     *
     * @param integer $id reason identifier
     *
     * @return void
     */
    public function delete($id)
    {
        if (!empty($id)) {
            $this->Contact_us_model->delete_reason($id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_reason', 'contact_us'));
        }
        redirect(site_url() . "admin/contact_us");
    }

    /**
     * Edit module settings
     *
     * @return void
     */
    public function settings()
    {
        $data = $this->Contact_us_model->get_settings();

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "default_alert_email" => $this->input->post('default_alert_email', true),
            );

            $validate_data = $this->Contact_us_model->validate_settings($post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $this->Contact_us_model->set_settings($validate_data["data"]);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_settings_save', 'contact_us'));
                redirect(site_url() . "admin/contact_us/settings");
            }
        }

        $this->view->assign('data', $data);
        $this->Menu_model->set_menu_active_item('admin_contacts_menu', 'settings_list_item');
        $this->view->setHeader(l('admin_header_settings', 'contact_us'));
        $this->view->render('settings');
    }

}
