<?php

namespace Pg\Modules\Contact_us\Models;

/**
 * Contact us module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Contact us install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Contact us
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Contact_us_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        'user_footer_menu' => array(
            'action' => 'none',
            'name' => '',
            'items' => array(
                'footer-menu-contact-item' => array('action' => 'create', 'link' => 'contact_us', 'status' => 1, 'sorter' => 2),
            ),
        ),
        'admin_menu' => array(
            'action' => 'none',
            'name' => '',
            'items' => array(
                'settings_items' => array(
                    'action' => 'none',
                    'items' => array(
                        'system-items' => array(
                            'action' => 'none',
                            'items' => array(
                                'contactus_menu_item' => array('action' => 'create', 'link' => 'admin/contact_us', 'status' => 1, 'sorter' => 6),
                            ),
                        ),
                    ),
                ),
            ),
        ),
        'admin_contacts_menu' => array(
            'action' => 'create',
            'name' => 'Admin mode - Content - Contact us',
            'items' => array(
                'reasons_list_item' => array('action' => 'create', 'link' => 'admin/contact_us', 'status' => 1),
                'settings_list_item' => array('action' => 'create', 'link' => 'admin/contact_us/settings', 'status' => 1),
            ),
        ),
    );

    /**
     * Notification configuration
     *
     * @var array
     */
    protected $notifications = array(
        'notifications' => array(
            array('gid' => 'contact_us_form', 'send_type' => 'simple'),
        ),
        'templates' => array(
            array('gid' => 'contact_us_form', 'name' => 'Contact us form mail', 'vars' => array("user_name", "user_email", "subject", "message", "reason", "form_date"), 'content_type' => 'text'),
        ),
    );

    /**
     * Class constructor
     *
     * @return Contact_us_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
        $this->CI->load->model('Install_model');
    }

    /**
     * Install data of menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->CI->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->CI->Install_model->language_file_read('contact_us', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');

            return false;
        }
        $this->CI->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, 'update', $gid, 0, $this->menu[$gid]["items"], $gid, $langs_file);
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->CI->load->helper('menu');

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, 'export', $gid, 0, $this->menu[$gid]["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array('menu' => $return);
    }

    /**
     * Uninstall data of menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->CI->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]['items']);
            }
        }
    }

    /**
     * Install data of site map module
     *
     * @return void
     */
    public function install_site_map()
    {
        $this->CI->load->model('Site_map_model');
        $site_map_data = array(
            'module_gid' => 'contact_us',
            'model_name' => 'Contact_us_model',
            'get_urls_method' => 'get_sitemap_urls',
        );
        $this->CI->Site_map_model->set_sitemap_module('contact_us', $site_map_data);
    }

    /**
     * Uninstall data of site map module
     *
     * @return void
     */
    public function deinstall_site_map()
    {
        $this->CI->load->model('Site_map_model');
        $this->CI->Site_map_model->delete_sitemap_module('contact_us');
    }

    /**
     * Install data of banners module
     *
     * @return void
     */
    public function install_banners()
    {
        ///// add banners module
        $this->CI->load->model('Contact_us_model');
        $this->CI->load->model('banners/models/Banner_group_model');
        $this->CI->load->model('banners/models/Banner_place_model');
        $this->CI->Banner_group_model->set_module("contact_us", "Contact_us_model", "_banner_available_pages");

        $group_id = $this->CI->Banner_group_model->get_group_id_by_gid('content_groups');
        ///add pages in group
        $pages = $this->CI->Contact_us_model->_banner_available_pages();
        if ($pages) {
            foreach ($pages as $key => $value) {
                $page_attrs = array(
                    'group_id' => $group_id,
                    'name' => $value['name'],
                    'link' => $value['link'],
                );
                $this->CI->Banner_group_model->add_page($page_attrs);
            }
        }
    }

    /**
     * Uninstall data of banners module
     *
     * @return void
     */
    public function deinstall_banners()
    {
        ///// delete banners module
        $this->CI->load->model("banners/models/Banner_group_model");
        $this->CI->Banner_group_model->delete_module("contact_us");
    }

    /**
     * Install data of notifications module
     *
     * @return void
     */
    public function install_notifications()
    {
        // add notification
        $this->CI->load->model('Notifications_model');
        $this->CI->load->model('notifications/models/Templates_model');

        foreach ($this->notifications['templates'] as $tpl) {
            $template_data = array(
                'gid' => $tpl['gid'],
                'name' => $tpl['name'],
                'vars' => serialize($tpl['vars']),
                'content_type' => $tpl['content_type'],
                'date_add' => date('Y-m-d H:i:s'),
                'date_update' => date('Y-m-d H:i:s'),
            );
            $tpl_ids[$tpl['gid']] = $this->CI->Templates_model->save_template(null, $template_data);
        }

        foreach ($this->notifications['notifications'] as $notification) {
            $notification_data = array(
                'gid' => $notification['gid'],
                'send_type' => $notification['send_type'],
                'id_template_default' => $tpl_ids[$notification['gid']],
                'date_add' => date("Y-m-d H:i:s"),
                'date_update' => date("Y-m-d H:i:s"),
            );
            $this->CI->Notifications_model->save_notification(null, $notification_data);
        }
    }

    /**
     * Import languages of notifications module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_notifications_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->CI->load->model('Notifications_model');

        $langs_file = $this->CI->Install_model->language_file_read('contact_us', 'notifications', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty notifications langs data');

            return false;
        }

        $this->CI->Notifications_model->update_langs($this->notifications, $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export languages of notifications module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_notifications_lang_export($langs_ids = null)
    {
        $this->CI->load->model('Notifications_model');
        $langs = $this->CI->Notifications_model->export_langs($this->notifications, $langs_ids);

        return array('notifications' => $langs);
    }

    /**
     * Uninstall data of notifications module
     *
     * @return void
     */
    public function deinstall_notifications()
    {
        $this->CI->load->model('Notifications_model');
        $this->CI->load->model('notifications/models/Templates_model');
        foreach ($this->notifications['templates'] as $tpl) {
            $this->CI->Templates_model->delete_template_by_gid($tpl['gid']);
        }
        foreach ($this->notifications['notifications'] as $notification) {
            $this->CI->Notifications_model->delete_notification_by_gid($notification['gid']);
        }
    }

    /**
     * Install data of social_networking module
     *
     * @return void
     */
    public function install_social_networking()
    {
        ///// add social netorking page
        $this->CI->load->model('social_networking/models/Social_networking_pages_model');
        $page_data = array(
            'controller' => 'contact_us',
            'method' => 'index',
            'name' => 'Contact us page',
            'data' => 'a:3:{s:4:"like";a:3:{s:8:"facebook";s:2:"on";s:9:"vkontakte";s:2:"on";s:6:"google";s:2:"on";}s:5:"share";a:4:{s:8:"facebook";s:2:"on";s:9:"vkontakte";s:2:"on";s:8:"linkedin";s:2:"on";s:7:"twitter";s:2:"on";}s:8:"comments";s:1:"1";}',
        );
        $this->CI->Social_networking_pages_model->save_page(null, $page_data);
    }

    /**
     * Uninstall data of social_networking module
     *
     * @return void
     */
    public function deinstall_social_networking()
    {
        ///// delete social netorking page
        $this->CI->load->model('social_networking/models/Social_networking_pages_model');
        $this->CI->Social_networking_pages_model->delete_pages_by_controller('contact_us');
    }

    /**
     * Install module data
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        //// load langs
        $seo_data = array(
            'module_gid' => 'contact_us',
            'model_name' => 'Contact_us_model',
            'get_settings_method' => 'get_seo_settings',
            'get_rewrite_vars_method' => 'request_seo_rewrite',
            'get_sitemap_urls_method' => 'get_sitemap_xml_urls',
        );
        $this->CI->pg_seo->set_seo_module('contact_us', $seo_data);

        // add seo link
        $this->CI->load->model('Seo_model');

        $xml_data = $this->CI->Seo_model->get_xml_route_file_content();

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:contact-us]',
        );
        $this->CI->pg_seo->set_settings('user', 'contact_us', 'index', $data);

        $xml_data['contact_us']['index'] = $this->CI->pg_seo->url_template_transform('contact_us', 'index', $data["url_template"], 'base', 'xml');

        $this->CI->Seo_model->set_xml_route_file_content($xml_data);
        $this->CI->Seo_model->rewrite_route_php_file();
    }

    /**
     * Import module languages
     *
     * @param array $langs_ids array languages identifiers
     *
     * @return void
     */
    public function _arbitrary_lang_install($langs_ids = null)
    {
        $langs_file = $this->CI->Install_model->language_file_read("contact_us", "arbitrary", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty contact_us arbitrary langs data");

            return false;
        }

        $post_data = array(
            "title" => $langs_file["seo_tags_index_title"],
            "keyword" => $langs_file["seo_tags_index_keyword"],
            "description" => $langs_file["seo_tags_index_description"],
            "header" => $langs_file["seo_tags_index_header"],
            "og_title" => $langs_file["seo_tags_index_og_title"],
            "og_type" => $langs_file["seo_tags_index_og_type"],
            "og_description" => $langs_file["seo_tags_index_og_description"],
        );
        $this->CI->pg_seo->set_settings("user", "contact_us", "index", $post_data);
    }

    /**
     * Export module languages
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        //// arbitrary
        $settings = $this->CI->pg_seo->get_settings("user", "contact_us", "index", $langs_ids);
        $arbitrary_return["seo_tags_index_title"] = $settings["title"];
        $arbitrary_return["seo_tags_index_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_index_description"] = $settings["description"];
        $arbitrary_return["seo_tags_index_header"] = $settings["header"];
        $arbitrary_return["seo_tags_index_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_index_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_index_og_description"] = $settings["og_description"];

        return array("arbitrary" => $arbitrary_return);
    }

    /**
     * Uninstall module data
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        // remove seo link
        $this->CI->load->model('Seo_model');

        $xml_data = $this->CI->Seo_model->get_xml_route_file_content();

        $data = array(
            'noindex' => 0,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->CI->pg_seo->set_settings('user', 'contact_us', 'index', $data);

        $xml_data['contact_us']['index'] = $this->CI->pg_seo->url_template_transform('contact_us', 'index', $data["url_template"], 'base', 'xml');

        $this->CI->Seo_model->set_xml_route_file_content($xml_data);
        $this->CI->Seo_model->rewrite_route_php_file();

        $this->CI->pg_seo->delete_seo_module('contact_us');
    }

}
