<?php

namespace Pg\Modules\Countries\Controllers;

/**
 * Countries module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Countries admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Countries
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Countries extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Countries
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Countries_model');
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'content_items');
    }

    /**
     * Manage countries
     *
     * @param integer $sort_mode sorting mode
     *
     * @return void
     */
    public function index($sort_mode = 0)
    {
        if ($sort_mode == 0) {
            $order_for_not_sorted = array(
                "priority" => "ASC",
                "lang_" . $this->pg_language->current_lang_id => "ASC",
            );
            $params_for_not_sorted = array();
        } else {
            $order_for_not_sorted = array(
                "lang_" . $this->pg_language->current_lang_id => "ASC",
            );
            $order_for_sorted = array(
                "priority" => "ASC",
                "lang_" . $this->pg_language->current_lang_id => "ASC",
            );

            $params_for_sorted["where"]["sorted !="] = 0;
            $params_for_not_sorted["where"]["sorted ="] = 0;

            $sorted_countries_list = $this->Countries_model->get_countries(
                    $order_for_sorted, $params_for_sorted, array(), $this->pg_language->current_lang_id
            );

            $this->view->assign('sorted', $sorted_countries_list);
        }
        $installed_countries_list = $this->Countries_model->get_countries(
                $order_for_not_sorted, $params_for_not_sorted, array(), $this->pg_language->current_lang_id
        );

        $this->view->assign('installed', $installed_countries_list);

        $this->view->assign('sort_mode', $sort_mode);

        $this->Menu_model->set_menu_active_item('admin_countries_menu', 'countries_list_item');
        $this->view->setHeader(l('admin_header_countries_list', 'countries'));
        $this->view->render('list');
    }

    /**
     * Manage country regions
     *
     * @param string  $country_code country code
     * @param integer $sort_mode    sorting mode
     *
     * @return void
     */
    public function country($country_code, $sort_mode = 0)
    {
        if (!$country_code) {
            $this->view->setRedirect(site_url() . 'admin/countries');
        }

        if (!empty($_SESSION["cities_list"]) && $_SESSION["cities_list"]['country'] != $country_code) {
            unset($_SESSION["districts_list"]);
        }

        $current_settings["country"] = $country_code;
        $_SESSION["cities_list"] = $current_settings;

        $country = $this->Countries_model->get_country($country_code, $this->pg_language->current_lang_id);
        $this->view->assign('country', $country);

        if ($sort_mode == 0) {
            $order_for_not_sorted = array(
                "priority" => "ASC",
                "lang_" . $this->pg_language->current_lang_id => "ASC",
            );
            $params_for_not_sorted = array();
        } else {
            $order_for_not_sorted = array(
                "lang_" . $this->pg_language->current_lang_id => "ASC",
            );
            $order_for_sorted = array(
                "priority" => "ASC",
                "lang_" . $this->pg_language->current_lang_id => "ASC",
            );

            $params_for_sorted["where"]["sorted !="] = 0;
            $params_for_not_sorted["where"]["sorted ="] = 0;

            $sorted_regions_list = $this->Countries_model->get_regions(
                    $country_code, $order_for_sorted, $params_for_sorted, array(), $this->pg_language->current_lang_id
            );

            $this->view->assign('sorted', $sorted_regions_list);
        }

        $installed_regions_list = $this->Countries_model->get_regions(
                $country_code, $order_for_not_sorted, $params_for_not_sorted, array(),
                $this->pg_language->current_lang_id
        );

        $this->view->assign('installed', $installed_regions_list);

        $this->view->assign('sort_mode', $sort_mode);

        $this->Menu_model->set_menu_active_item('admin_countries_menu', 'countries_list_item');
        $this->view->setBackLink(site_url() . "admin/countries");
        $this->view->setHeader(l('admin_header_regions_list', 'countries') . $country["name"]);
        $this->view->render('list_regions');
    }

    /**
     * Manage region cities
     *
     * @param string  $country_code country code
     * @param integer $id_region    region identifier
     * @param integer $sort_mode    sorting mode
     * @param integer $page         page of results
     *
     * @return void
     */
    public function region($country_code, $id_region, $sort_mode = 0, $page = 1)
    {
        if (!$country_code) {
            $this->view->setRedirect(site_url() . 'admin/countries');
        }
        if (!$id_region) {
            $this->view->setRedirect(site_url() . 'admin/countries/country/' . $country_code);
        }

        $country = $this->Countries_model->get_country($country_code, $this->pg_language->current_lang_id);
        $this->view->assign('country', $country);

        $region = $this->Countries_model->get_region($id_region, $this->pg_language->current_lang_id);
        $this->view->assign('region', $region);

        $current_settings = isset($_SESSION["cities_list"]) ? $_SESSION["cities_list"] : array();
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }
        if (!isset($current_settings["search"])) {
            $current_settings["search"] = "";
        }

        $params["where"]["id_region"] = $id_region;

        if (isset($_POST["search"])) {
            $current_settings["search"] = trim(strip_tags($this->input->post("search", true)));
        }
        if (!empty($current_settings["search"])) {
            $params["where"]["name LIKE"] = $current_settings["search"] . "%";
            $this->view->assign('search', $current_settings["search"]);
        }

        $cities_count = $this->Countries_model->get_cities_count($params);

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $cities_count, $items_on_page);
        $current_settings["page"] = $page;
        $current_settings["id_region"] = $id_region;
        $current_settings["country"] = $country_code;

        $_SESSION["cities_list"] = $current_settings;

        if ($sort_mode == 0) {
            $order_for_not_sorted = array(
                "priority" => "ASC",
                "lang_" . $this->pg_language->current_lang_id => "ASC",
            );
        } else {
            $page = 1;
            $items_on_page = null;

            $order_for_not_sorted = array(
                "lang_" . $this->pg_language->current_lang_id => "ASC",
            );
            $order_for_sorted = array(
                "priority" => "ASC",
                "lang_" . $this->pg_language->current_lang_id => "ASC",
            );

            $params_for_sorted["where"]["sorted !="] = 0;
            $params["where"]["sorted ="] = 0;

            $sorted_cities_list = $this->Countries_model->get_cities(
                    $page, $items_on_page, $order_for_sorted, $params_for_sorted, array(),
                    $this->pg_language->current_lang_id
            );

            $this->view->assign('sorted', $sorted_cities_list);
        }

        if ($cities_count) {
            $cities_list = $this->Countries_model->get_cities(
                    $page, $items_on_page, $order_for_not_sorted, $params, array(), $this->pg_language->current_lang_id
            );
            $this->view->assign('installed', $cities_list);
        }

        $this->load->helper("navigation");
        $url = site_url() . "admin/countries/region/" . $country_code . "/" . $id_region . "/0/";
        $page_data = get_admin_pages_data($url, $cities_count, $items_on_page, $page, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->view->assign('page', $page);

        $this->view->assign('sort_mode', $sort_mode);

        $use_district = $this->pg_module->get_module_config('countries', 'use_district');
        $this->view->assign('use_district', $use_district);

        $this->Menu_model->set_menu_active_item('admin_countries_menu', 'countries_list_item');
        $this->view->setBackLink(site_url() . "admin/countries/country/" . $country_code);
        $this->view->setHeader(l('admin_header_cities_list', 'countries') . $region['name']);
        $this->view->render('list_cities');
    }

    /**
     * Edit country data
     *
     * @param string $country_code country code
     *
     * @return void
     */
    public function country_edit($country_code = "")
    {
        if ($country_code) {
            $data = $this->Countries_model->get_country(
                    $country_code, $this->pg_language->current_lang_id, $this->pg_language->languages
            );
        } else {
            $data = array();
        }
        foreach ($this->pg_language->languages as $lang_id => $lang_data) {
            $validate_lang[$lang_id] = $data['lang_' . $lang_id];
        }

        if (!empty($validate_lang)) {
            $this->view->assign('validate_lang', $validate_lang);
        }

        $this->view->assign('languages', $this->pg_language->languages);
        $this->view->assign('languages_count', count($this->pg_language->languages));
        $this->view->assign('cur_lang', $this->pg_language->current_lang_id);

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "code" => $this->input->post('code', true),
                "name" => $this->input->post('name', true),
            );

            $langs = $this->input->post("langs", true);
            if ($post_data['name'] == '') {
                $post_data['name'] = $langs[$this->pg_language->current_lang_id];
            }

            $validate_data = $this->Countries_model->validate('country', $country_code, $post_data, $langs);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $save_data = $validate_data["data"];
                $update_type = !empty($country_code) ? "edit" : "add";
                if ($update_type == "add") {
                    $save_data["priority"] = $this->Countries_model->get_country_max_priority() + 1;
                }
                $this->Countries_model->save_country(
                        $country_code, $save_data, $update_type, $validate_data['langs']
                );

                if ($update_type == "edit") {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_country', 'countries'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_country', 'countries'));
                }

                $url = site_url() . "admin/countries";
                $this->view->setRedirect($url);
            }
            $data = array_merge($data, $validate_data["data"]);
        }

        $this->view->assign('data', $data);

        $this->view->setHeader(l('admin_header_country_edit', 'countries'));
        $this->view->render('edit_country_form');
    }

    /**
     * Edit region data
     *
     * @param string  $country_code country code
     * @param integer $id_region    region identifier
     *
     * @return void
     */
    public function region_edit($country_code, $id_region = '')
    {
        if (!$country_code) {
            $this->view->setRedirect(site_url() . 'admin/countries');
        } else {
            $country_data = $this->Countries_model->get_country($country_code, $this->pg_language->current_lang_id);
            if (empty($country_data)) {
                $this->view->setRedirect(site_url() . 'admin/countries');
            }
        }

        if ($id_region) {
            $data = $this->Countries_model->get_region(
                    $id_region, $this->pg_language->current_lang_id, $this->pg_language->languages
            );
        } else {
            $data = array();
        }

        foreach ($this->pg_language->languages as $lang_id => $lang_data) {
            $validate_lang[$lang_id] = $data['lang_' . $lang_id];
        }

        if (!empty($validate_lang)) {
            $this->view->assign('validate_lang', $validate_lang);
        }

        $this->view->assign('languages', $this->pg_language->languages);
        $this->view->assign('languages_count', count($this->pg_language->languages));
        $this->view->assign('cur_lang', $this->pg_language->current_lang_id);

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "country_code" => $country_code,
                "name" => $this->input->post('name', true),
            );

            $langs = $this->input->post("langs", true);
            if ($post_data['name'] == '') {
                $post_data['name'] = $langs[$this->pg_language->current_lang_id];
            }

            $validate_data = $this->Countries_model->validate('region', $id_region, $post_data, $langs);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $save_data = $validate_data["data"];
                if (empty($id_region)) {
                    $save_data["priority"] = $this->Countries_model->get_region_max_priority($country_code) + 1;
                }
                $this->Countries_model->save_region($id_region, $save_data, $validate_data['langs']);

                if ($id_region) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_region', 'countries'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_region', 'countries'));
                }

                $url = site_url() . "admin/countries/country/" . $country_code;
                $this->view->setRedirect($url);
            }
            $data = array_merge($data, $validate_data["data"]);
        }

        $this->view->assign('data', $data);
        $this->view->assign('country', $country_data);

        $this->view->setHeader(l('admin_header_region_edit', 'countries'));
        $this->view->render('edit_region_form');
    }

    /**
     * Edit city data
     *
     * @param string  $country_code country code
     * @param integer $id_region    region identifier
     * @param integer $id_city      city identifier
     *
     * @return void
     */
    public function city_edit($country_code, $id_region, $id_city = '')
    {
        if (!$country_code) {
            $this->view->setRedirect(site_url() . 'admin/countries');
        } else {
            $country_data = $this->Countries_model->get_country($country_code, $this->pg_language->current_lang_id);
            if (empty($country_data)) {
                $this->view->setRedirect(site_url() . 'admin/countries');
            }
        }

        if (!$id_region) {
            $this->view->setRedirect(site_url() . 'admin/countries/country/' . $country_code);
        } else {
            $region_data = $this->Countries_model->get_region($id_region, $this->pg_language->current_lang_id);
            if (empty($region_data)) {
                $this->view->setRedirect(site_url() . 'admin/countries/country/' . $country_code);
            }
        }

        if ($id_city) {
            $data = $this->Countries_model->get_city(
                    $id_city, $this->pg_language->current_lang_id, $this->pg_language->languages
            );
        } else {
            $data = array();
        }

        foreach ($this->pg_language->languages as $lang_id => $lang_data) {
            $validate_lang[$lang_id] = $data['lang_' . $lang_id];
        }

        if (!empty($validate_lang)) {
            $this->view->assign('validate_lang', $validate_lang);
        }

        $this->view->assign('languages', $this->pg_language->languages);
        $this->view->assign('languages_count', count($this->pg_language->languages));
        $this->view->assign('cur_lang', $this->pg_language->current_lang_id);

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "country_code" => $country_code,
                "id_region" => $id_region,
                "latitude" => $this->input->post('latitude', true),
                "longitude" => $this->input->post('longitude', true),
                "name" => $this->input->post('name', true),
            );

            $langs = $this->input->post("langs", true);
            if ($post_data['name'] == '') {
                $post_data['name'] = $langs[$this->pg_language->current_lang_id];
            }

            if (!empty($region_data["code"])) {
                $post_data["region_code"] = $region_data["code"];
            }

            $validate_data = $this->Countries_model->validate('city', $id_city, $post_data, $langs);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $save_data = $validate_data["data"];
                $this->Countries_model->save_city($id_city, $save_data, $validate_data['langs']);

                if ($id_city) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_city', 'countries'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_city', 'countries'));
                }

                $url = site_url() . "admin/countries/region/" . $country_code . "/" . $id_region;
                $this->view->setRedirect($url);
            }
            $data = array_merge($data, $validate_data["data"]);
        }

        $this->view->assign('data', $data);
        $this->view->assign('country', $country_data);
        $this->view->assign('region', $region_data);

        $this->view->setHeader(l('admin_header_city_edit', 'countries'));
        $this->view->render('edit_city_form');
    }

    /**
     * Remove country data
     *
     * @param string $country_code country code
     *
     * @return void
     */
    public function country_delete($country_code)
    {
        if (!empty($country_code)) {
            $this->Countries_model->delete_country($country_code);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_country', 'countries'));
        }
        $url = site_url() . "admin/countries";
        $this->view->setRedirect($url);
    }

    /**
     * Remove region data
     *
     * @param string  $country_code country code
     * @param integer $id_region    region identifier
     *
     * @return void
     */
    public function region_delete($country_code, $id_region)
    {
        if (!empty($id_region)) {
            $this->Countries_model->delete_region($id_region);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_region', 'countries'));
        }
        $url = site_url() . "admin/countries/country/" . $country_code;
        $this->view->setRedirect($url);
    }

    /**
     * Remove city data
     *
     * @param string  $country_code country code
     * @param integer $id_region    region identifier
     * @param integer $id_city      city identifier
     *
     * @return void
     */
    public function city_delete($country_code, $id_region, $id_city)
    {
        if (!empty($id_city)) {
            $this->Countries_model->delete_city($id_city);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_city', 'countries'));
        }
        $cur_set = $_SESSION["cities_list"];
        $url = site_url() . "admin/countries/region/" . $country_code . "/" . $id_region . "/0/" . $cur_set["page"];
        $this->view->setRedirect($url);
    }

    /**
     * Install location data
     *
     * Available steps: country, region, city
     *
     * @param string $step         install step
     * @param string $country_code country code
     *
     * @return void
     */
    public function install($step = 'country', $country_code = '')
    {
        @set_time_limit(0);

        $this->Menu_model->set_menu_active_item('admin_countries_menu', 'countries_install_item');

        switch ($step) {
            case 'country': {
                    $filter = $country_code ? $country_code : 'all';
                    $countries_list = $this->Countries_model->get_cache_countries();
                    $installed_countries_list = $this->Countries_model->get_countries(array("name" => "ASC"));

                    $filter_data["all"] = count($countries_list);
                    $filter_data["installed"] = count($installed_countries_list);
                    $filter_data["not_installed"] = count($countries_list) - count($installed_countries_list);

                    switch ($filter) {
                        case "installed":
                            foreach ($countries_list as $k => $country) {
                                if (!isset($installed_countries_list[$country["code"]])) {
                                    unset($countries_list[$k]);
                                }
                            }
                            break;
                        case "not-installed":
                            foreach ($countries_list as $k => $country) {
                                if (isset($installed_countries_list[$country["code"]])) {
                                    unset($countries_list[$k]);
                                }
                            }
                            break;
                        case "all":
                        default:
                            break;
                    }

                    $this->view->assign('list', $countries_list);
                    $this->view->assign('installed', $installed_countries_list);
                    $this->view->assign('filter', $filter);
                    $this->view->assign('filter_data', $filter_data);

                    $this->pg_theme->add_js('admin-countries-selected.js', 'countries');
                    $this->view->setHeader(l('admin_header_install_countries_list', 'countries'));
                    $this->view->render('install_country_list');
                    break;
                }

            case 'region': {
                    if (!$country_code) {
                        $this->view->setRedirect(site_url() . 'admin/countries/install');
                    }
                    $country = $this->Countries_model->get_cache_country_by_code($country_code);
                    if (!empty($country)) {
                        $regions_list = $this->Countries_model->get_cache_regions($country_code);
                        $installed_regions_list = $this->Countries_model->get_regions_by_code($country_code,
                                                                                              array("code" => "ASC"));
                        $this->view->assign('country', $country);
                        $this->view->assign('list', $regions_list);
                        $this->view->assign('installed', $installed_regions_list);
                    }

                    $this->view->setBackLink(site_url() . "admin/countries/install");
                    $this->view->setHeader(l('admin_header_install_regions_list', 'countries'));
                    $this->view->render('install_region_list');
                    break;
                }

            case 'city': {
                    if (!$country_code) {
                        $this->view->setRedirect(site_url() . 'admin/countries/install');
                    }

                    $country = $this->Countries_model->get_cache_country_by_code($country_code);
                    if (empty($country)) {
                        $this->view->setRedirect(site_url() . 'admin/countries/install');
                    }

                    $regions = $this->input->post('region', true);
                    if (empty($regions)) {
                        $this->view->setRedirect(site_url() . 'admin/countries/install/region/' . $country_code);
                    }

                    $regions_list = $this->Countries_model->get_cache_regions($country_code);
                    foreach ($regions_list as $key => $region) {
                        if (!in_array($region["code"], $regions)) {
                            unset($regions_list[$key]);
                        }
                    }

                    $this->view->assign('country', $country);
                    $this->view->assign('regions', $regions);
                    $this->view->assign('regions_list', $regions_list);
                    $this->view->assign('back_link', site_url() . "admin/countries/install/region/" . $country["code"]);

                    $this->pg_theme->add_js('admin-countries.js', 'countries');
                    $this->view->setHeader(l('admin_header_install_cities_list', 'countries'));
                    $this->view->render('install_city_list');
                    break;
                }

            case 'selected_countries': {
                    $countries = $this->input->post('countries', true);
                    if (empty($countries)) {
                        $this->view->setRedirect(site_url() . 'admin/countries/install/');
                    }

                    $list_regions = array();
                    $full_list = array();
                    $countries_list = $this->Countries_model->get_cache_countries();
                    foreach ($countries_list as $c_key => $country) {
                        if (!in_array($country["code"], $countries)) {
                            unset($countries_list[$c_key]);
                        } else {
                            $regions_list[$c_key] = $this->Countries_model->get_cache_regions($country['code']);
                            foreach ($regions_list[$c_key] as $key => $region) {
                                $list_regions[$key] = $region['code'];
                            }
                            $full_list[$country['code']] = $list_regions;
                            $list_regions = array();
                        }
                    }
                    $this->view->assign('countries_list_json', json_encode(array_keys($full_list)));
                    $this->view->assign('full_list', json_encode($full_list));
                    $this->view->assign('countries_list', $countries_list);
                    $this->view->assign('regions_list', $regions_list);
                    $this->view->assign('back_link', site_url() . "admin/countries/install/");

                    $this->pg_theme->add_js('admin-countries-selected.js', 'countries');
                    $this->view->setHeader(l('admin_header_install_countries_list', 'countries'));
                    $this->view->render('install_countries_list');
                    break;
                }
        }
    }

    /**
     * Install cities data
     *
     * @param string $country_code country code
     * @param string $region_code  region code
     *
     * @return void
     */
    public function ajax_install_cities($country_code, $region_code)
    {
        @set_time_limit(0);
        $this->view->output(
                $this->Countries_model->install_cities($country_code, $region_code, $this->pg_language->languages));

        $this->view->render();
    }

    public function ajax_install_country($country_code)
    {
        @set_time_limit(0);
        $this->view->output($this->Countries_model->install_country($country_code));
        $this->view->render();
    }

    /**
     * Resort countries
     *
     * @return boolean
     */
    public function ajax_save_country_sorter()
    {
        $item_data = $this->input->post('sorter');
        $item_data = json_decode($item_data, true);
        $item_data = $item_data["parent_0"];
        $countries_with_priority = array();

        if (!empty($item_data)) {
            foreach ($item_data as $key => $sorter) {
                $country_code = strtoupper(substr(str_replace("item_", "", $key), 0, 2));
                $countries_with_priority[] = $country_code;
                if (empty($country_code)) {
                    continue;
                }
                $this->Countries_model->set_country_priority($country_code, $sorter);
            }
        }

        $this->Countries_model->set_default_priority_country($countries_with_priority);

        return true;
    }

    /**
     * Resort regions data
     *
     * @param string $country_code
     *
     * @return boolean
     */
    public function ajax_save_region_sorter($country_code)
    {
        $item_data = $this->input->post('sorter');
        $item_data = json_decode($item_data, true);
        $item_data = $item_data["parent_0"];
        $regions_with_priority = array();

        if (!empty($item_data)) {
            foreach ($item_data as $key => $sorter) {
                $id_region = intval(str_replace("item_", "", $key));
                $regions_with_priority[] = $id_region;
                if (empty($id_region)) {
                    continue;
                }
                $this->Countries_model->set_region_priority($id_region, $sorter);
            }
        }

        $this->Countries_model->set_default_priority_region($country_code, $regions_with_priority);

        return true;
    }

    /**
     * Resort cities data
     *
     * @param string  $country_code country code
     * @param integer $region_id    region identifier
     * @param integer $page         page of results
     *
     * @return boolean
     */
    public function ajax_save_city_sorter($country_code, $region_id, $page)
    {
        $item_data = $this->input->post('sorter');
        $item_data = json_decode($item_data, true);
        $item_data = $item_data['parent_0'];

        $cities_with_priority = array();

        if (!empty($item_data)) {
            foreach ($item_data as $key => $sorter) {
                $city_id = intval(str_replace('item_', '', $key));
                $cities_with_priority[] = $city_id;
                if (empty($city_id)) {
                    continue;
                }
                $this->Countries_model->set_city_priority($city_id, $sorter);
            }
        }

        $this->Countries_model->set_default_priority_city($country_code, $region_id, $cities_with_priority);

        return true;
    }

    /**
     * Manage districts
     *
     * @param string  $country_code country code
     * @param integer $region_id    region identifier
     * @param integer $city_id      city identifier
     * @param integer $sort_mode    sorting mode
     * @param integer $page         page of results
     *
     * @return void
     */
    public function city($country_code, $id_region, $city_id, $sort_mode = 0, $page = 1)
    {
        if (!$country_code) {
            $this->view->setRedirect(site_url() . 'admin/countries');
        }
        if (!$id_region) {
            $this->view->setRedirect(site_url() . 'admin/countries/country/' . $country_code);
        }

        if (!$city_id) {
            $this->view->setRedirect(site_url() . 'admin/countries/country/' . $country_code . '/' . $region_id);
        }

        $country = $this->Countries_model->get_country($country_code, $this->pg_language->current_lang_id);
        $this->view->assign('country', $country);

        $region = $this->Countries_model->get_region($id_region, $this->pg_language->current_lang_id);
        $this->view->assign('region', $region);

        $city = $this->Countries_model->get_city($city_id, $this->pg_language->current_lang_id);
        $this->view->assign('city', $city);

        $current_settings = isset($_SESSION["districts_list"]) ? $_SESSION["districts_list"] : array();
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }
        if (!isset($current_settings["search"])) {
            $current_settings["search"] = "";
        }

        $params["where"]["id_city"] = $city_id;

        if (isset($_POST["search"])) {
            $current_settings["search"] = trim(strip_tags($this->input->post("search", true)));
        }
        if (!empty($current_settings["search"])) {
            $params["where"]["name LIKE"] = $current_settings["search"] . "%";
            $this->view->assign('search', $current_settings["search"]);
        }

        $districts_count = $this->Countries_model->get_districts_count($params);

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $districts_count, $items_on_page);
        $current_settings["page"] = $page;
        $current_settings["id_region"] = $id_region;
        $current_settings["country"] = $country_code;
        $current_settings["city_id"] = $city_id;

        $_SESSION["districts_list"] = $current_settings;

        if ($sort_mode == 0) {
            $order_for_not_sorted = array("priority" => "ASC", "lang_" . $this->pg_language->current_lang_id => "ASC");
        } else {
            $page = 1;
            $items_on_page = null;

            $order_for_not_sorted = array("lang_" . $this->pg_language->current_lang_id => "ASC");
            $order_for_sorted = array("priority" => "ASC", "lang_" . $this->pg_language->current_lang_id => "ASC");

            $params_for_sorted["where"]["sorted !="] = 0;
            $params["where"]["sorted ="] = 0;

            $sorted_cities_list = $this->Countries_model->get_cities($page, $items_on_page, $order_for_sorted,
                                                                     $params_for_sorted, array(),
                                                                     $this->pg_language->current_lang_id);

            $this->view->assign('sorted', $sorted_cities_list);
        }

        if ($districts_count) {
            $districts_list = $this->Countries_model->get_districts($page, $items_on_page, $order_for_not_sorted,
                                                                    $params, array(),
                                                                    $this->pg_language->current_lang_id);
            $this->view->assign('districts', $districts_list);
        }

        $this->load->helper('navigation');
        $url = site_url() . 'admin/countries/city/' . $country_code . '/' . $id_region . '/' . $city_id . '/0/';
        $page_data = get_admin_pages_data($url, $districts_count, $items_on_page, $page, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->view->assign('page', $page);

        $this->view->assign('sort_mode', $sort_mode);

        $this->Menu_model->set_menu_active_item('admin_countries_menu', 'countries_list_item');
        $this->view->setBackLink(site_url() . "admin/countries/region/" . $country_code . "/" . $id_region);
        $this->view->setHeader(l('admin_header_districts_list', 'countries') . $city['name']);
        $this->view->render('list_districts');
    }

    /**
     * Edit district data
     *
     * @param string  $country_code country code
     * @param integer $region_id    region identifier
     * @param integer $city_id      city identifier
     * @param integer $district_id  district identifier
     *
     * @return void
     */
    public function district_edit($country_code, $id_region, $city_id, $district_id = '')
    {
        if (!$country_code) {
            $this->view->setRedirect(site_url() . 'admin/countries');
        } else {
            $country_data = $this->Countries_model->get_country($country_code, $this->pg_language->current_lang_id);
            if (empty($country_data)) {
                $this->view->setRedirect(site_url() . 'admin/countries');
            }
        }

        if (!$id_region) {
            $this->view->setRedirect(site_url() . 'admin/countries/country/' . $country_code);
        } else {
            $region_data = $this->Countries_model->get_region($id_region, $this->pg_language->current_lang_id);
            if (empty($region_data)) {
                $this->view->setRedirect(site_url() . 'admin/countries/country/' . $country_code);
            }
        }

        if (!$city_id) {
            $this->view->setRedirect(site_url() . 'admin/countries/region/' . $country_code . '/' . $id_region);
        } else {
            $city_data = $this->Countries_model->get_city($city_id, $this->pg_language->current_lang_id);
            if (empty($city_data)) {
                $this->view->setRedirect(site_url() . 'admin/countries/region/' . $country_code . '/' . $id_region);
            }
        }

        if ($district_id) {
            $data = $this->Countries_model->get_district($district_id, $this->pg_language->current_lang_id,
                                                         $this->pg_language->languages);
        } else {
            $data = array();
        }

        foreach ($this->pg_language->languages as $lang_id => $lang_data) {
            if (!empty($data['lang_' . $lang_id])) {
                $validate_lang[$lang_id] = $data['lang_' . $lang_id];
            } else {
                $validate_lang[$lang_id] = '';
            }
        }

        if (!empty($validate_lang)) {
            $this->view->assign('validate_lang', $validate_lang);
        }

        $this->view->assign('languages', $this->pg_language->languages);
        $this->view->assign('languages_count', count($this->pg_language->languages));
        $this->view->assign('cur_lang', $this->pg_language->current_lang_id);

        if ($this->input->post('btn_save')) {
            $post_data = array(
                'country_code' => $country_code,
                'id_region' => $id_region,
                'id_city' => $city_id,
                'latitude' => $this->input->post('latitude', true),
                'longitude' => $this->input->post('longitude', true),
                'name' => $this->input->post('name', true),
            );

            $langs = $this->input->post("langs", true);
            if ($post_data['name'] == '') {
                $post_data['name'] = $langs[$this->pg_language->current_lang_id];
            }

            if (!empty($region_data["code"])) {
                $post_data["region_code"] = $region_data["code"];
            }

            $validate_data = $this->Countries_model->validate('district', $district_id, $post_data, $langs);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $save_data = $validate_data["data"];
                $this->Countries_model->save_district($district_id, $save_data, $validate_data['langs']);

                if ($district_id) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_district_updated', 'countries'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_district_added', 'countries'));
                }

                $url = site_url() . 'admin/countries/city/' . $country_code . '/' . $id_region . '/' . $city_id;
                $this->view->setRedirect($url);
            }
            $data = array_merge($data, $validate_data["data"]);
        }

        $this->view->assign('data', $data);
        $this->view->assign('country', $country_data);
        $this->view->assign('region', $region_data);
        $this->view->assign('city', $city_data);

        $this->view->setHeader(l('admin_header_district_edit', 'countries'));
        $this->view->render('edit_district_form');
    }

    /**
     * Remove district data
     *
     * @param string  $country_code country code
     * @param integer $region_id    region identifier
     * @param integer $city_id      city identifier
     * @param integer $district_id  district identifier
     *
     * @return void
     */
    public function district_delete($country_code, $id_region, $city_id, $district_id)
    {
        if (!empty($district_id)) {
            $this->Countries_model->delete_district($district_id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_district_deleted', 'countries'));
        }
        $cur_set = $_SESSION["districts_list"];
        $url = site_url() . "admin/countries/city/$country_code/$id_region/$city_id/0/" . $cur_set["page"];
        $this->view->setRedirect($url);
    }

    /**
     * Settings page
     */
    public function settings()
    {
        $new_cfg = $this->input->post('settings');
        if(!empty($new_cfg)) {
            $this->Countries_model->setModuleConfig($new_cfg);
        }
        $cfg = $this->Countries_model->getModuleConfig();
        $this->view->setHeader(l('admin_header_settings_edit', 'countries'));
        $this->view->assign('countries_config', $cfg);
        
        $this->view->render('settings');
    }

    /**
     * Resort districts data
     *
     * @param string  $country_code country code
     * @param integer $region_id    region identifier
     * @param integer $city_id      city identifier
     * @param integer $page         page of results
     *
     * @return boolean
     */
    public function ajax_save_district_sorter($country_code, $region_id, $city_id, $page = 1)
    {
        $item_data = $this->input->post('sorter');
        $item_data = json_decode($item_data, true);
        $item_data = $item_data['parent_0'];

        $districts_with_priority = array();

        if (!empty($item_data)) {
            foreach ($item_data as $key => $sorter) {
                $district_id = intval(str_replace('item_', '', $key));
                $districts_with_priority = $district_id;
                if (empty($district_id)) {
                    continue;
                }
                $this->Countries_model->set_district_priority($district_id, $sorter);
            }
        }

        $this->Countries_model->set_default_priority_district(
                $country_code, $region_id, $city_id, $districts_with_priority
        );

        return true;
    }

    /**
     * Get location form by ajax
     *
     * Location type: region, city, district
     *
     * @param string $type location type
     * @param string $var  location identifier/code
     *
     * @return void
     */
    public function ajax_get_form($type, $var = 0)
    {
        $data = array();

        switch ($type) {
            case "region":
                $data["country"] = $this->Countries_model->get_country($var, $this->pg_language->current_lang_id);
                break;
            case "city":
                $data["region"] = $this->Countries_model->get_region($var, $this->pg_language->current_lang_id);
                $data["country"] = $this->Countries_model->get_country($data["region"]["country_code"],
                                                                       $this->pg_language->current_lang_id);
                break;
            case "district":
                $data['city'] = $this->Countries_model->get_city($var, $this->pg_language->current_lang_id);
                $data['region'] = $this->Countries_model->get_region($data['city']['id_region'],
                                                                     $this->pg_language->current_lang_id);
                $data['country'] = $this->Countries_model->get_country($data['region']['country_code'],
                                                                       $this->pg_language->current_lang_id);
                break;
        }

        $this->view->assign("data", $data);
        $this->view->assign("type", $type);
        $this->view->render('ajax_country_form');
    }

}
