<?php

namespace Pg\Modules\Countries\Models;

/**
 * Countries module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Countries install model
 *
 * @package PG_RealEstate
 * @subpackage Countries
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Countries_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        'admin_menu' => array(
            'action' => 'none',
            'name' => '',
            'items' => array(
                'settings_items' => array(
                    'action' => 'none',
                    'items' => array(
                        'content_items' => array(
                            'action' => 'none',
                            'items' => array(
                                'countries_menu_item' => array(
                                    'action' => 'create', 'link' => 'admin/countries', 'status' => 1, 'sorter' => 1,
                                ),
                            ),
                        ),
                    ),
                ),
            ),
        ),
        'admin_countries_menu' => array(
            'action' => 'create',
            'name' => 'Admin mode - Content - Countries',
            'items' => array(
                'countries_list_item' => array(
                    'action' => 'create', 'link' => 'admin/countries', 'status' => 1,
                ),
                'countries_install_item' => array(
                    'action' => 'create', 'link' => 'admin/countries/install', 'status' => 1,
                ),
                'countries_settings_item' => array(
                    'action' => 'create', 'link' => 'admin/countries/settings', 'status' => 1,
                ),
            ),
        ),
    );

    /**
     * Constructor
     *
     * @return Countries_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        // load langs
        $this->ci->load->model('Install_model');
    }

    /**
     * Install data of menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('countries', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');
            return false;
        }

        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items(
                    $this->menu, 'update', $gid, 0, $this->menu[$gid]["items"], $gid, $langs_file
            );
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items(
                    $this->menu, 'export', $gid, 0, $this->menu[$gid]["items"], $gid, $langs_ids
            );
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall data of menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data['action'] === 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]['items']);
            }
        }
    }

    /**
     * Install module data
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        // add entries for lang data updates
        $lang_dm_data = array(
            'module' => 'countries',
            'model' => 'Countries_model',
            'method_add' => 'lang_dedicate_module_callback_add',
            'method_delete' => 'lang_dedicate_module_callback_delete',
        );
        $this->ci->pg_language->add_dedicate_modules_entry($lang_dm_data);

        $this->ci->load->model('Countries_model');

        foreach ($this->ci->pg_language->languages as $lang) {
            $this->ci->Countries_model->lang_dedicate_module_callback_add($lang['id']);
        }
        return;
    }

    /**
     * Uninstall module data
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        // delete entries in dedicate modules
        $lang_dm_data['where'] = array(
            'module' => 'countries',
            'model' => 'Countries_model',
        );
        $this->ci->pg_language->delete_dedicate_modules_entry($lang_dm_data);
    }

}
