<?php

namespace Pg\Modules\Currencies\Models;

/**
 * Currencies module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * XE currency rates model (from xe.com)
 *
 * @package 	PG_RealEstate
 * @subpackage 	Currencies
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class XE_currency_rates_model extends \Model
{

    /**
     * Link to CodeIgnitor object
     *
     * @var object
     */
    protected $ci;

    /**
     * Xe url template
     *
     * @var string
     */
    protected $xe_url = "http://www.xe.com/currencyconverter/convert/?Amount=1&From=%s&To=%s";

    /**
     * Class constructor
     *
     * @return XE_currency_rates_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Update exists currency rates
     *
     * @param string  $base_currency GUID of basic currency
     * @param array   $currencies    site currencies
     * @param boolean $use_curl      use curl extension
     *
     * @return array
     */
    public function update_rates($base_currency, $currencies, $use_curl = false)
    {
        if (empty($currencies)) {
            return array();
        }

        $this->ci->load->library("simple_html_dom");

        $return = array();
        foreach ($currencies as $need_currency) {
            if ($need_currency["gid"] == $base_currency) {
                continue;
            }

            $url_rate = sprintf($this->xe_url, $need_currency["gid"], $base_currency);

            if ($use_curl) {
                $curl = curl_init();
                curl_setopt($curl, CURLOPT_URL, $url_rate);
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
                $content = curl_exec($curl);
                curl_close($curl);
            } else {
                $content = file_get_contents($url_rate);
            }
            $html = str_get_html($content);
            $cur_content = array();
            //Cnvrsn
            foreach ($html->find(".uccMin table tr.uccRes") as $key_c => $tr) {
                foreach ($tr->find("td") as $key2 => $td) {
                    if ($key2 != 2) {
                        continue;
                    }
                    $tmp = explode("&nbsp;", $td->plaintext);
                    $tmp[0] = floatval($tmp[0]);
                    if ($tmp[0] == 0) {
                        continue;
                    }
                    $return[$need_currency["gid"]] = $tmp[0];
                }
            }
        }

        return $return;
    }

}
