<?php

/**
 * Export module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * SyndaFeed listings driver export model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Export
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Export_synda_listings_model extends Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Driver configuration
     *
     * @var array
     */
    public $drivers = array(
        "gid"      => "synda_listings",
        "link"     => "http://www.syndafeed.com",
        "settings" => array(),
        "elements" => array(
            array("name" => "PublisherId", "type" => "int", "required" => 1),
            array("name" => "ListingId", "type" => "int", "required" => 1),
            array("name" => "PropertyId", "type" => "int", "required" => 0),
            array("name" => "GMTLastModified", "type" => "text", "required" => 1),
            array("name" => "ListingStatus", "type" => "int", "required" => 1),
            array("name" => "PropertyType", "type" => "text", "required" => 1),
            array("name" => "AgentId", "type" => "int", "required" => 1),
            array("name" => "DistressSale", "type" => "text", "required" => 0),
            array("name" => "RentalPeriod", "type" => "text", "required" => 1),
            array("name" => "CapRate", "type" => "text", "required" => 0),
            array("name" => "Currency", "type" => "text", "required" => 1),
            array("name" => "Price", "type" => "int", "required" => 1),
            array("name" => "PropertyTax", "type" => "int", "required" => 0),
            array("name" => "HomeownerDues", "type" => "int", "required" => 0),
            array("name" => "StreetAddress", "type" => "text", "required" => 0),
            array("name" => "StreetAddress.hide", "type" => "int", "required" => 0),
            array("name" => "District", "type" => "text", "required" => 0),
            array("name" => "City", "type" => "text", "required" => 1),
            array("name" => "StateProvince", "type" => "text", "required" => 1),
            array("name" => "Country", "type" => "text", "required" => 1),
            array("name" => "PostalCode", "type" => "text", "required" => 0),
            array("name" => "StructureArea", "type" => "text", "required" => 0),
            array("name" => "LotLengthWidth", "type" => "text", "required" => 0),
            array("name" => "IrregularLot", "type" => "text", "required" => 0),
            array("name" => "LandArea", "type" => "text", "required" => 0),
            array("name" => "BedRooms", "type" => "text", "required" => 0),
            array("name" => "BathRooms", "type" => "text", "required" => 0),
            array("name" => "ReceptionRooms", "type" => "int", "required" => 0),
            array("name" => "TotalRooms", "type" => "text", "required" => 0),
            array("name" => "YearBuilt", "type" => "text", "required" => 0),
            array("name" => "AVMDeny", "type" => "int", "required" => 0),
            array("name" => "Latitude", "type" => "text", "required" => 0),
            array("name" => "Longitude", "type" => "text", "required" => 0),
            array("name" => "PhotoURL", "type" => "url", "required" => 0),
            array("name" => "VirtualTourURL", "type" => "url", "required" => 0),
            array("name" => "Previews.Begins", "type" => "text", "required" => 0),
            array("name" => "Previews.Ends", "type" => "text", "required" => 0),
            array("name" => "Previews.Notes", "type" => "text", "required" => 0),
            array("name" => "GMTOffset", "type" => "text", "required" => 0),
            array("name" => "DescriptionLang.Caption", "type" => "text", "required" => 1),
            array("name" => "DescriptionLang.Description", "type" => "text", "required" => 1),
            array("name" => "DescriptionLang.DetailsURL", "type" => "text", "required" => 1),
        ),
        "editable" => 0,
        "status"   => 1,
    );

    /**
     *  Demo data
     *
     *  @return void
     */
    public $demo_data = array(
        'gid_driver'      => 'synda_listings',
        'module'          => 'listings',
        'id_object'       => '2',
        'name'            => 'Test sample',
        'relations'       => 'a:20:{i:0;a:4:{s:4:"name";s:11:"PublisherId";s:4:"type";s:3:"int";s:4:"link";s:7:"id_user";s:5:"label";s:57:"���������� ������������� (int, id)";}i:1;a:4:{s:4:"name";s:9:"ListingId";s:4:"type";s:3:"int";s:4:"link";s:2:"id";s:5:"label";s:37:"����� ���������� (int)";}i:3;a:4:{s:4:"name";s:15:"GMTLastModified";s:4:"type";s:4:"text";s:4:"link";s:13:"date_modified";s:5:"label";s:25:"��������� (text)";}i:4;a:4:{s:4:"name";s:13:"ListingStatus";s:4:"type";s:3:"int";s:4:"link";s:6:"status";s:5:"label";s:24:"������� (int, id)";}i:5;a:4:{s:4:"name";s:12:"PropertyType";s:4:"type";s:4:"text";s:4:"link";s:13:"property_type";s:5:"label";s:38:"��� ������������ (text)";}i:10;a:4:{s:4:"name";s:8:"Currency";s:4:"type";s:4:"text";s:4:"link";s:12:"gid_currency";s:5:"label";s:19:"������ (text)";}i:11;a:4:{s:4:"name";s:5:"Price";s:4:"type";s:3:"int";s:4:"link";s:5:"price";s:5:"label";s:14:"���� (int)";}i:14;a:4:{s:4:"name";s:13:"StreetAddress";s:4:"type";s:4:"text";s:4:"link";s:7:"address";s:5:"label";s:17:"����� (text)";}i:17;a:4:{s:4:"name";s:4:"City";s:4:"type";s:4:"text";s:4:"link";s:7:"id_city";s:5:"label";s:17:"����� (text)";}i:18;a:4:{s:4:"name";s:13:"StateProvince";s:4:"type";s:4:"text";s:4:"link";s:9:"id_region";s:5:"label";s:19:"������ (text)";}i:19;a:4:{s:4:"name";s:7:"Country";s:4:"type";s:4:"text";s:4:"link";s:10:"id_country";s:5:"label";s:19:"������ (text)";}i:20;a:4:{s:4:"name";s:10:"PostalCode";s:4:"type";s:4:"text";s:4:"link";s:3:"zip";s:5:"label";s:36:"�������� ������ (text)";}i:31;a:4:{s:4:"name";s:8:"Latitude";s:4:"type";s:4:"text";s:4:"link";s:3:"lat";s:5:"label";s:19:"������ (text)";}i:32;a:4:{s:4:"name";s:9:"Longitude";s:4:"type";s:4:"text";s:4:"link";s:3:"lon";s:5:"label";s:21:"������� (text)";}i:33;a:4:{s:4:"name";s:8:"PhotoURL";s:4:"type";s:3:"url";s:4:"link";s:7:"uploads";s:5:"label";s:14:"���� (url)";}i:38;a:4:{s:4:"name";s:9:"GMTOffset";s:4:"type";s:4:"text";s:4:"link";s:11:"output_name";s:5:"label";s:25:"��������� (text)";}i:39;a:4:{s:4:"name";s:23:"DescriptionLang.Caption";s:4:"type";s:4:"text";s:4:"link";s:8:"headline";s:5:"label";s:40:"�������� �������� (text)";}i:6;a:4:{s:4:"name";s:7:"AgentId";s:4:"type";s:3:"int";s:4:"link";s:7:"id_user";s:5:"label";s:57:"���������� ������������� (int, id)";}i:40;a:4:{s:4:"name";s:27:"DescriptionLang.Description";s:4:"type";s:4:"text";s:4:"link";s:13:"fe_comments_1";s:5:"label";s:48:"����������� - �������� (text)";}i:41;a:4:{s:4:"name";s:26:"DescriptionLang.DetailsURL";s:4:"type";s:4:"text";s:4:"link";s:7:"seolink";s:5:"label";s:50:"URL ��������� ���������� (text)";}}',
        'admin_form_data' => '',
        'output_type'     => 'browser',
        'file_format'     => 'listings.xml',
        'append_date'     => '1',
        'scheduler'       => 'a:1:{s:4:"type";s:6:"manual";}',
        'published'       => '0',
        'editable'        => '0',
        'status'          => '1',
        'date_created'    => '',
    );

    /**
     * Not available symbols
     *
     * @var array
     */
    protected $search = array('&', "'", '"', ">", "<");

    /**
     * Symbols for replace
     *
     * @var array
     */
    protected $replace = array('&amp;', '&apos;', '&quot;', "&gt;", "&lt;");

    /**
     * Content type
     *
     * @var object
     */
    public $content_file = "text/xml";

    /**
     * Property types options
     *
     * @var array
     */
    protected $property_types = array('sfam', 'apt', 'land', 'duplex', 'multi', 'town', 'condo', 'comm', 'manu');

    /**
     * Listing status codes options
     *
     * @var array
     */
    protected $listing_status_codes = array('a', 's');

    /**
     * Rental period codes options
     *
     * @var array
     */
    protected $rental_period_codes = array('a', 's', 'w', 'm');

    /**
     * Distress sale codes options
     *
     * @var array
     */
    protected $distress_sale_codes = array('v', 's', 'f', 'b');

    /**
     * Constructor
     *
     * @return Export_synda_listings_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
    }

    /**
     * Return filename of export data by default
     *
     * @return string
     */
    public function get_filename()
    {
        return "listings.xml";
    }

    /**
     * Generate output file
     *
     * @param string $filename output filename
     * @param array  $data     export data
     * @param array  $settings driver settings
     *
     * @return void
     */
    public function generate($filename, $data, $settings)
    {
        $f = fopen($filename, "wb");

        fputs($f, '<?xml version="1.0" encoding="UTF-8"?>' . "\n");
        fputs($f, '<Listings>' . "\n");
        foreach ($data as $i => $row) {
            if (!$i) {
                continue;
            }

            fputs($f, '	<Listing>' . "\n");
            fputs($f, '			<PublisherId>' . $row['PublisherId'] . '</PublisherId>' . "\n");
            fputs($f, '			<ListingId>' . $row['ListingId'] . '</ListingId>' . "\n");

            if (isset($row['PropertyId']) && !empty($row['PropertyId'])) {
                fputs($f, '			<PropertyId>' . $row['PropertyId'] . '</PropertyId>' . "\n");
            }

            fputs($f, '			<GMTLastModified>' . $row['GMTLastModified'] . '</GMTLastModified>' . "\n");
            fputs($f, '			<ListingStatus>' . $row['ListingStatus'] . '</ListingStatus>' . "\n");
            fputs($f, '			<PropertyType>' . $row['PropertyType'] . '</PropertyType>' . "\n");
            fputs($f, '			<AgentId>' . $row['AgentId'] . '</AgentId>' . "\n");

            if (isset($row['DistressSale']) && !empty($row['DistressSale'])) {
                fputs($f, '			<DistressSale>' . $row['DistressSale'] . '</DistressSale>' . "\n");
            }

            fputs($f, '			<RentalPeriod>' . $row['RentalPeriod'] . '</RentalPeriod>' . "\n");

            if (isset($row['CapRate']) && !empty($row['CapRate'])) {
                fputs($f, '			<CapRate>' . $row['CapRate'] . '</CapRate>' . "\n");
            }

            fputs($f, '			<Currency>' . $row['Currency'] . '</Currency>' . "\n");
            fputs($f, '			<Price>' . $row['Price'] . '</Price>' . "\n");

            if (isset($row['PropertyTax']) && !empty($row['PropertyTax'])) {
                fputs($f, '			<PropertyTax>' . $row['PropertyTax'] . '</PropertyTax>' . "\n");
            }

            if (isset($row['HomeownerDues']) && !empty($row['HomeownerDues'])) {
                fputs($f, '			<HomeownerDues>' . $row['HomeownerDues'] . '</HomeownerDues>' . "\n");
            }

            if (isset($row['StreetAddress']) && !empty($row['StreetAddress'])) {
                fputs($f, '			<StreetAddress hide="false"><![CDATA[' . $row['StreetAddress'] . ']]></StreetAddress>' . "\n");
            }

            if (isset($row['District']) && !empty($row['District'])) {
                fputs($f, '			<District><![CDATA[' . $row['District'] . ']]></District>' . "\n");
            }

            fputs($f, '			<City><![CDATA[' . $row['City'] . ']]></City>' . "\n");
            fputs($f, '			<StateProvince><![CDATA[' . $row['StateProvince'] . ']]></StateProvince>' . "\n");
            fputs($f, '			<Country>' . $row['Country'] . '</Country>' . "\n");

            if (isset($row['PostalCode']) && !empty($row['PostalCode'])) {
                fputs($f, '			<PostalCode>' . $row['PostalCode'] . '</PostalCode>' . "\n");
            }

            if (isset($row['StructureArea']) && !empty($row['StructureArea'])) {
                fputs($f, '			<StructureArea>' . $row['StructureArea'] . '</StructureArea>' . "\n");
            }

            if (isset($row['LotLengthWidth']) && !empty($row['LotLengthWidth'])) {
                fputs($f, '			<LotLengthWidth>' . $row['LotLengthWidth'] . '</LotLengthWidth>' . "\n");
            }

            if (isset($row['IrregularLot']) && !empty($row['IrregularLot'])) {
                fputs($f, '			<IrregularLot>' . $row['IrregularLot'] . '</IrregularLot>' . "\n");
            }

            if (isset($row['LandArea']) && !empty($row['LandArea'])) {
                fputs($f, '			<LandArea>' . $row['LandArea'] . '</LandArea>' . "\n");
            }

            if (isset($row['BedRooms']) && !empty($row['BedRooms'])) {
                fputs($f, '			<BedRooms>' . $row['BedRooms'] . '</BedRooms>' . "\n");
            }

            if (isset($row['BathRooms']) && !empty($row['BathRooms'])) {
                fputs($f, '			<BathRooms>' . $row['BathRooms'] . '</BathRooms>' . "\n");
            }

            if (isset($row['ReceptionRooms']) && !empty($row['ReceptionRooms'])) {
                fputs($f, '			<ReceptionRooms>' . $row['ReceptionRooms'] . '</ReceptionRooms>' . "\n");
            }

            if (isset($row['TotalRooms']) && !empty($row['TotalRooms'])) {
                fputs($f, '			<TotalRooms>' . $row['TotalRooms'] . '</TotalRooms>' . "\n");
            }

            if (isset($row['YearBuilt']) && !empty($row['YearBuilt'])) {
                fputs($f, '			<YearBuilt>' . $row['YearBuilt'] . '</YearBuilt>' . "\n");
            }

            if (isset($row['AVMDeny']) && $row['AVMDeny']) {
                fputs($f, '			<AVMDeny>1</AVMDeny>' . "\n");
            }

            if (isset($row['Latitude']) && !empty($row['Latitude'])) {
                fputs($f, '			<Latitude>' . $row['Latitude'] . '</Latitude>' . "\n");
            }

            if (isset($row['Longitude']) && !empty($row['Longitude'])) {
                fputs($f, '			<Longitude>' . $row['Longitude'] . '</Longitude>' . "\n");
            }

            if (isset($row['PhotoURL']) && !empty($row['PhotoURL'])) {
                fputs($f, '			<PhotoURL>' . $row['PhotoURL'] . '</PhotoURL>' . "\n");
            }

            if (isset($row['VirtualTourURL']) && !empty($row['VirtualTourURL'])) {
                fputs($f, '			<VirtualTourURL>' . $row['VirtualTourURL'] . '</VirtualTourURL>' . "\n");
            }

            if (isset($row['Previews']) && !empty($row['Previews'])) {
                fputs($f, '			<Previews>' . "\n");

                foreach ((array) $row['Previews'] as $preview) {
                    fputs($f, '				<Preview>' . "\n");
                    fputs($f, '					<Begins>' . $preview['Begins'] . '</Begins>' . "\n");
                    fputs($f, '					<Ends>' . $preview['Ends'] . '</Ends>' . "\n");
                    fputs($f, '					<Notes><![CDATA[' . $preview['Notes'] . ']]></Notes>' . "\n");
                    fputs($f, '				</Preview>' . "\n");
                }

                fputs($f, '			</Previews>' . "\n");
            }

            if (isset($row['GMTOffset']) && !empty($row['GMTOffset'])) {
                fputs($f, '			<GMTOffset>' . $row['GMTOffset'] . '</GMTOffset>' . "\n");
            }

            fputs($f, '			<DescriptionLang value="' . $this->CI->pg_language->get_lang_code_by_id($this->CI->pg_language->current_lang_id) . '">' . "\n");
            fputs($f, '				<Caption><![CDATA[' . $row['DescriptionLang.Caption'] . ']]></Caption>' . "\n");
            fputs($f, '				<Description><![CDATA[' . $row['DescriptionLang.Description'] . ']]></Description>' . "\n");
            fputs($f, '				<DetailsURL>' . $row['DescriptionLang.DetailsURL'] . '</DetailsURL>' . "\n");
            fputs($f, '			</DescriptionLang>' . "\n");
            fputs($f, '	</Listing>' . "\n");
        }
        fputs($f, '</Listings>' . "\n");

        fclose($f);
    }

    /**
     * Validate export data
     *
     * @param array $data export data
     *
     * @return array
     */
    public function validate_data($data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data['PublisherId'])) {
            $return['data']['PublisherId'] = intval($data['PublisherId']);
            if (empty($return['data']['PublisherId'])) {
                $return["errors"][] = "PublisherId";
            }
        } else {
            $return["errors"][] = "PublisherId";
        }

        if (isset($data['ListingId'])) {
            $return['data']['ListingId'] = intval($data['ListingId']);
            if (empty($return['data']['ListingId'])) {
                $return["errors"][] = "ListingId";
            }
        } else {
            $return["errors"][] = "ListingId";
        }

        if (isset($data['PropertyId'])) {
            $return['data']['PropertyId'] = $data['PropertyId'];
        }

        if (isset($data['GMTLastModified'])) {
            $value = strtotime($data['GMTLastModified']);
            if ($value > 0) {
                $return['data']['GMTLastModified'] = date('c', $value);
            } else {
                $return["errors"][] = "GMTLastModified";
            }
        } else {
            $return["errors"][] = "GMTLastModified";
        }

        if (isset($data["ListingStatus"])) {
            $return['data']['ListingStatus'] = 'a';
            if (empty($return['data']['ListingStatus'])) {
                $return["errors"][] = "ListingStatus";
            }
        } else {
            $return['data']['ListingStatus'] = 'a';
        }

        if (isset($data['PropertyType'])) {
            $return['data']['PropertyType'] = $data['PropertyType'];
            if (!in_array($data["PropertyType"], $this->property_types)) {
                if (isset($data['PropertyType2'])) {
                    $return['data']['PropertyType'] = $data['PropertyType2'];
                    if (!in_array($data["PropertyType"], $this->property_types)) {
                        if (isset($data['PropertyType3'])) {
                            $return['data']['PropertyType'] = $data['PropertyType3'];
                            if (!in_array($data["PropertyType"], $this->property_types)) {
                                if (isset($data['PropertyType4'])) {
                                    $return['data']['PropertyType'] = $data['PropertyType4'];
                                    if (!in_array($data["PropertyType"], $this->property_types)) {
                                        if (isset($data['PropertyType5'])) {
                                            $return['data']['PropertyType'] = $data['PropertyType5'];
                                            if (!in_array($data["PropertyType"], $this->property_types)) {
                                                if (isset($data['PropertyType6'])) {
                                                    $return['data']['PropertyType'] = $data['PropertyType6'];
                                                    if (!in_array($data["PropertyType"], $this->property_types)) {
                                                        $return['data']['PropertyType'] = current($this->property_types);
                                                        //$return["errors"][] = "PropertyType";
                                                    }
                                                }
                                            }
                                        } else {
                                            $return['data']['PropertyType'] = current($this->property_types);
                                            //$return["errors"][] = "PropertyType";
                                        }
                                    }
                                } else {
                                    $return['data']['PropertyType'] = current($this->property_types);
                                    //$return["errors"][] = "PropertyType";
                                }
                            }
                        } else {
                            $return['data']['PropertyType'] = current($this->property_types);
                            //$return["errors"][] = "PropertyType";
                        }
                    }
                } else {
                    $return['data']['PropertyType'] = current($this->property_types);
                    //$return["errors"][] = "PropertyType";
                }
            }
        } else {
            $return['data']['PropertyType'] = current($this->property_types);
            //$return["errors"][] = "PropertyType";
        }

        if (isset($data['AgentId'])) {
            $return['data']['AgentId'] = intval($data['AgentId']);
            if (empty($data["AgentId"])) {
                $return["errors"][] = "AgentId";
            }
        } else {
            $return["errors"][] = "AgentId";
        }

        if (isset($data['DistressSale'])) {
            if (in_array($data["DistressSale"], $this->distress_sale_codes)) {
                $return['data']['DistressSale'] = $data['DistressSale'];
            }
        }

        if (isset($data['RentalPeriod'])) {
            if (in_array($data["RentalPeriod"], $this->rental_period_codes)) {
                $return['data']['RentalPeriod'] = $data['RentalPeriod'];
            } else {
                //$return["errors"][] = "RentalPeriod";
            }
        }

        if (isset($data['CapRate'])) {
            $return['data']['CapRate'] = $data['CapRate'];
        }

        if (isset($data['Currency'])) {
            $return['data']['Currency'] = $data['Currency'];
            if (empty($return['data']['Currency'])) {
                $return["errors"][] = "Currency";
            }
        } else {
            $return["errors"][] = "Currency";
        }

        if (isset($data['Price'])) {
            $return['data']['Price'] = $data['Price'];
            if (empty($return['data']['Price'])) {
                $return["errors"][] = "Price";
            }
        } else {
            $return["errors"][] = "Price";
        }

        if (isset($data['PropertyTax'])) {
            $return['data']['PropertyTax'] = $data['PropertyTax'];
        }

        if (isset($data['HomeownerDues'])) {
            $return['data']['HomeownerDues'] = $data['HomeownerDues'];
        }

        if (isset($data['StreetAddress'])) {
            $return['data']['StreetAddress'] = $data['StreetAddress'];
        }

        if (isset($data['District'])) {
            $return['data']['District'] = $data['District'];
        }

        if (isset($data['City'])) {
            $return['data']['City'] = $data['City'];
            if (empty($return['data']['City'])) {
                $return["errors"][] = "City";
            }
        } else {
            $return["errors"][] = "City";
        }

        if (isset($data['StateProvince'])) {
            $return['data']['StateProvince'] = $data['StateProvince'];
            if (empty($return['data']['StateProvince'])) {
                $return["errors"][] = "StateProvince";
            }
        } else {
            $return["errors"][] = "StateProvince";
        }

        if (isset($data['Country'])) {
            $return['data']['Country'] = $data['Country'];
            if (empty($return['data']['Country'])) {
                $return["errors"][] = "Country";
            }
        } else {
            $return["errors"][] = "Country";
        }

        if (isset($data['PostalCode'])) {
            $return['data']['PostalCode'] = $data['PostalCode'];
        }

        if (isset($data['StructureArea'])) {
            $return['data']['StructureArea'] = $data['StructureArea'];
        }

        if (isset($data['LotLengthWidth'])) {
            $return['data']['LotLengthWidth'] = intval($data['LotLengthWidth']);
        }

        if (isset($data['IrregularLot'])) {
            $return['data']['IrregularLot'] = $data['IrregularLot'];
        }

        if (isset($data['LandArea'])) {
            $return['data']['LandArea'] = $data['LandArea'];
        }

        if (isset($data['BedRooms'])) {
            $return['data']['BedRooms'] = $data['BedRooms'];
        }

        if (isset($data['BathRooms'])) {
            $return['data']['BathRooms'] = $data['BathRooms'];
        }

        if (isset($data['ReceptionRooms'])) {
            $return['data']['ReceptionRooms'] = $data['ReceptionRooms'];
        }

        if (isset($data['TotalRooms'])) {
            $return['data']['TotalRooms'] = $data['TotalRooms'];
        }

        if (isset($data['YearBuilt'])) {
            if (strlen($data['YearBuilt']) == 4) {
                $return['data']['YearBuilt'] = intval($data['YearBuilt']);
            }
        }

        if (isset($data['AVMDeny'])) {
            $return['data']['AVMDeny'] = $data['AVMDeny'] ? 1 : 0;
        }

        if (isset($data['Latitude'])) {
            $return['data']['Latitude'] = floatval($data['Latitude']);
        }

        if (isset($data['Longitude'])) {
            $return['data']['Longitude'] = floatval($data['Longitude']);
        }

        if (isset($data['PhotoURL'])) {
            if (!empty($data['PhotoURL'])) {
                $return['data']['PhotoURL'] = array_shift(explode('|', $data['PhotoURL']));
            }
        }

        if (isset($data['VirtualTourURL'])) {
            if (!empty($data['VirtualTourURL'])) {
                $return['data']['VirtualTourURL'] = array_shift(explode('|', $data['VirtualTourURL']));
            }
        }

        if (isset($data['Previews.Begins'])) {
            if (!empty($data['Previews.Begins'])) {
                $return['data']['Previews.Begins'] = explode('|', $data['Previews.Begins']);
                foreach ($return['data']['Previews.Begins'] as $key => $value) {
                    $value = strtotime($value);
                    if ($value > 0) {
                        $return['data']['Previews'][$key]['Begins'] = date('Y-m-d H:m:s', $value);
                    }
                }
            }
        }

        if (isset($data['Previews.Ends'])) {
            if (!empty($data['Previews.Ends'])) {
                $return['data']['Previews.Ends'] = explode('|', $data['Previews.Ends']);
                foreach ($return['data']['Previews.Ends'] as $key => $value) {
                    $value = strtotime($value);
                    if ($value > 0) {
                        $return['data']['Previews'][$key]['Ends'] = date('Y-m-d H:m:s', $value);
                    }
                }
            }
        }

        if (isset($data['Previews.Notes'])) {
            if (!empty($data['Previews.Notes'])) {
                $return['data']['Previews.Notes'] = explode('|', $data['Previews.Notes']);
                foreach ($return['data']['Previews.Notes'] as $key => $value) {
                    $return['data']['Previews'][$key]['Notes'] = $value;
                }
            }
        }

        if (isset($data['GMTOffset'])) {
            $return['data']['GMTOffset'] = $data['GMTOffset'];
        }

        if (isset($data['DescriptionLang.Caption'])) {
            $return['data']['DescriptionLang.Caption'] = $data['DescriptionLang.Caption'];
            if (empty($return['data']['DescriptionLang.Caption'])) {
                $return["errors"][] = "DescriptionLang.Caption";
            }
        } else {
            $return["errors"][] = "DescriptionLang.Caption";
        }

        if (isset($data['DescriptionLang.Description'])) {
            $return['data']['DescriptionLang.Description'] = $data['DescriptionLang.Description'];
            if (empty($return['data']['DescriptionLang.Description'])) {
                $return["errors"][] = "DescriptionLang.Description";
            }
        } else {
            $return["errors"][] = "DescriptionLang.Description";
        }

        if (isset($data['DescriptionLang.DetailsURL'])) {
            $return['data']['DescriptionLang.DetailsURL'] = $data['DescriptionLang.DetailsURL'];
            if (empty($return['data']['DescriptionLang.DetailsURL'])) {
                $return["errors"][] = "DescriptionLang.DetailsURL";
            }
        } else {
            $return["errors"][] = "DescriptionLang.DetailsURL";
        }

        return $return;
    }
}
