<?php

/**
 * Export module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * XML driver export model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Export
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Export_xml_model extends Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Driver configuration
     *
     * @var array
     */
    public $drivers = array(
        "gid" => "xml",
        "link" => "http://www.w3.org/XML/",
        "settings" => array(),
        "elements" => array(
            array("name" => "id", "type" => "int", "required" => 0),
            array("name" => "id_user", "type" => "int", "required" => 0),
            array("name" => "user_name", "type" => "text", "required" => 0),
            array("name" => "email", "type" => "text", "required" => 0),
            array("name" => "phone", "type" => "text", "required" => 0),
            array("name" => "id_type", "type" => "int", "required" => 0),
            array("name" => "type", "type" => "text", "required" => 0),
            array("name" => "id_category", "type" => "int", "required" => 0),
            array("name" => "category_name", "type" => "text", "required" => 0),
            array("name" => "id_property_type", "type" => "int", "required" => 0),
            array("name" => "property_type", "type" => "text", "required" => 0),
            array("name" => "status", "type" => "text", "required" => 0),
            array("name" => "country_code", "type" => "text", "required" => 0),
            array("name" => "id_region", "type" => "int", "required" => 0),
            array("name" => "id_city", "type" => "int", "required" => 0),
            array("name" => "country", "type" => "text", "required" => 0),
            array("name" => "region", "type" => "text", "required" => 0),
            array("name" => "region_code", "type" => "text", "required" => 0),
            array("name" => "city", "type" => "text", "required" => 0),
            array("name" => "address", "type" => "text", "required" => 0),
            array("name" => "zip", "type" => "text", "required" => 0),
            array("name" => "sold", "type" => "int", "required" => 0),
            array("name" => "price", "type" => "int", "required" => 0),
            array("name" => "price_max", "type" => "int", "required" => 0),
            array("name" => "gid_currency", "type" => "text", "required" => 0),
            array("name" => "square", "type" => "int", "required" => 0),
            array("name" => "square_max", "type" => "int", "required" => 0),
            array("name" => "square_unit", "type" => "text", "required" => 0),
            array("name" => "date_created", "type" => "text", "required" => 0),
            array("name" => "date_modified", "type" => "text", "required" => 0),
            array("name" => "date_publish", "type" => "text", "required" => 0),
            array("name" => "date_expire", "type" => "text", "required" => 0),
            array("name" => "date_open", "type" => "text", "required" => 0),
            array("name" => "uploads", "type" => "file", "required" => 0),
        ),
        "editable" => 1,
        "status" => 1,
    );

    private $demo_relations = [
        ['name' => 'id', 'type' => 'int', 'link' => 'id', 'label' => 'ID ref # (int)',],
        ['name' => 'user_name', 'type' => 'text', 'link' => 'id_user', 'label' => 'Listing provided by (text)',],
        ['name' => 'phone', 'type' => 'text', 'link' => 'user_phone', 'label' => 'Agent phone (text)',],
            ['name' => 'email', 'type' => 'text', 'link' => 'user_email', 'label' => 'Agent email (text)',],
        ['name' => 'type', 'type' => 'text', 'link' => 'type_code', 'label' => 'Type of operation (text, code)',],
        ['name' => 'category_name', 'type' => 'text', 'link' => 'id_category', 'label' => 'Category (text)',],
        ['name' => 'property_type', 'type' => 'text', 'link' => 'property_type', 'label' => 'Property type (text)',],
        ['name' => 'status', 'type' => 'text', 'link' => 'status', 'label' => 'Active (text)',],
        ['name' => 'country', 'type' => 'text', 'link' => 'id_country', 'label' => 'Country (text)',],
        ['name' => 'region', 'type' => 'text', 'link' => 'id_region', 'label' => 'Region (text)',],
        ['name' => 'region_code', 'type' => 'text', 'link' => 'region_code', 'label' => 'Postal code (text)',],
        ['name' => 'city', 'type' => 'text', 'link' => 'id_city', 'label' => 'City (text)',],
        ['name' => 'address', 'type' => 'text', 'link' => 'address', 'label' => 'Address (text)',],
        ['name' => 'zip', 'type' => 'text', 'link' => 'zip', 'label' => 'Postal code (text)',],
        ['name' => 'sold', 'type' => 'int', 'link' => 'sold', 'label' => 'Recently sold (int)',],
        ['name' => 'price', 'type' => 'int', 'link' => 'price', 'label' => 'Price (int)',],
        ['name' => 'price_max', 'type' => 'int', 'link' => 'price_max', 'label' => 'Max price (int)',],
        ['name' => 'gid_currency', 'type' => 'text', 'link' => 'gid_currency', 'label' => 'Currency (text)',],
        ['name' => 'square', 'type' => 'int', 'link' => 'square', 'label' => 'Property size (int)',],
        ['name' => 'square_max', 'type' => 'int', 'link' => 'square_max', 'label' => 'Property size (max) (int)',],
        ['name' => 'square_unit', 'type' => 'text', 'link' => 'square_unit', 'label' => 'Unit (text)',],
        ['name' => 'date_created', 'type' => 'text', 'link' => 'date_created', 'label' => 'Created (text)',],
    ];

    /**
     *  Demo data
     *
     *  @return void
     */
    public $demo_data = array(
        'gid_driver' => 'xml',
        'id_object' => '2',
        'module' => 'listings',
        'name' => 'Test sample',
        'relations' => null, // See the constructor
        'admin_form_data' => '',
        'output_type' => 'browser',
        'file_format' => 'test.xml',
        'append_date' => '1',
        'scheduler' => 'a:1:{s:4:"type";s:6:"manual";}',
        'published' => '0',
        'editable' => '0',
        'status' => '1',
        'date_created' => '',
    );

    /**
     * Not available symbols
     *
     * @var array
     */
    protected $search = array('&', "'", '"', ">", "<");

    /**
     * Symbols for replace
     *
     * @var array
     */
    protected $replace = array('&amp;', '&apos;', '&quot;', "&gt;", "&lt;");

    /**
     * Content type
     *
     * @var array
     */
    public $content_type = 'text/xml';

    /**
     * Constructor
     *
     * @return Export_xml_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
        $this->demo_data['relations'] = serialize($this->demo_relations);
    }

    /**
     * Return filename of export data by default
     *
     * @return string
     */
    public function get_filename()
    {
        return "export.xml";
    }

    /**
     * Generate output file
     *
     * @param string $filename output filename
     * @param array  $data     export data
     * @param array  $settings driver settings
     *
     * @return void
     */
    public function generate($filename, $data, $settings)
    {
        $f = fopen($filename, "wb");
        fputs($f, '<?xml version="1.0" encoding="utf-8"?>' . "\n");
        fputs($f, '<items>' . "\n");
        foreach ($data as $i => $row) {
            if (!$i) {
                continue;
            }
            fputs($f, '	<item>' . "\n");
            foreach ($row as $key => $value) {
                $value = str_replace($this->search, $this->replace, $value);
                fputs($f, '			<' . $key . '>' . $value . '</' . $key . '>' . "\n");
            }
            fputs($f, '	</item>' . "\n");
        }
        fputs($f, '</items>' . "\n");
        fclose($f);
    }

    /**
     * Validate export data
     *
     * @param array $data export data
     *
     * @return array
     */
    public function validate_data($data)
    {
        $return = array("errors" => array(), "data" => array());
        $return["data"] = $data;

        return $return;
    }

}
