<?php

namespace Pg\Modules\Export\Models;

use Pg\Libraries\Setup;

/**
 * Export module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Export install model
 *
 * @package	 	PG_RealEstate
 * @subpackage 	Export
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Export_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    protected $modules_data;
    /**
     * Constructor
     *
     * @return Export_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->modules_data = Setup::getModuleData('export', Setup::TYPE_MODULES_DATA);
    }

    /**
     * Install data of menu module
     *
     * @return array
     */
    public function install_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $this->modules_data['menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], get($menu_data["name"]));
            linked_install_process_menu_items($this->modules_data['menu'], "create", $gid, 0, $this->modules_data['menu'][$gid]["items"]);
        }
    }

    /**
     * Import languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("export", "menu", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data");
            return false;
        }

        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['menu'], "update", $gid, 0, $this->modules_data['menu'][$gid]["items"], $gid, $langs_file);
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu'], "export", $gid, 0, $this->modules_data['menu'][$gid]["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall data of menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper("menu");
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $this->modules_data['menu'][$gid]["items"]);
            }
        }
    }

    /**
     * Install data of moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        // install moderators permissions
        $this->ci->load->model("Moderators_model");

        foreach ((array) $this->modules_data['moderators'] as $method_data) {
            $validate_data = array("errors" => array(), "data" => $method_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Moderators_model->save_method(null, $validate_data["data"]);
        }
    }

    /**
     * Import languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("export", "moderators", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty moderators langs data");

            return false;
        }

        // install moderators permissions
        $this->ci->load->model("Moderators_model");
        $params["where"]["module"] = "export";
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method["method"]])) {
                $this->ci->Moderators_model->save_method($method["id"], array(), $langs_file[$method["method"]]);
            }
        }
    }

    /**
     * Export languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model("Moderators_model");
        $params["where"]["module"] = "export";
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method["method"]] = $method["langs"];
        }

        return array("moderators" => $return);
    }

    /**
     * Uninstall data of moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        $this->ci->load->model("Moderators_model");
        $params = array();
        $params["where"]["module"] = "export";
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install data of cronjob module
     *
     * @return void
     */
    public function install_cronjob()
    {
        $this->ci->load->model("Cronjob_model");
        $cron_data = array(
            "name" => "Export scheduler",
            "module" => "export",
            "model" => "Export_model",
            "method" => "cron_export",
            "cron_tab" => "0 1 * * *",
            "status" => "1",
        );
        $this->ci->Cronjob_model->save_cron(null, $cron_data);
    }

    /**
     * Uninstall data of cronjob module
     *
     * @return void
     */
    public function deinstall_cronjob()
    {
        $this->ci->load->model("Cronjob_model");
        $cron_data = array();
        $cron_data["where"]["module"] = "export";
        $this->ci->Cronjob_model->delete_cron_by_param($cron_data);
    }

    /**
     * Install module data
     *
     * @return array
     */
    public function _arbitrary_installing()
    {
        if(!empty($this->modules_data['seo_data'])) {
            // seo
            $this->ci->pg_seo->set_seo_module("export", (array) $this->modules_data['seo_data']);
        }

        // add seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        foreach ($this->modules_data['seo_settings'] as $seo) {
            $data = array(
                'noindex' => 1,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => $seo['url_template'],
            );
            $this->ci->pg_seo->set_settings('user', 'export', $seo['method'], $data);

            $xml_data['export'][$seo['method']] = $this->ci->pg_seo->url_template_transform('export', $seo['method'], $data["url_template"], 'base', 'xml');
        }

        $this->ci->Seo_model->set_xml_route_file_content($xml_data);
        $this->ci->Seo_model->rewrite_route_php_file();
    }

    /**
     * Import module languages
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function _arbitrary_lang_install($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("export", "arbitrary", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty export arbitrary langs data");

            return false;
        }

        foreach ($this->modules_data['seo_settings'] as $seo) {
            $post_data = array(
                "title" => $langs_file["seo_tags_{$seo['method']}_title"],
                "keyword" => $langs_file["seo_tags_{$seo['method']}_keyword"],
                "description" => $langs_file["seo_tags_{$seo['method']}_description"],
                "header" => $langs_file["seo_tags_{$seo['method']}_header"],
                "og_title" => $langs_file["seo_tags_{$seo['method']}_og_title"],
                "og_type" => $langs_file["seo_tags_{$seo['method']}_og_type"],
                "og_description" => $langs_file["seo_tags_{$seo['method']}_og_description"],
            );
            $this->ci->pg_seo->set_settings("user", "export", $seo['method'], $post_data);
        }
        return;
    }

    /**
     * Export module languages
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids = null)
    {
        // arbitrary
        $arbitrary_return = [];
        foreach (array_keys($this->modules_data['seo_settings']) as $seo_method) {
            $settings = $this->ci->pg_seo->get_settings('user', 'export', $seo_method, $langs_ids);
            foreach ($this->ci->pg_language->languages as $lang) {
                $arbitrary_return["seo_tags_{$seo_method}_title"][$lang['id']] = $settings['meta_' . $lang['id']]['title'];
                $arbitrary_return["seo_tags_{$seo_method}_keyword"][$lang['id']] = $settings['meta_' . $lang['id']]['keyword'];
                $arbitrary_return["seo_tags_{$seo_method}_description"][$lang['id']] = $settings['meta_' . $lang['id']]['description'];
                $arbitrary_return["seo_tags_{$seo_method}_header"][$lang['id']] = $settings['meta_' . $lang['id']]['header'];
                $arbitrary_return["seo_tags_{$seo_method}_og_title"][$lang['id']] = $settings['og_' . $lang['id']]['og_title'];
                $arbitrary_return["seo_tags_{$seo_method}_og_type"][$lang['id']] = $settings['og_' . $lang['id']]['og_type'];
                $arbitrary_return["seo_tags_{$seo_method}_og_description"][$lang['id']] = $settings['og_' . $lang['id']]['og_description'];
            }
        }
        return array('arbitrary' => $arbitrary_return);
    }

    /**
     * Unistall module data
     *
     * @return array
     */
    public function _arbitrary_deinstalling()
    {
        // remove seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        foreach (array_keys($this->modules_data['seo_settings']) as $seo_method) {
            $data = array(
                'noindex' => 0,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => '',
            );
            $this->ci->pg_seo->set_settings('user', 'export', $seo_method, $data);
            $xml_data['export'][$seo_method] = $this->ci->pg_seo->url_template_transform('export', $seo_method, $data['url_template'], 'base', 'xml');
        }

        $this->ci->Seo_model->set_xml_route_file_content($xml_data);
        $this->ci->Seo_model->rewrite_route_php_file();

        // seo
        $this->ci->pg_seo->delete_seo_module('export');
    }

}
