<?php

/**
 * Export module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

define("EXPORT_MODULES_TABLE", DB_PREFIX . "export_modules");

/**
 * Export module model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Export
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Export_module_model extends Model
{

    /**
     * Link to CodeIgniter model
     *
     * @var object
     */
    protected $ci;

    /**
     * Properties of module object in data source
     *
     * @var array
     */
    protected $attrs = array(
        "id",
        "module",
        "model",
        "callback_get_fields",
        "callback_get_admin_form",
        "callback_get_user_form",
        "callback_process_form",
        "callback_export_data",
        "sorter",
        "date_created",
        "date_modified",
    );

    /**
     * Module data cache
     *
     * @var array
     */
    protected $module_cache = array();

    /**
     * Constructor
     *
     * @return Export_module_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return module object by identifier
     *
     * @param integer $module_id module identifier
     * @param boolean $formatted format results
     *
     * @return array
     */
    public function get_module_by_id($module_id, $formatted = true)
    {
        if (empty($this->module_cache[$module_id])) {
            $this->ci->db->select(implode(", ", $this->attrs))
                ->from(EXPORT_MODULES_TABLE)
                ->where("id", $module_id);
            $results = $this->ci->db->get()->result_array();
            if (!empty($results) && is_array($results)) {
                if ($formatted) {
                    $this->module_cache[$module_id] = $this->format_module($results[0]);
                } else {
                    return $results[0];
                }
            } else {
                return [];
            }
        }

        return $this->module_cache[$module_id];
    }

    /**
     * Return module objects as array
     *
     * @param boolean $formatted format results
     *
     * @return array
     */
    public function get_modules($formatted = true, array $attrs = [])
    {
        $data = array();
        $this->ci->db->select(implode(", ", $this->attrs))
            ->from(EXPORT_MODULES_TABLE)
            ->order_by('sorter');
        foreach ($attrs as $name => $value) {
            $this->ci->db->where($name, $value);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $result) {
                if ($formatted) {
                    $data[] = $this->format_module($result);
                } else {
                    $data[] = $result;
                }
            }
        }

        return $data;
    }

    /**
     * Save module object to data source
     *
     * @param integer $module_id module identifer
     * @param array   $data      module data
     *
     * @return integer
     */
    public function save_module($module_id, $data)
    {
        if (!$module_id) {
            $data["date_created"] = date("Y-m-d H:i:s");
            $data["date_modified"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(EXPORT_MODULES_TABLE, $data);
            $module_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where("id", $module_id);
            $this->ci->db->update(EXPORT_MODULES_TABLE, $data);
        }

        return $module_id;
    }

    /**
     * Remove export module object by guid
     *
     * @param string $module_gid module guid
     *
     * @return void
     */
    public function remove_module($module_gid)
    {
        $this->ci->db->where("module", $module_gid);
        $this->ci->db->delete(EXPORT_MODULES_TABLE);
    }

    /**
     * Validate module data for saving to data source
     *
     * @param integer $id   module identifier
     * @param array   $data data for validation
     *
     * @return array
     */
    public function validate_module($id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
            if (empty($return['data']['id'])) {
                unset($return['data']['id']);
            }
        }

        if (isset($data['module'])) {
            $return['data']['module'] = trim(strip_tags($data["module"]));
            if (empty($return['data']['module'])) {
                $return["errors"][] = l("error_empty_module", "export");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_module", "export");
        }

        if (isset($data['model'])) {
            $return['data']['model'] = trim(strip_tags($data["model"]));
            if (empty($return['data']['model'])) {
                $return["errors"][] = l("error_empty_model", "export");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_model", "export");
        }

        if (isset($data['callback_get_fields'])) {
            $return['data']['callback_get_fields'] = trim(strip_tags($data["callback_get_fields"]));
            if (empty($return['data']['callback_get_fields'])) {
                $return["errors"][] = l("error_empty_callback_get_fields", "export");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_callback_get_fields", "export");
        }

        if (isset($data['callback_get_admin_form'])) {
            $return['data']['callback_get_admin_form'] = trim(strip_tags($data["callback_get_admin_form"]));
            if (empty($return['data']['callback_get_admin_form'])) {
                $return["errors"][] = l("error_empty_callback_get_admin_form", "export");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_callback_get_admin_form", "export");
        }

        if (isset($data['callback_get_user_form'])) {
            $return['data']['callback_get_user_form'] = trim(strip_tags($data["callback_get_user_form"]));
            if (empty($return['data']['callback_get_user_form'])) {
                $return["errors"][] = l("error_empty_callback_get_user_form", "export");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_callback_get_user_form", "export");
        }

        if (isset($data['callback_process_form'])) {
            $return['data']['callback_process_form'] = trim(strip_tags($data["callback_process_form"]));
            if (empty($return['data']['callback_process_form'])) {
                $return["errors"][] = l("error_empty_callback_process_form", "export");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_callback_process_form", "export");
        }

        if (isset($data['callback_export_data'])) {
            $return['data']['callback_export_data'] = trim(strip_tags($data["callback_export_data"]));
            if (empty($return['data']['callback_export_data'])) {
                $return["errors"][] = l("error_empty_callback_export_data", "export");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_callback_export_data", "export");
        }

        if (isset($data['sorter'])) {
            $return['data']['sorter'] = intval($data['sorter']);
        }

        if (isset($data['date_created'])) {
            $value = strtotime($data['date_created']);
            if ($value > 0) {
                $return['data']['date_created'] = date("Y-m-d", $value);
            }
        }

        if (isset($data['date_modified'])) {
            $value = strtotime($data['date_modified']);
            if ($value > 0) {
                $return['data']['date_modified'] = date("Y-m-d", $value);
            }
        }

        return $return;
    }

    /**
     * Format module data
     *
     * @param array $data module data
     *
     * @return array
     */
    public function format_module($data)
    {
        $data["output_name"] = l("module_" . $data["module"], "export");

        return $data;
    }

    /**
     * Return module object by default
     *
     * @param integer $module_id module identifier
     *
     * @return array
     */
    public function format_default_module($module_id)
    {
        return;
    }

    /**
     * Return available fields for exporting
     *
     * @param integer $module_id module identifier
     *
     * @return array
     */
    public function get_module_fields($module_id)
    {
        $module = $this->get_module_by_id($module_id);
        if (empty($module["module"]) || empty($module["model"]) ||
            empty($module["callback_get_fields"])) {
            return array();
        }
        $this->ci->load->model($module["module"] . "/models/" . $module["model"], $module["model"]);
        $fields = $this->ci->{$module["model"]}->{$module["callback_get_fields"]}();

        return $fields;
    }

    /**
     * Return content of search form
     *
     * @param integer $module_id  module identifier
     * @param array   $data       export settings
     * @param boolean $admin_mode turn on if it is admin mode
     *
     * @return array
     */
    public function get_search_form($module_id, $data, $admin_mode = true)
    {
        $module = $this->get_module_by_id($module_id);
        if (empty($module["module"]) || empty($module["model"]) || empty($module["callback_get_admin_form"])) {
            return '';
        }
        $this->ci->load->model($module["module"] . "/models/" . $module["model"], $module["model"]);
        if ($admin_mode) {
            return $this->ci->{$module["model"]}->{$module["callback_get_admin_form"]}($data);
        } else {
            return $this->ci->{$module["model"]}->{$module["callback_get_user_form"]}($data);
        }
    }

    /**
     * Process submit from search form
     *
     * @param integer $module_id  module identifier
     * @param boolean $admin_mode turn on if it is admin mode
     *
     * @return array
     */
    public function process_search_form($module_id, $admin_mode = true)
    {
        $module = $this->get_module_by_id($module_id);
        if (empty($module["module"]) || empty($module["model"]) ||
            empty($module["callback_get_admin_form"])) {
            return array();
        }
        $this->ci->load->model($module["module"] . "/models/" . $module["model"], $module["model"]);

        return $this->ci->{$module["model"]}->{$module["callback_process_form"]}($admin_mode);
    }

    /**
     * Return export data from module
     *
     * @param integer $module_id module identifier
     * @param array   $data      export settings
     * @param array   $relations selection relations
     * @param integer $user_id   user identifier
     *
     * @return array
     */
    public function export_data($module_id, $data = array(), $relations = array(), $user_id = 0)
    {
        $module = $this->get_module_by_id($module_id);
        if (empty($module["module"]) || empty($module["model"]) ||
            empty($module["callback_export_data"])) {
            return array();
        }
        $this->ci->load->model($module["module"] . "/models/" . $module["model"], $module["model"]);

        return $this->ci->{$module["model"]}->{$module["callback_export_data"]}($data, $relations, $user_id);
    }

    /**
     * Import languages to export module
     *
     * @param array $data       module data
     * @param array $langs_file languages from file
     * @param array $langs_ids  languages identifiers from file
     *
     * @return void
     */
    public function update_lang($data, $langs_file, $langs_ids)
    {
        foreach ($data as $value) {
            $this->ci->pg_language->pages->set_string_langs(
                "export", $value, $langs_file[$value], $langs_ids);
        }
    }

    /**
     * Export languages from export module
     *
     * @param array $data      module data
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function export_lang($data, $langs_ids)
    {
        $langs = array();

        return array_merge($langs, $this->ci->pg_language->export_langs("export", (array) $data, $langs_ids));
    }

}
