<?php

namespace Pg\Modules\Geomap\Models;

/**
 * Geomap module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Bing maps driver model
 *
 * version 7
 *
 * @package 	PG_RealEstate
 * @subpackage 	Geomap
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Bingmapsv7_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Constructor
     *
     * @return Bingmapsv7_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Create map content
     *
     * @param string $key           map driver key
     * @param array  $settings      map driver settings
     * @param array  $view_settings view settings
     * @param array  $markers       markers data
     * @param string $map_id        map identifier
     *
     * @return string
     */
    public function create_html($key, $settings, $view_settings, $markers = array(), $map_id = false)
    {
        $this->ci->view->assign('map_reg_key', $key);
        $this->ci->view->assign('settings', $settings);
        $this->ci->view->assign('view_settings', $view_settings);
        $this->ci->view->assign('markers', $markers);
        $this->ci->view->assign('map_id', $map_id);

        if (@include(APPPATH . 'modules/geomap/install/languages' . EXT)) {
            $lang_code = $this->ci->pg_language->languages[$this->ci->pg_language->current_lang_id]['code'];
            if (isset($languages['bing'][$lang_code])) {
                $this->ci->view->assign('lang_code', $languages['bing'][$lang_code]);
            }
        }
        $this->ci->view->assign('rand', rand(100000, 999999));
        return $this->ci->view->fetch('bingmapsv7_html', null, 'geomap');
    }

    /**
     * Update map content
     *
     * @param string $map_id  map identifier
     * @param array  $markers markers data
     *
     * @return string
     */
    public function update_html($map_id, $markers = array())
    {
        $this->ci->view->assign('map_id', $map_id);
        $this->ci->view->assign('markers', $markers);

        return $this->ci->view->fetch('bingmapsv7_update', null, 'geomap');
    }

    /**
     * Append geocoder object to map
     *
     * @param string $key map driver key
     *
     * @return string
     */
    public function create_geocoder($key)
    {
        if (@include(APPPATH . 'modules/geomap/install/languages' . EXT)) {
            $lang_code = $this->pg_language->languages[$this->pg_language->current_lang_id]['code'];
            if (isset($languages['bing'][$lang_code])) {
                $this->ci->view->assign('lang_code', $languages['bing'][$lang_code]);
            }
        }

        return $this->ci->view->fetch('bingmapsv7_geocoder', null, 'geomap');
    }

    /**
     * Append street view to map
     *
     * @param string $key map driver key
     *
     * @return string
     */
    public function create_street_view($key)
    {
        return;
    }

}
