<?php

namespace Pg\Modules\Listings\Models;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Listings Dynamic Blocks Model
 * 
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	models
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_dynamic_blocks_model extends \Model
{

    /**
     * Link to CodeIgniter object
     * 
     * @var object
     */
    protected $ci;

    /**
     * Constructor
     *
     * return Listings_dynamic_blocks_model
     */
    public function __construct()
    {
        parent::__construct();

        $this->ci = & get_instance();
    }

    /**
     * Return listings slider with search form block
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block width
     * @return string
     */
    public function _dynamic_block_get_form_slider_listings($params, $view = 'latest_added_1600_440', $width = 100)
    {
        $sep_pos = strrpos(substr($view, 0, strrpos($view, '_')), '_');
        $type = substr($view, 0, $sep_pos);
        $view = substr($view, $sep_pos + 1);
        if (!isset($params['count']))
            $params['count'] = 8;
        if (!isset($params['columns']))
            $params['columns'] = 3;
        $this->ci->view->assign('block_type', $type);
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_slider_and_form', 'user', 'listings');
    }

    /**
     * Return listings slider block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block width
     * @return string
     */
    public function _dynamic_block_get_slider_listings($params, $view = 'latest_added_980_440', $width = 100)
    {
        $count = $params['count'] ? intval($params['count']) : 8;
        $sep_pos = strrpos(substr($view, 0, strrpos($view, '_')), '_');
        $type = substr($view, 0, $sep_pos);
        $view = substr($view, $sep_pos + 1);
        if (!isset($params['count']))
            $params['count'] = 8;
        if (!isset($params['columns']))
            $params['columns'] = 3;
        $this->ci->view->assign('block_type', $type);
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_slider', 'user', 'listings');
    }

    /**
     * Return featured listings block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block size
     * @return string
     */
    public function _dynamic_block_get_featured_listings($params, $view = 'gallery_middle', $width = 100)
    {
        $sep_pos = strrpos($view, '_');
        $size = substr($view, $sep_pos + 1);
        $view = substr($view, 0, $sep_pos);
        if (!isset($params['count']))
            $params['count'] = 8;
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_size', $size);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_featured_listings', 'user', 'listings');
    }

    /**
     * Return latest added listings block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block size
     * @return string
     */
    public function _dynamic_block_get_latest_added_listings($params, $view = 'gallery_middle', $width = 100)
    {
        $sep_pos = strrpos($view, '_');
        $size = substr($view, $sep_pos + 1);
        $view = substr($view, 0, $sep_pos);
        if (!isset($params['count']))
            $params['count'] = 8;
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_size', $size);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_latest_added_listings', 'user', 'listings');
    }

    /**
     * Return listings for sale block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block size
     * @return string
     */
    public function _dynamic_block_get_sale_listings($params, $view = 'gallery_middle', $width = 100)
    {
        $sep_pos = strrpos($view, '_');
        $size = substr($view, $sep_pos + 1);
        $view = substr($view, 0, $sep_pos);
        if (!isset($params['count']))
            $params['count'] = 8;
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_size', $size);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_sale_listings', 'user', 'listings');
    }

    /**
     * Return listings for buy block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block size
     * @return string
     */
    public function _dynamic_block_get_buy_listings($params, $view = 'gallery_middle', $width = 100)
    {
        $sep_pos = strrpos($view, '_');
        $size = substr($view, $sep_pos + 1);
        $view = substr($view, 0, $sep_pos);
        if (!isset($params['count']))
            $params['count'] = 8;
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_size', $size);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_buy_listings', 'user', 'listings');
    }

    /**
     * Return listings for rent block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block size
     * @return string
     */
    public function _dynamic_block_get_rent_listings($params, $view = 'gallery_middle', $width = 100)
    {
        $sep_pos = strrpos($view, '_');
        $size = substr($view, $sep_pos + 1);
        $view = substr($view, 0, $sep_pos);
        if (!isset($params['count']))
            $params['count'] = 8;
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_size', $size);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_rent_listings', 'user', 'listings');
    }

    /**
     * Return listings for lease block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block size
     * @return string
     */
    public function _dynamic_block_get_lease_listings($params, $view = 'gallery_middle', $width = 100)
    {
        $sep_pos = strrpos($view, '_');
        $size = substr($view, $sep_pos + 1);
        $view = substr($view, 0, $sep_pos);
        if (!isset($params['count']))
            $params['count'] = 8;
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_size', $size);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_lease_listings', 'user', 'listings');
    }

    /**
     * Return listings for sale in categories block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block width
     * @return string
     */
    public function _dynamic_block_get_sale_categories_search($params, $view = 'default', $width = 100)
    {
        if (!isset($params['columns']))
            $params['columns'] = 3;
        $this->ci->view->assign('block_columns', $params['columns']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_categories_and_sale_listings', 'user', 'listings');
    }

    /**
     * Return listings for buy in categories block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block width
     * @return string
     */
    public function _dynamic_block_get_buy_categories_search($params, $view = 'default', $width = 100)
    {
        if (!isset($params['columns']))
            $params['columns'] = 3;
        $this->ci->view->assign('block_columns', $params['columns']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_categories_and_buy_listings', 'user', 'listings');
    }

    /**
     * Return listings for rent in categories block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block width
     * @return string
     */
    public function _dynamic_block_get_rent_categories_search($params, $view = 'default', $width = 100)
    {
        if (!isset($params['columns']))
            $params['columns'] = 3;
        $this->ci->view->assign('block_columns', $params['columns']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_categories_and_rent_listings', 'user', 'listings');
    }

    /**
     * Return listings for lease in categories block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block width
     * @return string
     */
    public function _dynamic_block_get_lease_categories_search($params, $view = 'default', $width = 100)
    {
        if (!isset($params['columns']))
            $params['columns'] = 3;
        $this->ci->view->assign('block_columns', $params['columns']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_categories_and_lease_listings', 'user', 'listings');
    }

    /**
     * Return listings for sale in regions block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block width
     * @return string
     */
    public function _dynamic_block_get_sale_regions_search($params, $view = 'default', $width = 100)
    {
        if (!isset($params['columns']))
            $params['columns'] = 3;
        $this->ci->view->assign('block_columns', $params['columns']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_regions_and_sale_listings', 'user', 'listings');
    }

    /**
     * Return listings for buy in regions block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @return string
     */
    public function _dynamic_block_get_buy_regions_search($params, $view = 'default', $width = 100)
    {
        if (!isset($params['columns']))
            $params['columns'] = 3;
        $this->ci->view->assign('block_columns', $params['columns']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_regions_and_buy_listings', 'user', 'listings');
    }

    /**
     * Return listings for rent in regions block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block width
     * @return string
     */
    public function _dynamic_block_get_rent_regions_search($params, $view = 'default', $width = 100)
    {
        if (!isset($params['columns']))
            $params['columns'] = 3;
        $this->ci->view->assign('block_columns', $params['columns']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_regions_and_rent_listings', 'user', 'listings');
    }

    /**
     * Return listings for lease in regions block
     * 
     * Callback method for dynamic blocks module
     * 
     * @param array $params block parameters
     * @param string $view block view
     * @param integer $width block width
     * @return string
     */
    public function _dynamic_block_get_lease_regions_search($params, $view = 'default', $width = 100)
    {
        if (!isset($params['columns']))
            $params['columns'] = 3;
        $this->ci->view->assign('block_columns', $params['columns']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_regions_and_lease_listings', 'user', 'listings');
    }

}
