<?php

use Pg\Modules\Listings_booking\Models\Listings_booking_model;
use Pg\Modules\Listings\Models\Listings_model;

/**
 * Listings booking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing booking software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Listings booking management
 *
 * @package	 	PG_RealEstate
 * @subpackage 	Listings booking
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing booking software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('listings_booking_block')) {

    /**
     * Booking form
     *
     * @param array $params block data
     *
     * @return string
     */
    function listings_booking_block($params)
    {
        $ci = &get_instance();
        $ci->load->model('Listings_model');

        $is_rent = $ci->Listings_model->operations_arr[$params['listing']['id_type']] === Listings_model::OPERATION_RENT || 
            (isset($params['listing']['operation_type']) && $params['listing']['operation_type'] === Listings_model::OPERATION_RENT);
        if (!$is_rent || !$params['listing']['use_calendar']) {
            return '';
        }

        if ($params['listing']['id_user'] === $ci->session->userdata('user_id')) {
            return '';
        }

        $ci->Listings_model->set_format_settings(['get_photos' => true,]);
        $listing = $ci->Listings_model->format_listing($params['listing']);
        $ci->Listings_model->set_format_settings(['get_photos' => false,]);
        $ci->view->assign('listing', $listing);
        $params_to_assign = ['template', 'separator', 'show_price', 'show_link', 'no_save', 'guests',];
        foreach ($params_to_assign as $param_to_assign) {
            if (isset($params[$param_to_assign])) {
                $ci->view->assign($param_to_assign, $params[$param_to_assign]);
            }
        }

        if (empty($params['template']) || ($params['template'] != 'button' && $params['template'] != 'icon' && $params['template'] != 'link')) {
            // switch ($params['listing']['price_period']) {
            //     // case Listings_booking_model::PRICE_PERIOD_DAY:
            //     //     $date_start = get($params['period_date']['date_start'], date('Y-m-d'));
            //     //     $date_end = get($params['period_date']['date_end'], date('Y-m-d', time() + max($params['listing']['calendar_period_min'], 1) * 86400));

            //     //     $ci->view->assign('current_date_start', $date_start);
            //     //     $ci->view->assign('current_date_end', $date_end);
            //     //     break;
            //     // case Listings_booking_model::PRICE_PERIOD_MONTH:
            //     //     $month_start = get($params['period_date']['month_start'], date('m'));
            //     //     $year_start = get($params['period_date']['year_start'], date('Y'));
            //     //     $month_end = get($params['period_date']['month_end'], ($month_start + max($params['listing']['calendar_period_min'], 1)) % 12);
            //     //     $year_end = get($params['period_date']['year_end'], $year_start + intval(($month_start + max($params['listing']['calendar_period_min'], 1) - 2) / 12));

            //     //     if (!$month_end) {
            //     //         $month_end = 12;
            //     //     }

            //     //     $ci->view->assign('current_month_start', $month_start);
            //     //     $ci->view->assign('current_year_start', $year_start);
            //     //     $ci->view->assign('current_month_end', $month_end);
            //     //     $ci->view->assign('current_year_end', $year_end);

            //     //     $date_start = $year_start . '-' . $month_start . '-01';
            //     //     $date_end = $year_end . '-' . $month_end . '-01';
            //     //     break;
            // }

            $ci->load->model('listings_booking/models/Listings_booking_model');
            $current_price = $ci->Listings_booking_model->get_period_price($params['listing'], $date_start, $date_end);
            $ci->view->assign('current_price', $current_price);
        }

        if (!empty($params['short_form'])) {
            $tpl = 'helper_booking_short_block';
        } else {
            $tpl = 'helper_booking_block';
        }

        if (!empty($params['popup_form'])) {
            $ci->view->assign('popup_form', '');
        } else {
            $ci->view->assign('popup_form', '1');
        }
        $ci->view->assign('rand', rand(100000, 999999));
        $ci->view->assign('is_guest', $ci->session->userdata('auth_type') !== 'user');
        return $ci->view->fetch($tpl, 'user', 'listings_booking');
    }

}

if (!function_exists('listings_calendar_block')) {

    /**
     * Calendar of booking periods
     *
     * @param array   $params     block parameters
     * @param boolean $only_block block without tables of periods
     *
     * @return string
     */
    function listings_calendar_block($params, $only_block = false)
    {
        if (empty($params['listing'])) {
            return '';
        }
        $ci = &get_instance();

        $listing = $params['listing'];

        if (!isset($params['count'])) {
            $params['count'] = 1;
        }
        if (empty($params['year'])) {
            $params['year'] = 'Y';
        }
        if (empty($params['month'])) {
            $params['month'] = 'm';
        }
        if (empty($params['template'])) {
            $params['temlate'] = 'view';
        }

        switch ($ci->session->userdata('auth_type')) {
            case 'admin':
                $edit = $ci->router->is_admin_class;
                break;
            case 'user':
                $edit = $listing['id_user'] == $ci->session->userdata('user_id') && !empty($params['edit']);
                break;
            default:
                $edit = false;
                break;
        }

        $ci->load->helper('start');

        if ($listing['price_negotiated']) {
            $price_default = '<span>' . l('text_booking_price_unknown', 'listings') . '</span>';
        } elseif ($listing['price_reduced'] > 0) {
            $price_default = currency_format_output(array('value' => $listing['price_reduced'], 'cur_gid' => $listing['gid_currency']));
        } else {
            $price_default = currency_format_output(array('value' => $listing['price'], 'cur_gid' => $listing['gid_currency']));
        }

        $count = $params['count'];
        $time = strtotime(date($params['year'] . '-' . $params['month'] . '-01'));

        $m = date('m', $time);
        $ci->view->assign('m', $m);

        $y = date('Y', $time);
        $ci->view->assign('y', $y);


        $periods = isset($listing['booking']['periods']) ? $listing['booking']['periods'] : array();

        switch ($listing['price_period']) {
            case Listings_booking_model::PRICE_PERIOD_MONTH:
                $now = strtotime('Y-m-01');
                $years = array();
                for ($j = 0; $j < $count; ++$j) {
                    $year = array();
                    for ($i = 0; $i < 12; ++$i) {
                        $price = $price_default;

                        $time = strtotime(($y + $j) . '-' . ($i + 1) . '-01');

                        $status = 'open';
                        $comment = '';
                        $start = false;
                        $end = false;
                        foreach ($periods as $period) {
                            if ($period['status'] === 'wait' || $period['status'] === 'decline') {
                                continue;
                            }
                            if ($time >= strtotime($period['date_start']) && $time <= strtotime($period['date_end'])) {
                                if ($period['price'] > 0) {
                                    $price = currency_format_output(array('value' => $period['price'] * 1, 'cur_gid' => $listing['gid_currency']));
                                } else {
                                    $price = $price_default;
                                }
                                $status = $period['status'];
                                if (!$edit) {
                                    continue;
                                }
                                if ($time == strtotime($period['date_start'])) {
                                    $comment = $period['comment'];
                                    $start = true;
                                }
                                if ($time == strtotime($period['date_end'])) {
                                    $end = true;
                                }
                                break;
                            }
                        }

                        if ($time < $now) {
                            $status = 'close';
                        }
                        $name = ld_option('month_full', 'start', $i + 1);

                        $year[] = array(
                            'price' => $price,
                            'status' => $status,
                            'month' => $name,
                            'comment' => $comment,
                            'start' => $start,
                            'end' => $end
                        );
                    }
                    $years[$y + $j] = array_chunk($year, 4);
                }
                $ci->view->assign('years', $years);

                $month_names_ds = ld('month_full', 'start');
                $month_names = array_chunk($month_names_ds['option'], 4);
                $ci->view->assign('month_names', $month_names);
                break;
        }
        $ci->view->assign('count', $count);
        $ci->view->assign('listing', $listing);
        $ci->view->assign('edit', $edit);
        $ci->view->assign('rand', rand(100000, 999999));
        $ci->view->assign('is_get_periods', $params['get_periods']);

        if ($only_block) {
            return $ci->view->fetch('calendar_block', null, 'listings_booking');
        }

        if (!empty($listing['gid_currency']) && $ci->pg_module->is_module_active('currencies')) {
            $ci->load->model('Currencies_model');
            $current_price_currency = $ci->Currencies_model->get_currency_by_gid($listing['gid_currency']);
            $params['listing']['currency_data'] = $current_price_currency;
            $ci->view->assign('current_price_currency', $current_price_currency);
        }

        $_SESSION['cur_listing'] = $params['listing'];

        $ci->view->assign('current_lang_id', $ci->pg_language->current_lang_id);
        $ci->view->assign('langs', $ci->pg_language->languages);

        if (isset($params['type'])) {
            $ci->view->assign('calendar_type', $params['type']);
        }

        return $ci->view->fetch('helper_calendar_block', null, 'listings_booking');
    }

    /**
     * Calendar month sort
     *
     * @return array
     */
    function calendar_month_ksort()
    {
        $ci = &get_instance();
        $month_names = ld('month_full', 'start');
        ksort($month_names['option']);
        $ci->view->assign('month_names', $month_names);
    }

}
