<?php

namespace Pg\Modules\Mailbox\Controllers;

/**
 * Mailbox module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Mailbox user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Mailbox
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Mailbox extends \Controller
{
    /**
     * Mailbox owner identifier
     *
     * @var integer
     */
    protected $owner_id = 0;

    /**
     * Constructor
     *
     * @return Mailbox
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Mailbox_model");

        if ($this->session->userdata("auth_type") != "user") {
            show_404();
        }

        $user_type = $this->session->userdata('user_type');

        $this->owner_id = $this->session->userdata('user_id');
        $this->load->model('Menu_model');
        $this->Menu_model->breadcrumbs_set_parent('mailbox_item');
        $this->Menu_model->set_menu_active_item($user_type . '_account_menu', 'mailbox_item');
    }

    /**
     * Manage chats
     *
     * @param integer $folder_id       folder identifier
     * @param string  $view            view mode
     * @param string  $order_key       sorting value
     * @param string  $order_direction order direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function index($folder_id = null, $view = "full", $order_key = 'new', $order_direction = 'ASC', $page = 1)
    {
        $this->chats($folder_id, $order_key, $order_direction, $page);
    }

    /**
     * Manage chats
     *
     * @param integer $folder_id       folder identifier
     * @param string  $order_key       sorting value
     * @param string  $order_direction order direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function chats($folder_id, $order_key = 'new', $order_direction = 'ASC', $page = 1)
    {
        $folder_id = intval($folder_id);
        $data = array(
            'order_key'       => $order_key,
            'order_direction' => $order_direction,
            'page'            => $page,
            'active_folder'   => !empty($folder_id) ? $folder_id : $this->Mailbox_model->default_folder,
        );

        $user_type = $this->session->userdata("user_type");
        $this->Menu_model->breadcrumbs_set_parent($user_type . '_mailbox_item');

		$this->view->assign('folders_block', $this->_folders_block($data['active_folder']));
		$this->view->assign('chats_block', $this->_chats_block($data['active_folder'], $order_key, $order_direction, $page));
		$this->view->assign('data', $data);
		$this->view->render('chats');
    }

    /**
     * Show chat messages
     *
     * @param integer $id_from_user sender identifier
     *
     * @return void
     */
    public function chat($id_from_user)
    {
        if ($this->pg_module->is_module_active('users_services')) {
            $this->load->model('Users_services_model');
            $return = $this->Users_services_model->available_contact_action($this->owner_id, $id_from_user);
            if (!$return["available"]) {
                show_404();

                return;
            }
        }

        $chat_data = $this->Mailbox_model->is_chat_exists($this->owner_id, $id_from_user);
        if ($chat_data === false) {
            $id_chat = $this->Mailbox_model->create_chat($this->owner_id, $id_from_user);
            $id_folder = $this->Mailbox_model->default_folder;
            $chat_data = $this->Mailbox_model->get_chat_by_id($id_chat);
        } else {
            $id_chat = $chat_data["id"];
            $id_folder = $chat_data["id_folder"];
        }

        $data = array(
            'id_folder'     => $id_folder,
            'id_chat'       => $id_chat,
            'active_chat'   => $id_chat,
            'active_folder' => $id_folder,
            "next_btn"      => false,
        );
		$this->view->assign('chat', $chat_data);

        $params["where"]["id_chat"] = $id_chat;

        $data['messages_count'] = $messages_count = $this->Mailbox_model->get_messages_count($params);
        if ($messages_count) {
            $messages = $this->Mailbox_model->get_messages($params, 0, $this->pg_module->get_module_config('mailbox', 'items_per_page'));
			$this->view->assign('messages', $messages);
            if (!empty($messages)) {
                foreach ($messages as $m) {
                    $ids[] = $m["id"];
                }
                $this->Mailbox_model->set_read_messages($id_chat, $ids);

                $date_format = $this->pg_date->get_format('date_time_literal', 'st');
				$this->view->assign('date_format', $date_format);
            }
            if (count($messages) < $messages_count) {
                $data["next_btn"] = true;
            }
        }
		$this->view->assign('data', $data);

        $user_type = $this->session->userdata('user_type');
        $this->Menu_model->breadcrumbs_set_parent($user_type . '_mailbox_item');

        $title = l('breadcrumb_chat_header', 'mailbox', '', 'text',
                    array('name' => $chat_data["user_data"]["output_name"]));
        $this->Menu_model->breadcrumbs_set_active($title);

		$this->view->render('chat');
    }

    /**
     * Manage folders
     *
     * @return void
     */
    public function folders()
    {
		$this->view->assign('folders_block', $this->_folders_list_block());
        $this->Menu_model->breadcrumbs_set_active(l('breadcrumb_folders_header', 'mailbox'));
		$this->view->render('folders');
    }

    /**
     * Block of chat messages
     *
     * @param integer $id_folder       folder identifier
     * @param string  $order_key       sorting value
     * @param string  $order_direction order direction
     * @param integer $page            page of results
     *
     * @return string
     */
    protected function _chats_block($id_folder = null, $order_key = 'new', $order_direction = 'ASC', $page = 1)
    {
        $folder = $this->Mailbox_model->get_folder_by_id($id_folder);
		$this->view->assign('folder', $folder);

        $chat_params["where"]["id_folder"] = $id_folder;
        $chat_params["where"]["id_com_user"] = $this->owner_id;
        $chats_count = $this->Mailbox_model->get_chats_count($chat_params);

        if (!$page) {
            $page = 1;
        }
        $items_on_page = $this->pg_module->get_module_config('mailbox', 'items_per_page');
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $chats_count, $items_on_page);

        $this->Mailbox_model->set_chats_format_settings('get_last_message', true);
        $chats = $this->Mailbox_model->get_chats_list($chat_params, $page, $items_on_page, array($order_key => $order_direction));
        $this->Mailbox_model->set_chats_format_settings('get_last_message', false);
        $this->view->assign('chats', $chats);

        $this->load->helper("navigation");
        $url = site_url() . "mailbox/ajax_view_chats/" . $id_folder . "/" . $order_key . "/" . $order_direction . "/";
        $page_data = get_user_pages_data($url, $chats_count, $items_on_page, $page, 'briefPage');
        $page_data['order'] = $order_key;
        $page_data['order_direction'] = $order_direction;
		$this->view->assign('page_data', $page_data);

		return $this->view->fetch('block_chats');
    }

    /**
     * Block of folder
     *
     * @param integer $id_active_folder active folder identfier
     *
     * @return string
     */
    protected function _folders_block($id_active_folder)
    {
        $folders = $this->Mailbox_model->get_user_folders($this->owner_id);
		$this->view->assign('folders', $folders);
		$this->view->assign('id_active_folder', $id_active_folder);
		return $this->view->fetch('block_folders');
    }

    /**
     * Block of folders list
     *
     * @return string
     */
    protected function _folders_list_block()
    {
        $folders = $this->Mailbox_model->get_user_folders($this->owner_id);
		$this->view->assign('folders', $folders);
        return $this->view->fetch('block_list_folders');
    }

    ///// AJAX actions

    /**
     * Show chats by ajax
     *
     * @param integer $id_folder       folder identifier
     * @param string  $order_key       sorting value
     * @param string  $order_direction order direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function ajax_view_chats($id_folder, $order_key = 'new', $order_direction = 'ASC', $page = 1)
    {
        $this->view->output($this->_chats_block($id_folder, $order_key, $order_direction, $page));
        $this->view->render();
    }

    /**
     * Show folder content by ajax
     *
     * @param integer $id_folder folder identifier
     *
     * @return void
     */
    public function ajax_view_folders($id_folder)
    {
        $this->view->output($this->_folders_block($id_folder));
        $this->view->render();
    }

    /**
     * Show folders by ajax
     *
     * @return void
     */
    public function ajax_view_list_folders()
    {
        $this->view->output($this->_folders_list_block());
        $this->view->render();
    }

    /**
     * Save folder data by ajax
     *
     * @param integer $id   folder identifier (post)
     * @param string  $name folder name (post)
     *
     * @return void
     */
    public function ajax_save_folder()
    {
        $id = intval($this->input->post('id'));
        $id_user = $this->owner_id;
        $post_data["name"] = $this->input->post('name', true);

        $validate_data = $this->Mailbox_model->validate_folder($id, $id_user, $post_data);
        if (!empty($validate_data["errors"])) {
            $return["error"] = implode("<br>", $validate_data["errors"]);
            
            $this->view->assign($return);

            $this->view->render();

            return;
        } else {
            $return = $validate_data["data"];
            $return["folder_type"] = "user";
            $return["id_user"] = $id_user;
            $return["id"] = $id = $this->Mailbox_model->save_folder($id, $return);
            $return["success"] = l('success_folder_save', 'mailbox');
            
            $this->view->assign($return);
            
            $this->view->render();

            return;
        }

        return;
    }

    /**
     * Remove folder data by ajax
     *
     * @param integer $id_folder folder identifier
     *
     * @return void
     */
    public function ajax_delete_folder($id_folder)
    {
        $this->Mailbox_model->delete_folder($id_folder);
        $return["success"] = l('success_folder_delete', 'mailbox');
        
        $this->view->assign($return);
        
        $this->view->render();

        return;
    }

    /**
     * Remove user's chats
     *
     * @param array $chats chats identifiers (post)
     *
     * @return void
     */
    public function ajax_delete_chats()
    {
        $chats = $this->input->post('chats');
        if (!empty($chats)) {
            $chat_params["where_in"]["id"] = $chats;
            $chat_params["where"]["id_com_user"] = $this->owner_id;
            $chat_list = $this->Mailbox_model->get_chats_list($chat_params);

            if (!empty($chat_list)) {
                foreach ($chat_list as $chat) {
                    $this->Mailbox_model->delete_chat($chat["id"]);
                }
            }
            $return["success"] = l('success_chats_delete', 'mailbox');
            
            $this->view->assign($return);
            
            $this->view->render();

            return;
        }

        $return["error"] = l('error_select_chats', 'mailbox');
        
        $this->view->assign($return);
        
        $this->view->render();

        return;
    }

    /**
     * Move chats to other folder by ajax
     *
     * @param integer $id_folder folder identifier (post)
     * @param array   $chats     chats identifiers
     *
     * @return void
     */
    public function ajax_move_chats()
    {
        $id_folder = intval($this->input->post('id_folder'));
        $chats = $this->input->post('chats');
        if (!empty($chats)) {
            $chat_params["where_in"]["id"] = $chats;
            $chat_params["where"]["id_com_user"] = $this->owner_id;
            $chat_list = $this->Mailbox_model->get_chats_list($chat_params);

            if (!empty($chat_list)) {
                foreach ($chat_list as $chat) {
                    $id_chat[] = $chat["id"];
                }
                $this->Mailbox_model->move_chat($id_chat, $id_folder);
            }
            $return["success"] = l('success_chats_moved', 'mailbox');
            
            $this->view->assign($return);
            
            $this->view->render();

            return;
        }

        $return["error"] = l('error_select_chats', 'mailbox');
        
        $this->view->assign($return);
        
        $this->view->render();

        return;
    }

    /**
     * Send message by ajax
     *
     * @param integer $id_chat chat identifier (post)
     * @param string  $message message text (post)
     *
     * @return void
     */
    public function ajax_send_message()
    {
        $id_chat = intval($this->input->post('id_chat', true));
        $post_data["message"] = $this->input->post('message', true);
        if (!$id_chat) {
            $this->view->render();

            return;
        }

        $validate_data = $this->Mailbox_model->validate_message($post_data);
        if (!empty($validate_data["errors"])) {
            $return["error"] = implode("<br>", $validate_data["errors"]);
            
            $this->view->assign($return);
            
            $this->view->render();

            return;
        } else {
            $this->Mailbox_model->create_message($id_chat, $validate_data["data"]["message"], "outbox", 1);

            /// get friend user chat and put message on it
            $chat = $this->Mailbox_model->get_chat_by_id($id_chat);
            $friend_chat = $this->Mailbox_model->is_chat_exists($chat["id_from_user"], $chat["id_com_user"]);
            if ($friend_chat === false) {
                $id_friend_chat = $this->Mailbox_model->create_chat($chat["id_from_user"], $chat["id_com_user"]);
            } else {
                $id_friend_chat = $friend_chat["id"];
            }
            $this->Mailbox_model->create_message($id_friend_chat, $validate_data["data"]["message"], "inbox", 1);

            $return["success"] = l('success_message_save', 'mailbox');
            
            $this->view->assign($return);
            
            $this->view->render();

            return;
        }

        return;
    }

    /**
     * Get chat messages by ajax
     *
     * @param integer $id_chat chat identifier
     * @param boolean $new     only new messages
     * @param integer $offset  message offset
     *
     * @return void
     */
    public function ajax_get_messages($id_chat, $new = true, $offset = 0)
    {
        $chat = $this->Mailbox_model->get_chat_by_id($id_chat);
        if ($this->pg_module->is_module_active('users_services')) {
            $this->load->model('Users_services_model');
            $return = $this->Users_services_model->available_contact_action($chat['id_com_user'], $chat['id_from_user']);
            if (!$return["available"]) {
                return '';
            }
        }

        $params["where"]["id_chat"] = $id_chat;
        if ($new) {
            $params["where"]["is_new"] = 1;
            $offset = $limit = 0;
            $messages_count = 0;
        } else {
            $offset = intval($offset);
            $limit = $this->pg_module->get_module_config('mailbox', 'items_per_page');
            $return['messages_count'] = $messages_count = $this->Mailbox_model->get_messages_count($params);
        }

        $messages = $this->Mailbox_model->get_messages($params, $offset, $limit);

		$this->view->assign('messages', $messages);

        $date_format = $this->pg_date->get_format('date_time_literal', 'st');
		$this->view->assign('date_format', $date_format);

		$return["html"] = $this->view->fetch('block_messages_item');
        $return["next_btn"] = false;

        if ($new && !empty($messages)) {
            foreach ($messages as $m) {
                $ids[] = $m["id"];
            }
            $this->Mailbox_model->set_read_messages($id_chat, $ids);
        } else {
            if ($messages_count > $offset + count($messages)) {
                $return["next_btn"] = true;
            }
        }

        $this->view->assign($return);
        
        $this->view->render();

        return;
    }

    /**
     * Remove message by ajax
     *
     * @param integer $id_message message identifier
     *
     * @return void
     */
    public function ajax_delete_message($id_message)
    {
        $message = $this->Mailbox_model->get_message_by_id($id_message);
        $id_chat = $message["id_chat"];
        $chat = $this->Mailbox_model->get_chat_by_id($id_chat);

        if ($chat["id_com_user"] != $this->owner_id) {
            $return["error"] = l('error_not_permissions_delete_message', 'mailbox');
            
            $this->view->assign($return);
            
            $this->view->render();

            return;
        } else {
            $this->Mailbox_model->delete_message($id_message);

            $return["success"] = l('message_successfully_deleted', 'mailbox');
            
            $this->view->assign($return);
            
            $this->view->render();

            return;
        }
    }

    /**
     * Get folder form by ajax
     *
     * @param integer $id_folder folder identifier
     *
     * @return void
     */
    public function ajax_form_folder($id_folder = 0)
    {
        $id_folder = intval($id_folder);
        if ($id_folder) {
            $folder = $this->Mailbox_model->get_folder_by_id($id_folder);
			$this->view->assign('folder', $folder);
        }

		$this->view->render('ajax_edit_folder');
    }

    /**
     * Get moving folder form by ajax
     *
     * @return void
     */
    public function ajax_open_move_folder()
    {
        $folders = $this->Mailbox_model->get_user_folders($this->owner_id);
		$this->view->assign('folders', $folders);

		$this->view->render('ajax_move_folder');
    }
}
