<?php

namespace Pg\Modules\Mobile\Controllers;

/**
 * Mobile module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Mobile admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Mobile
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Mobile extends \Controller
{

    /**
     * Controller
     *
     * @return Admin_Mobile
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'add_ons_items');
    }

    /**
     * Manage module settings
     *
     * @return void
     */
    public function index()
    {
        $this->settings();
    }

    /**
     * Manage module settings
     *
     * @return void
     */
    public function settings()
    {
        if ($this->input->post('btn_save')) {
            $this->load->model('Mobile_model');

            $post_data['index_type_block'] = $this->input->post('index_type_block', true);
            $post_data['index_view_block'] = $this->input->post('index_view_block', true);
            $post_data['index_count_block'] = $this->input->post('index_count_block', true);
            $post_data['geomap_driver'] = $this->input->post('geomap_driver', true);

            $validate_data = $this->Mobile_model->validate_settings($post_data);
            if (!empty($validate_data['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                $data = $validate_data['data'];
            } else {
                foreach ($validate_data['data'] as $setting => $value) {
                    $this->pg_module->set_module_config('mobile', $setting, $value);
                }
                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_settings_saved', 'mobile'));
                $data = $validate_data['data'];
            }
        } else {
            $data['index_type_block'] = $this->pg_module->get_module_config('mobile', 'index_type_block');
            $data['index_view_block'] = $this->pg_module->get_module_config('mobile', 'index_view_block');
            $data['index_count_block'] = $this->pg_module->get_module_config('mobile', 'index_count_block');
            $data['geomap_driver'] = $this->pg_module->get_module_config('mobile', 'geomap_driver');
        }
        $this->view->assign('data', $data);

        $this->view->setHeader(l('admin_header_settings', 'mobile'));

        $this->Menu_model->set_menu_active_item('admin_mobile_menu', 'mobile_settings_item');

        $this->view->render('settings');
    }

    /**
     * Manage applications settings
     *
     * @return void
     */
    public function apps()
    {
        $this->load->model('Mobile_model');

        if ($this->input->post('btn_save')) {
            $post_data['download_apps_used'] = $this->input->post('download_apps_used', true);
            $post_data['download_apps_android'] = $this->input->post('download_apps_android', true);
            $post_data['download_apps_ios'] = $this->input->post('download_apps_ios', true);

            foreach ($this->pg_language->languages as $lang_id => $lang_data) {
                $post_data['download_apps_title'] = $this->input->post('download_apps_title', true);
                $post_data['download_apps_text'] = $this->input->post('download_apps_text', true);
            }

            $validate_data = $this->Mobile_model->validate_settings($post_data);
            $validate_upload = $this->Mobile_model->validate_download_apps_background('download_apps_background');
            $validate_upload['errors'] = !empty($validate_upload['error']) ? $validate_upload['error'] : array();
            $validate_data['errors'] = array_merge($validate_data['errors'], $validate_upload['errors']);
            if (!empty($validate_data['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                $data = array_merge($validate_data['data'], $validate_data['langs']);
            } else {
                $download_apps_background_delete = $this->input->post('download_apps_background_delete', true);
                if ($download_apps_background_delete) {
                    $this->Mobile_model->delete_download_apps_background();
                }

                foreach ($validate_data['data'] as $setting => $value) {
                    $this->pg_module->set_module_config('mobile', $setting, $value);
                }

                foreach ($validate_data['langs'] as $setting => $value) {
                    foreach ($value as $k => $v) {
                        $this->pg_language->pages->set_string('mobile', $setting, $v, $k);
                    }
                }

                $data = array_merge($validate_data['data'], $validate_data['langs']);

                $result = $this->Mobile_model->upload_download_apps_background('download_apps_background');
                if (!empty($result['file'])) {
                    $data['download_apps_background'] = $result['file'];
                } else {
                    $data['download_apps_background'] = $this->pg_module->get_module_config('mobile', 'download_apps_background');
                }

                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_download_apps_saved', 'mobile'));
            }
        } else {
            $data['download_apps_used'] = $this->pg_module->get_module_config('mobile', 'download_apps_used');
            $data['download_apps_android'] = $this->pg_module->get_module_config('mobile', 'download_apps_android');
            $data['download_apps_ios'] = $this->pg_module->get_module_config('mobile', 'download_apps_ios');

            foreach ($this->pg_language->languages as $lang_id => $lang_data) {
                $data['download_apps_title'][$lang_id] = $this->pg_language->pages->get_string('mobile', 'download_apps_title', $lang_id);
                $data['download_apps_text'][$lang_id] = $this->pg_language->pages->get_string('mobile', 'download_apps_text', $lang_id);
            }

            $data['download_apps_background'] = $this->pg_module->get_module_config('mobile', 'download_apps_background');
        }

        $data = $this->Mobile_model->format_settings($data);

        $this->view->assign('data', $data);

        $current_lang_id = $this->pg_language->current_lang_id;
        $this->view->assign('current_lang_id', $current_lang_id);
        $this->view->assign('languages', $this->pg_language->languages);

        $this->view->setHeader(l('admin_header_download_apps', 'mobile'));
        $this->view->render('apps');
    }

}
