<?php

namespace Pg\Modules\Moderation\Models;

/**
 * Moderation module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define('MODERATION_TYPE_TABLE', DB_PREFIX . 'moderation_type');

/**
 * Moderation type Model
 *
 * ТИП МОДЕРИРУЕМОГО ОБЪЕКТА:
 * 1. название,
 * 2. настройка(как модерировать),
 *     три типа модерации:
 *       премодерация (2) - при добавлении статус ставится 0 затем уже только уведомления что контент менялся,
 *       постмодерация (1) - добавляется сразу, но при каждом изменении уходит уведомление об изменении,
 *       без модерации (0) - тип заведен, но возможно нужно отключить модерацию вообще
 *       без модерации, и снаружи не видно что есть тип (-1) - тип заведен как в случае с юзерами только для общего хранения настройки проверки badwords
 * 3. название модели/модуля(video) - как название папки модуля , инклудим шаблон используя эту строку
 * 4. путь к модели (video/models/Video_model - чтобы подключать не только дефолтные модели)
 * 5. view_link, edit_link - для связей с просмотром и редактированием (скидываем эти действия на модули соотв объектов, за это модерация не отвечает)
 * 6. метод получения списка объектов (по id) (ids) (модели, которую завели выше)  --- ничего не знаем о тех данных,
 *     которые есть в объекте, поэтому спрашиваем модель, должен возвращать массив где ключи - id объектов
 * 7. шаблон строки в списке (лежит в папке модуля объекта) --- нет общего шаблона для отображения preview разных объектов (так чтобы можно было апрувить
 *     без просмотра например, какие данные для этого нужны известно опять-таки только на уровне модели объекта), assign в data
 * 8. метод установки статуса (id, status) - чтобы апрувить
 * 9. метод удаления объекта (id) - чтобы удалить:)
 * 10. check_badwords- проверять или нет бэдвордс, функция все равно одна для всех типов, туда передается только текст, но она проверяет нужно делать проверку или нет
 *
 * TODO: рассылать или нет оповещения админу, по каждому конкретному типу
 *
 * @package PG_RealEstate
 * @subpackage Moderation
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Moderation_type_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Moderation types cache
     *
     * @var array
     */
    public $types_cache;

    /**
     * Constructor
     *
     * @return Moderation_type_model
     */
    public function __construct()
    {
        parent::__construct();

        $this->ci = &get_instance();
        $this->types_cache["id"] = $this->types_cache["name"] = array();
    }

    /**
     * Return type object by identifier
     *
     * @param integer $type_id type identifier
     *
     * @return array
     */
    public function get_type_by_id($type_id)
    {
        if (!isset($this->types_cache["id"][$type_id]) || empty($this->types_cache["id"][$type_id])) {
            $type_id = intval($type_id);
            if (!$type_id) {
                return false;
            }

            $this->ci->db->select('id, name, mtype, view_link, edit_link, module, model, method_get_list, method_set_status, allow_to_decline, method_delete_object, template_list_row, date_add, check_badwords')->from(MODERATION_TYPE_TABLE)->where('id',
                                                                                                                                                                                                                                                       $type_id);

            //_compile_select;
            $result = $this->ci->db->get()->result();
            if (!empty($result)) {
                $rt = get_object_vars($result[0]);
                $this->types_cache["id"][$rt["id"]] = $rt;
                $this->types_cache["name"][$rt["name"]] = $rt;
            } else {
                $this->types_cache["id"][$type_id] = array();
            }
        }

        return $this->types_cache["id"][$type_id];
    }

    /**
     * Return type object by GUID
     *
     * @param string $name type GUID
     *
     * @return array
     */
    public function get_type_by_name($name)
    {
        if (!isset($this->types_cache["name"][$name]) || empty($this->types_cache["name"][$name])) {
            $this->ci->db->select('id, name, mtype, view_link, edit_link, module, model, method_get_list, method_set_status, allow_to_decline, method_delete_object, template_list_row, date_add, check_badwords')->from(MODERATION_TYPE_TABLE)->where('name',
                                                                                                                                                                                                                                                       $name);

            //_compile_select;
            $result = $this->ci->db->get()->result();
            if (!empty($result)) {
                $rt = get_object_vars($result[0]);
                $this->types_cache["name"][$rt["name"]] = $rt;
                $this->types_cache["id"][$rt["id"]] = $rt;
            } else {
                $this->types_cache["name"][$name] = array();
            }
        }

        return $this->types_cache["name"][$name];
    }

    /**
     * Return types objects as array
     *
     * @return array/false
     */
    public function get_types()
    {
        $this->ci->db->select('id, name, mtype, view_link, edit_link, module, model, method_get_list, method_set_status, allow_to_decline, method_delete_object, template_list_row, date_add, check_badwords')->from(MODERATION_TYPE_TABLE)->order_by('id ASC');
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            foreach ($result as $res_obj) {
                $rt = get_object_vars($res_obj);
                $this->types_cache["name"][$rt["name"]] = $rt;
                $this->types_cache["id"][$rt["id"]] = $rt;
                $res[] = $rt;
            }

            return $res;
        } else {
            return false;
        }
    }

    /**
     * Save type object to data source
     *
     * @param integer $type_id type identifier
     * @param array   $attrs   type data
     * @param array   $langs   languages data
     *
     * @return integer
     */
    public function save_type($type_id, $attrs, $langs = array())
    {
        if (!is_array($attrs) || !count($attrs)) {
            return false;
        }
        $type_id = intval($type_id);
        if (!$type_id) {
            $attrs["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(MODERATION_TYPE_TABLE, $attrs);
            $insId = $this->ci->db->query('SELECT LAST_INSERT_ID() as last_id')->result();
            $type_id = $insId[0]->last_id;
        } else {
            $this->ci->db->where('id', $type_id);
            $this->ci->db->update(MODERATION_TYPE_TABLE, $attrs);
        }

        if (!empty($langs) && !empty($attrs["name"])) {
            $lang_ids = array_keys($this->ci->pg_language->languages);
            $this->ci->pg_language->pages->set_string_langs('moderation', "mtype_" . $attrs["name"], $langs, $lang_ids);
        }

        return $type_id;
    }

    /**
     * Remove type object by identifier
     *
     * @param integer $type_id type identifier
     *
     * @return void
     */
    public function delete_type($type_id)
    {
        $type_id = intval($type_id);
        if (!$type_id) {
            return false;
        }
        $type_data = $this->get_type_by_id($type_id);

        $this->ci->db->delete(MODERATION_TYPE_TABLE, array('id' => $type_id));
        unset($this->types_cache["id"][$type_id]);

        $this->ci->load->model('Moderation_model');
        $this->ci->Moderation_model->delete_moderation_items_by_type_id($type_id);

        $this->ci->pg_language->pages->delete_string('moderation', "mtype_" . $type_data["name"]);

        return;
    }

    /**
     * Import languages of type objects
     *
     * @param array $data       types data
     * @param array $langs_file languages data
     *
     * @return void
     */
    public function update_langs($data, $langs_file)
    {
        foreach ($data as $mtype) {
            $this->ci->pg_language->pages->set_string_langs(
                    'moderation', 'mtype_' . $mtype['name'], $langs_file['mtype_' . $mtype['name']],
                    array_keys($langs_file['mtype_' . $mtype['name']])
            );
        }
    }

    /**
     * Export languages of type objects
     *
     * @param array $data      types data
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function export_langs($data, $langs_ids)
    {
        $gids = array();
        foreach ($data as $mtype) {
            $gids[] = 'mtype_' . $mtype['name'];
        }

        return $this->ci->pg_language->export_langs('moderation', $gids, $langs_ids);
    }

}
