<?php

namespace Pg\Modules\Social_networking\Models\Widgets;

/**
 * Social networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Social networking facebook widgets model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Social networking
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Facebook_widgets_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Available widgets
     *
     * @var array
     */
    public $widget_types = array(
        'comments',
        'like',
        'share',
    );

    /**
     * Current page url
     *
     * @var string
     */
    public $url;

    /**
     * Page header data is loaded
     *
     * @var boolean
     */
    public $head_loaded = false;

    /**
     * Class constructor
     *
     * @return Facebook_widgets_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https://' : 'http://';
        $this->url .= ($_SERVER['SERVER_PORT'] != '80') ? $_SERVER['SERVER_NAME'] . ':' . $_SERVER['SERVER_PORT'] . $_SERVER['REQUEST_URI'] : $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI'];
    }

    /**
     * Return widgets data for page header
     *
     * @param array  $service_data service data
     * @param string $locale       locale code
     * @param array  $types        widget types
     *
     * @return string
     */
    public function get_header($service_data = array(), $locale = 'en_US', $types = array())
    {
        $header = '';
        $appid = isset($service_data['app_key']) ? $service_data['app_key'] : false;
        $header = $appid ? ('
			<meta property="fb:app_id" content="' . $appid . '"/>
			<div id="fb-root"></div>
			<script>(function(d, s, id) {
				var js, fjs = d.getElementsByTagName(s)[0];
				if (d.getElementById(id)) return;
				js = d.createElement(s); js.id = id;
				js.src = "//connect.facebook.net/' . $locale . '/sdk.js#xfbml=1&appId=' . $appid . '&version=v2.0";
				fjs.parentNode.insertBefore(js, fjs);
			}(document, \'script\', \'facebook-jssdk\'));</script>
		') : '';
        $this->head_loaded = $header ? true : false;

        return $header;
    }

    /**
     * Return code of 'Like' widget
     *
     * @return string
     */
    public function get_like()
    {
        if ($this->head_loaded) {
            return '<div class="fb-like" data-href="' . urlencode($this->url) . '" data-send="false" data-layout="button_count" data-width="100" data-show-faces="true" data-font="segoe ui"></div>';
        } else {
            return '';
        }
    }

    /**
     * Return code of 'Share' widget
     *
     * @return string
     */
    public function get_share()
    {
        return $this->head_loaded ? '<div class="fb-send" data-href="' . urlencode($this->url) . '"></div>' : '';
    }

    /**
     * Return code of 'Comment' widget
     *
     * @return string
     */
    public function get_comments()
    {
        return $this->head_loaded ? '<div class="fb-comments" data-href="' . urlencode($this->url) . '" data-num-posts="2" data-width="470"></div>' : '';
    }

}
