<?php

namespace Pg\Modules\Upload_gallery\Controllers;

/**
 * Upload gallery module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Upload gallery api side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Upload gallery
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Upload_gallery extends \Controller
{

    /**
     * Constructor
     *
     * @return Upload_gallery
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Upload_gallery_model");
        $this->load->model('Listings_model');
        $this->Listings_model->set_format_settings('get_price_str', false);
    }

    /**
     * Upload photo to gallery
     *
     * @param integer $gallery_type_id gallery type identifier
     * @param integer $object_id       object identifier
     * @param integer $photo_id        photo identifier
     *
     * @return void
     */
    public function photo_upload($gallery_type_id)
    {
        $object_id = intval($this->input->post('object_id', true));

        $gallery_type_id = intval($gallery_type_id);
        if (!$gallery_type_id) {
            $this->set_api_content('error', l('error_not_access', 'upload_gallery'));
            exit;
        }

        $object_id = intval($this->input->post('object_id', true));
        if (!$object_id) {
            $this->set_api_content('error', l('error_not_access', 'upload_gallery'));
            exit;
        }

        $photo_id = intval($this->input->post('photo_id', true));

        $type_data = $this->Upload_gallery_model->get_type_by_id($gallery_type_id);        
        if ($type_data["module"] && $type_data["model"] && $type_data["callback"]) {
            if (strtolower($type_data["module"] . '_model') != strtolower($type_data["model"])) {
                $model_path = $type_data["module"] . "/models/" . $type_data["model"];
            } else {
                $model_path = $type_data["model"];
            }
            try {
                $this->load->model($model_path);
                $is_access = $this->{$type_data["model"]}->{$type_data["callback"]}("check_access", $object_id);
                if (!$is_access) {
                    $this->set_api_content('error', l('error_not_access', 'upload_gallery'));
                    exit;
                }
            } catch (Exception $e) {
                $this->set_api_content('error', l('error_not_access', 'upload_gallery'));
                exit;
            }
        } else {
            $this->set_api_content('error', l('error_not_access', 'upload_gallery'));
            exit;
        }

        if ($photo_id) {
            $photo = $this->Upload_gallery_model->get_file_by_id($photo_id);
            if ($object_id != $photo['object_id']) {
                $this->set_api_content('error', l('error_not_access', 'upload_gallery'));
                exit;
            }
        }

        $data = array('type_id' => $type_data['id'], 'object_id' => $object_id);
        
        $return = $this->Upload_gallery_model->save_file($photo_id, $data, 'photo_file', true);
        
        if (empty($return['errors'])) {
            if (strtolower($type_data["module"] . '_model') != strtolower($type_data["model"])) {
                $model_path = $type_data["module"] . "/models/" . $type_data["model"];
            } else {
                $model_path = $type_data["model"];
            }
            try {
                $this->load->model($model_path);                
                $this->{$type_data["model"]}->{$type_data["callback"]}("upload_file", $object_id);                
            } catch (Exception $e) {                
                $this->set_api_content('error', l('error_not_access', 'upload_gallery'));
                exit;
            }
        }

        if (!empty($return['errors'])) {
            $this->set_api_content('error', $return['errors']);
        } else {
            unset($return['errors']);
            $this->set_api_content('data', $return);
        }
    }

}
