<?php

namespace Pg\Modules\Upload_gallery\Models;

/**
 * Upload gallery module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define("GALLERY_TYPES_TABLE", DB_PREFIX . "gallery_types");
define("GALLERY_TABLE", DB_PREFIX . "gallery");

/**
 * Upload gallery Model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Upload gallery
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Upload_gallery_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Gallery types cache
     *
     * @var array
     */
    protected $type_cache = array();

    /**
     * Gallery type properties in data source
     *
     * @var array
     */
    protected $type_attrs = array(
        "id",
        "gid",
        "name",
        "date_add",
        "gid_upload_config",
        "max_items_count",
        "use_moderation",
        "module",
        "model",
        "callback",
    );

    /**
     * Gallery upload properties in data source
     *
     * @var array
     */
    protected $file_attrs = array(
        "id",
        "type_id",
        "object_id",
        "date_add",
        "date_update",
        "status",
        "comment",
        "file_name",
        "settings",
        "sorter",
    );

    /**
     * Moderation object of gallery object
     *
     * @var string
     */
    protected $moderation_type = "upload_gallery";

    /**
     * Temporal path
     *
     * @var string
     */
    protected $temp_path = 'upload_gallery';

    /**
     * Constructor
     *
     * @return Upload_gallery_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();

        $this->temp_path = TEMPPATH . $this->temp_path;
    }

    /**
     * Return type data by identifier
     *
     * @param integer $type_id type identifier
     *
     * @return array
     */
    public function get_type_by_id($type_id)
    {
        if (!$type_id) {
            return false;
        }

        if (!isset($this->type_cache["id"][$type_id]) || empty($this->type_cache["id"][$type_id])) {
            $type_id = intval($type_id);
            $this->ci->db->select(implode(", ", $this->type_attrs))->from(GALLERY_TYPES_TABLE)->where("id", $type_id);
            $result = $this->ci->db->get()->result_array();
            if (!empty($result)) {
                $rt = $result[0];
                $this->type_cache["id"][$rt["id"]] = $rt;
                $this->type_cache["gid"][$rt["gid"]] = $rt;
            } else {
                $this->type_cache["id"][$type_id] = array();
            }
        }

        return $this->type_cache["id"][$type_id];
    }

    /**
     * Return type data by guid
     *
     * @param string $type_gid type guid
     *
     * @return array
     */
    public function get_type_by_gid($type_gid)
    {
        if (!$type_gid) {
            return false;
        }

        if (!isset($this->type_cache["gid"][$type_gid]) || empty($this->type_cache["gid"][$type_gid])) {
            $this->ci->db->select(implode(", ", $this->type_attrs))->from(GALLERY_TYPES_TABLE)->where("gid", $type_gid);
            $result = $this->ci->db->get()->result_array();
            if (!empty($result)) {
                $rt = $result[0];
                $this->type_cache["id"][$rt["id"]] = $rt;
                $this->type_cache["gid"][$rt["gid"]] = $rt;
            } else {
                $this->type_cache["gid"][$type_gid] = array();
            }
        }

        return $this->type_cache["gid"][$type_gid];
    }

    /**
     * Return types objects as array
     *
     * @return array
     */
    public function get_types_list()
    {
        $this->ci->db->select(implode(", ", $this->type_attrs))->from(GALLERY_TYPES_TABLE)->order_by("id ASC");
        $result = $this->ci->db->get()->result_array();
        if (!empty($result)) {
            foreach ($result as $rt) {
                $this->type_cache["gid"][$rt["gid"]] = $rt;
                $this->type_cache["id"][$rt["id"]] = $rt;
                $res[] = $rt;
            }

            return $res;
        } else {
            return array();
        }
    }

    /**
     * Return number of types objects
     *
     * @return integer
     */
    public function get_types_count()
    {
        $this->ci->db->select("COUNT(*) AS cnt")->from(GALLERY_TYPES_TABLE);
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            return intval($result[0]->cnt);
        } else {
            return 0;
        }
    }

    /**
     * Validate type object for saving to data source
     *
     * @param integer $type_id type identifier
     * @param array   $data    type data
     *
     * @return array
     */
    public function validate_type($type_id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["gid"])) {
            $return["data"]["gid"] = strtolower(preg_replace("/[\s\n\r_]{1,}/", "-",
                                                             trim(preg_replace("/[^a-z_0-9\-]/i", "-",
                                                                               strip_tags($data["gid"])))));
            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l("error_gid_empty", "upload_gallery");
            }
        }

        if (isset($data["name"])) {
            $return["data"]["name"] = strip_tags($data["name"]);
            if (empty($return["data"]["name"])) {
                $return["errors"][] = l("error_name_empty", "upload_gallery");
            }
        }

        if (isset($data["gid_upload_config"])) {
            $return["data"]["gid_upload_config"] = strval($data["gid_upload_config"]);
        }
        if (isset($data["max_items_count"])) {
            $return["data"]["max_items_count"] = intval($data["max_items_count"]);
        }
        if (isset($data["use_moderation"])) {
            $return["data"]["use_moderation"] = intval($data["use_moderation"]);
        }

        if (isset($data["module"])) {
            $return["data"]["module"] = $data["module"];
        }
        if (isset($data["model"])) {
            $return["data"]["model"] = $data["model"];
        }
        if (isset($data["callback"])) {
            $return["data"]["callback"] = $data["callback"];
        }

        return $return;
    }

    /**
     * Save type object to data source
     *
     * @param integer $type_id type identifier
     * @param array   $attrs   type data
     *
     * @return integer
     */
    public function set_type($type_id, $attrs)
    {
        if (!is_array($attrs) || !count($attrs)) {
            return false;
        }
        $type_id = intval($type_id);
        if (!$type_id) {
            $attrs["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(GALLERY_TYPES_TABLE, $attrs);
            $type_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where("id", $type_id);
            $this->ci->db->update(GALLERY_TYPES_TABLE, $attrs);
        }

        return $type_id;
    }

    /**
     * Remove type data from data source by identifier
     *
     * @param integer $type_id type idnetifier
     *
     * @return void
     */
    public function delete_type($type_id)
    {
        $this->ci->db->where("id", $type_id);
        $this->ci->db->delete(GALLERY_TYPES_TABLE);
    }

    /**
     * Return file data by identifier
     *
     * @param integer $id file identifier
     *
     * @return array
     */
    public function get_file_by_id($id)
    {
        $this->ci->db->select(implode(", ", $this->file_attrs))->from(GALLERY_TABLE)->where("id", $id);
        $result = $this->ci->db->get()->result_array();
        if (!empty($result)) {
            $return = $this->format_file($result[0]);
        } else {
            $return = array();
        }

        return $return;
    }

    /**
     * Return file cover
     *
     * @param integer $count number of file
     *
     * @return void
     */
    public function get_cover($count = 1)
    {
        
    }

    /**
     * Return file cover list
     *
     * @param integer $count number of file
     *
     * @return void
     */
    public function get_covers($count = 1)
    {
        
    }

    /**
     * Return files objects as array
     *
     * @param array   $params    sql criteria of query to data source
     * @param string  $limit     items limit
     * @param boolean $formatted format data
     *
     * @return array
     */
    public function get_files_by_param($params, $limit = false, $formatted = true)
    {
        $this->ci->db->select(implode(", ", $this->file_attrs));
        $this->ci->db->from(GALLERY_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }
        $this->ci->db->order_by("sorter ASC");

        if (!empty($limit)) {
            $this->ci->db->limit($limit, 0);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = ($formatted) ? $this->format_file($r) : $r;
            }

            return $data;
        }

        return array();
    }

    /**
     * Return number of files objects
     *
     * @param array $params sql criteria of query to data source
     *
     * @return integer
     */
    public function get_files_by_param_count($params)
    {
        $this->ci->db->select("COUNT(*) AS cnt");
        $this->ci->db->from(GALLERY_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }
        $this->ci->db->order_by("sorter ASC");

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Format file data
     *
     * @param array $data file data
     *
     * @return array
     */
    public function format_file($data)
    {
        $data["type_data"] = $type_data = $this->get_type_by_id($data["type_id"]);

        if (!empty($data["id"])) {
            $data["postfix"] = $type_data["gid"] . "/" . $data["object_id"] . "/" . $data["id"];
        }

        $this->ci->load->model("Uploads_model");
        if (!empty($data["file_name"])) {
            $data["media"] = $this->ci->Uploads_model->format_upload($type_data["gid_upload_config"], $data["postfix"],
                                                                     $data["file_name"]);
        } else {
            $data["media"] = $this->ci->Uploads_model->format_default_upload($type_data["gid_upload_config"]);
        }

        $data["settings"] = $data["settings"] ? (array) unserialize($data["settings"]) : array();

        return $data;
    }

    /**
     * Format files data
     *
     * @return void
     */
    public function format_files()
    {
        
    }

    /**
     * Validate file data for saving to data source
     *
     * @param integer $id      file identifier
     * @param integer $type_id type identifier
     * @param array   $data    file data
     *
     * @return array
     */
    public function validate_file_data($id, $type_id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["comment"])) {
            $return["data"]["comment"] = strip_tags($data["comment"]);

            $type_data = $this->get_type_by_id($type_id);
            if ($type_data["use_moderation"] && !empty($return["data"]["comment"])) {
                if ($this->ci->pg_module->is_module_active("moderation")) {
                    $this->ci->load->model("moderation/models/Moderation_badwords_model");
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type,
                                                                                     $return["data"]["comment"]);
                    if ($bw_count) {
                        $return["errors"][] = l("error_badwords_comment", "upload_gallery");
                    }
                }
            }
        }

        if (isset($data["settings"])) {
            $return["data"]["settings"] = serialize($data["settings"]);
        }

        return $return;
    }

    /**
     * Save file object to data source
     *
     * @param integer $id             file idnetifier
     * @param array   $data           file data
     * @param string  $upload_name    upload name
     * @param boolean $moderation_use need moderation
     *
     * @return array
     */
    public function save_file($id, $data, $upload_name = "", $moderation_use = true)
    {
        $moderation_use = $moderation_use && $this->ci->pg_module->is_module_active("moderation");
        if ($moderation_use) {
            $this->ci->load->model("Moderation_model");
        }

        $return = array("id" => $id, "errors" => array());
        $id = intval($id);
        $flag_new_photo = ($id) ? false : true;

        ///// save text data
        if (!$id) {
            $sorter_param["where"]["type_id"] = $data["type_id"];
            $sorter_param["where"]["object_id"] = $data["object_id"];
            $data["sorter"] = $this->get_files_by_param_count($sorter_param) + 1;

            $data["status"] = 1;
            if ($data["type_id"]) {
                $type_data = $this->get_type_by_id($data["type_id"]);
                if ($type_data['max_items_count']) {
                    if ($data["sorter"] > $type_data['max_items_count']) {
                        $return['errors'][] = l('error_max_items_count', 'upload_gallery');

                        return $return;
                    }
                }

                if ($moderation_use && $type_data["use_moderation"]) {
                    $data["status"] = intval($this->ci->Moderation_model->get_moderation_type_status($this->moderation_type));
                }
            }

            $data["date_update"] = $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(GALLERY_TABLE, $data);
            $return["id"] = $id = $this->ci->db->insert_id();
        } else {
            $data["date_update"] = date("Y-m-d H:i:s");
            $this->ci->db->where("id", $id);
            $this->ci->db->update(GALLERY_TABLE, $data);
        }

        ////// add moderation alert if type use moderation
        if ($moderation_use) {
            $data = $this->get_file_by_id($id);
            $type_data = $this->get_type_by_id($data["type_id"]);
            if ($type_data["use_moderation"]) {
                $this->ci->Moderation_model->add_moderation_item($this->moderation_type, $id);
            }
        }

        ////// upload file
        if (!empty($upload_name) && isset($_FILES[$upload_name]) &&
                is_array($_FILES[$upload_name]) && is_uploaded_file($_FILES[$upload_name]["tmp_name"])) {
            $data = $this->get_file_by_id($id);
            $type_data = $this->get_type_by_id($data["type_id"]);

            $this->ci->load->model("Uploads_model");
            $img_return = $this->ci->Uploads_model->upload($type_data["gid_upload_config"], $data["postfix"],
                                                           $upload_name);
            if (!empty($img_return["errors"])) {
                if ($flag_new_photo) {
                    $this->delete_file($id);
                }
                $return["errors"] = $img_return["errors"];
            } else {
                $upload = $this->ci->Uploads_model->format_upload($type_data["gid_upload_config"], $data["postfix"],
                                                                  $img_return["file"]);
                $image_size = getimagesize($upload["file_path"]);
                $image_data["file_name"] = $img_return["file"];
                if (isset($image_data["settings"])) {
                    $image_data["settings"] = unserialize($image_data["settings"]);
                } else {
                    $image_data["settings"] = array();
                }
                $image_data["settings"]["width"] = $image_size[0];
                $image_data["settings"]["height"] = $image_size[1];
                $image_data["settings"] = serialize($image_data["settings"]);
                $this->save_file($id, $image_data, "", $moderation_use);
            }
        } else {
            if ($flag_new_photo) {
                $this->delete_file($id);
            }
        }

        return $return;
    }

    /**
     * Save local file object to data source
     *
     * @param integer $id             photo identifier
     * @param array   $data           photo data
     * @param string  $file_path      upload path
     * @param boolean $moderation_use need mnoderation
     *
     * @return array
     */
    public function save_local_file($id, $data, $file_path = "", $moderation_use = true)
    {
        $return = array("id" => $id, "errors" => array());

        $moderation_use = $moderation_use && $this->ci->pg_module->is_module_active("moderation");
        if ($moderation_use) {
            $this->ci->load->model("Moderation_model");
        }

        $id = intval($id);
        $flag_new_photo = ($id) ? false : true;

        ///// save text data
        if (!$id) {
            $sorter_param["where"]["type_id"] = $data["type_id"];
            $sorter_param["where"]["object_id"] = $data["object_id"];
            $data["sorter"] = $this->get_files_by_param_count($sorter_param) + 1;

            $data["status"] = 1;
            if ($data["type_id"] && $moderation_use) {
                $type_data = $this->get_type_by_id($data["type_id"]);
                if ($type_data["use_moderation"]) {
                    $data["status"] = intval($this->ci->Moderation_model->get_moderation_type_status($this->moderation_type));
                }
            }

            $data["date_update"] = $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(GALLERY_TABLE, $data);
            $return["id"] = $id = $this->ci->db->insert_id();
        } else {
            $data["date_update"] = date("Y-m-d H:i:s");
            $this->ci->db->where("id", $id);
            $this->ci->db->update(GALLERY_TABLE, $data);
        }

        ////// add moderation alert if type use moderation
        if ($moderation_use) {
            $data = $this->get_file_by_id($id);
            $type_data = $this->get_type_by_id($data["type_id"]);
            if ($type_data["use_moderation"]) {
                $this->ci->Moderation_model->add_moderation_item($this->moderation_type, $id);
            }
        }

        ////// upload file
        if (!empty($file_path)) {
            $data = $this->get_file_by_id($id);
            $type_data = $this->get_type_by_id($data["type_id"]);

            $this->ci->load->model("Uploads_model");
            $img_return = $this->ci->Uploads_model->upload_exist($type_data["gid_upload_config"], $data["postfix"],
                                                                 $file_path);
            if (!empty($img_return["errors"])) {
                if ($flag_new_photo) {
                    $this->delete_file($id);
                }
                $return["errors"] = $img_return["errors"];
            } else {
                if (!empty($img_return['path'])) {
                    $image_size = getimagesize($img_return['path'] . $img_return['file']);
                } else {
                    $image_size = getimagesize($file_path);
                }

                $image_data["file_name"] = $img_return["file"];
                if (isset($image_data["settings"])) {
                    $image_data["settings"] = unserialize($image_data["settings"]);
                } else {
                    $image_data["settings"] = array();
                }
                $image_data["settings"]["width"] = $image_size[0];
                $image_data["settings"]["height"] = $image_size[1];
                $image_data["settings"] = serialize($image_data["settings"]);
                $this->save_file($id, $image_data, "", $moderation_use);
            }
        }

        return $return;
    }

    /**
     * Save file object to data source
     *
     * @param integer $id             photo idetifier
     * @param array   $data           photo data
     * @param string  $upload_name    upload name
     * @param boolean $moderation_use need moderation
     *
     * @return array
     */
    public function save_file_data($id, $data, $moderation_use = true)
    {
        $return = array("id" => $id, "errors" => array());

        $moderation_use = $moderation_use && $this->ci->pg_module->is_module_active("moderation");
        if ($moderation_use) {
            $this->ci->load->model("Moderation_model");
        }

        $id = intval($id);

        ///// save text data
        if (!$id) {
            $sorter_param["where"]["type_id"] = $data["type_id"];
            $sorter_param["where"]["object_id"] = $data["object_id"];
            $data["sorter"] = $this->get_files_by_param_count($sorter_param) + 1;

            $data["status"] = 1;
            if ($data["type_id"] && $moderation_use) {
                $type_data = $this->get_type_by_id($data["type_id"]);
                if ($type_data["use_moderation"]) {
                    $data["status"] = intval($this->ci->Moderation_model->get_moderation_type_status($this->moderation_type));
                }
            }

            $data["date_update"] = $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(GALLERY_TABLE, $data);
            $return["id"] = $id = $this->ci->db->insert_id();
        } else {
            $data["date_update"] = date("Y-m-d H:i:s");
            $this->ci->db->where("id", $id);
            $this->ci->db->update(GALLERY_TABLE, $data);
        }

        ////// add moderation alert if type use moderation
        if ($moderation_use) {
            $data = $this->get_file_by_id($id);
            $type_data = $this->get_type_by_id($data["type_id"]);
            if ($type_data["use_moderation"]) {
                $this->ci->Moderation_model->add_moderation_item($this->moderation_type, $id);
            }
        }

        return $return;
    }

    /**
     * Remove file data from data source by identifier
     *
     * @param integer $object_id file identifier
     *
     * @return void
     */
    public function delete_file($object_id)
    {
        $data = $this->get_file_by_id($object_id);
        $type_data = $this->get_type_by_id($data["type_id"]);

        if (!empty($data["file_name"])) {
            $this->ci->load->model("Uploads_model");
            $this->ci->Uploads_model->delete_upload($type_data["gid_upload_config"], $data["postfix"],
                                                    $data["file_name"]);
        }
        $this->ci->Uploads_model->delete_path($type_data["gid_upload_config"], $data["postfix"]);

        $this->ci->db->where("id", $object_id);
        $this->ci->db->delete(GALLERY_TABLE);

        //// update sorter
        $this->refresh_sorter($data["type_id"], $data["object_id"]);

        //// delete moderation items
        if ($this->ci->pg_module->is_module_active("moderation")) {
            $this->ci->load->model("Moderation_model");
            $this->ci->Moderation_model->delete_moderation_item_by_obj($this->moderation_type, $object_id);
        }

        if ($type_data["module"] && $type_data["model"] && $type_data["callback"]) {
            if (strtolower($type_data["module"] . '_model') != strtolower($type_data["model"])) {
                $model_path = $type_data["module"] . "/models/" . $type_data["model"];
            } else {
                $model_path = $type_data["model"];
            }
            try {
                $this->ci->load->model($model_path);
                $this->ci->{$type_data["model"]}->{$type_data["callback"]}("delete_file", $data['object_id']);
            } catch (Exception $e) {
                
            }
        }
    }

    /**
     * Set sorting value of file object
     *
     * @param integer $id     file identifier
     * @param integer $sorter sorting value
     *
     * @return void
     */
    public function save_sorter($id, $sorter)
    {
        $data["sorter"] = intval($sorter);
        $data["date_update"] = date("Y-m-d H:i:s");
        $this->ci->db->where("id", $id);
        $this->ci->db->update(GALLERY_TABLE, $data);
    }

    /**
     * Remove file object by paramenters
     *
     * @param array $params query parameters
     *
     * @return void
     */
    public function delete_files_by_param($params)
    {
        $files = $this->ci->Upload_gallery_model->get_files_by_param($params);
        foreach ($files as $file) {
            $this->ci->Upload_gallery_model->delete_file($file["id"]);
        }
        unset($files);
    }

    /**
     * Resort files data
     *
     * @param integer $type_id   type identifier
     * @param integer $object_id owner's object identifier
     *
     * @return void
     */
    public function refresh_sorter($type_id, $object_id)
    {
        $results = $this->ci->db->select("id, sorter")->from(GALLERY_TABLE)->where("type_id", $type_id)->where("object_id",
                                                                                                           $object_id)->order_by('sorter ASC')->get()->result_array();
        if (!empty($results)) {
            $i = 1;
            foreach ($results as $r) {
                $data["sorter"] = $i;
                $this->ci->db->where("id", $r["id"]);
                $this->ci->db->update(GALLERY_TABLE, $data);
                ++$i;
            }
        }
    }

    /**
     * Create files rotation
     *
     * @return void
     */
    public function create_rotation()
    {
        
    }

    ///// moderation functions

    /**
     * Return moderate objects as array
     *
     * @param array $object_ids file identifiers
     *
     * @return array
     */
    public function _moder_get_list($object_ids)
    {
        $params["where_in"]["id"] = $object_ids;
        $photos = $this->get_files_by_param($params);

        if (!empty($photos)) {
            foreach ($photos as $photo) {
                if (!isset($photo['media']['thumbs']['small']) && !empty($photo['media']['thumbs'])) {
                    $photo['media']['thumbs']['small'] = array_pop($photo['media']['thumbs']);
                }
                $return[$photo["id"]] = $photo;
            }

            return $return;
        } else {
            return array();
        }
    }

    /**
     * Approve/decline upload object
     *
     * Available status values: 1 - approve, 0 - decline
     *
     * @param integer $object_id upload object identifier
     * @param integer $status    approve status
     *
     * @return void
     */
    public function _moder_set_status($object_id, $status)
    {
        $data["status"] = intval($status);
        $data["date_update"] = date("Y-m-d H:i:s");
        $this->ci->db->where("id", $object_id);
        $this->ci->db->update(GALLERY_TABLE, $data);

        $file = $this->get_file_by_id($object_id);
        if ($file["type_data"]["module"] && $file["type_data"]["model"] && $file["type_data"]["callback"]) {
            if (strtolower($file["type_data"]["module"] . '_model') != strtolower($file["type_data"]["model"])) {
                $model_path = $file["type_data"]["module"] . "/models/" . $file["type_data"]["model"];
            } else {
                $model_path = $file["type_data"]["model"];
            }
            try {
                $this->ci->load->model($model_path);
                $this->ci->{$file["type_data"]["model"]}->{$file["type_data"]["callback"]}("moder_set_status",
                                                                                           $object_id, $status);
            } catch (Exception $e) {
                
            }
        }
    }

    /**
     * Remove upload object by identifier
     *
     * @param integer $object_id upload object identifier
     *
     * @return void
     */
    public function _moder_delete($object_id)
    {
        $file = $this->get_file_by_id($object_id);

        $this->delete_file($object_id);

        if ($file["type_data"]["module"] && $file["type_data"]["model"] && $file["type_data"]["callback"]) {
            if (strtolower($file["type_data"]["module"] . '_model') != strtolower($file["type_data"]["model"])) {
                $model_path = $file["type_data"]["module"] . "/models/" . $file["type_data"]["model"];
            } else {
                $model_path = $file["type_data"]["model"];
            }
            try {
                $this->ci->load->model($model_path);
                $this->ci->{$file["type_data"]["model"]}->{$file["type_data"]["callback"]}("moder_delete",
                                                                                           $file['object_id'],
                                                                                           $file['status']);
            } catch (Exception $e) {
                
            }
        }
    }

    /**
     * Remove upload path
     *
     * @param array   $type_data type data
     * @param integer $object_id linked object identifier
     *
     * @return void
     */
    public function delete_path($type_data, $object_id)
    {
        $this->ci->Uploads_model->delete_path($type_data["gid_upload_config"], $type_data["gid"] . "/" . $object_id);
    }

    /**
     * Recrop file thumb
     *
     * @param integer $file_id      file identifier
     * @param integer $thumb_prefix thumb prefix
     *
     * @return void
     */
    public function recrop_file($file_id, $thumb_prefix)
    {
        $photo = $this->get_file_by_id($file_id);

        $recrop_data["x1"] = $this->input->post("x1", true);
        $recrop_data["y1"] = $this->input->post("y1", true);
        $recrop_data["width"] = $this->input->post("width", true);
        $recrop_data["height"] = $this->input->post("height", true);

        $this->ci->load->model("Uploads_model");
        $this->ci->Uploads_model->recrop_upload($photo["type_data"]["gid_upload_config"], $photo["postfix"],
                                                $photo["file_name"], $recrop_data, $thumb_prefix);

        if ($photo["type_data"]["module"] && $photo["type_data"]["model"] && $photo["type_data"]["callback"]) {
            if (strtolower($photo["type_data"]["module"] . '_model') != strtolower($photo["type_data"]["model"])) {
                $model_path = $photo["type_data"]["module"] . "/models/" . $photo["type_data"]["model"];
            } else {
                $model_path = $photo["type_data"]["model"];
            }
            try {
                $this->ci->load->model($model_path);
                $this->ci->{$photo["type_data"]["model"]}->{$photo["type_data"]["callback"]}("recrop", $file_id,
                                                                                             array('thumb_prefix' => $thumb_prefix, 'settings' => $photo['settings']));
            } catch (Exception $e) {
                
            }
        }
    }

    /**
     * Rotate file image
     *
     * @param integer $upload_id file identifier
     * @param integer $angle     rotate angle
     *
     * @return void
     */
    public function rotate_file($upload_id, $angle)
    {
        $photo = $this->get_file_by_id($upload_id);

        $this->ci->load->model("Uploads_model");
        $this->ci->Uploads_model->rotate_upload($photo["type_data"]["gid_upload_config"], $photo["postfix"],
                                                $photo["file_name"], $angle);

        $data["settings"]["width"] = $photo["settings"]["height"];
        $data["settings"]["height"] = $photo["settings"]["width"];

        $validation_data = $this->validate_file_data($upload_id, $photo["type_id"], $data);
        $this->save_file($upload_id, $validation_data["data"], "", false);

        if ($photo["type_data"]["module"] && $photo["type_data"]["model"] && $photo["type_data"]["callback"]) {
            if (strtolower($photo["type_data"]["module"] . '_model') != strtolower($photo["type_data"]["model"])) {
                $model_path = $photo["type_data"]["module"] . "/models/" . $photo["type_data"]["model"];
            } else {
                $model_path = $photo["type_data"]["model"];
            }
            try {
                $this->ci->load->model($model_path);
                $this->ci->{$photo["type_data"]["model"]}->{$photo["type_data"]["callback"]}("rotate", $upload_id,
                                                                                             array('settings' => $photo['settings']));
            } catch (Exception $e) {
                
            }
        }

        return $data;
    }

    /**
     * Import languages related data of upload_gallery types
     *
     * @param array $data       upload_gallery types data
     * @param array $langs_file data related by languages
     * @param array $langs_ids  languages identifiers
     *
     * @return void
     */
    public function update_langs($data, $langs_file, $langs_ids)
    {
        $gids = array();
        foreach ((array) $data as $type_data) {
            $gids[] = 'btn_' . $type_data['gid'] . '_reorder';
            $gids[] = 'error_' . $type_data['gid'] . '_max_reached';
            $gids[] = 'header_' . $type_data['gid'] . '_add';
            $gids[] = 'header_' . $type_data['gid'] . '_edit';

            foreach ((array) $type_data['thumbs'] as $thumb_gid) {
                $gids[] = 'header_' . $type_data['gid'] . '_size_' . $thumb_gid;
            }

            $gids[] = 'header_' . $type_data['gid'] . '_upload';
            $gids[] = 'success_' . $type_data['gid'] . '_deleted';
            $gids[] = 'success_' . $type_data['gid'] . '_recrop';
            $gids[] = 'success_' . $type_data['gid'] . '_rotate';
            $gids[] = 'success_' . $type_data['gid'] . '_saved';
            $gids[] = 'success_' . $type_data['gid'] . '_sorter';
            $gids[] = 'text_' . $type_data['gid'] . '_active';
            $gids[] = 'text_' . $type_data['gid'] . '_max_count';
            $gids[] = 'text_' . $type_data['gid'] . '_max_size';
            $gids[] = 'text_' . $type_data['gid'] . '_min_width_and_height';
            $gids[] = 'text_' . $type_data['gid'] . '_moderate';
            $gids[] = 'text_' . $type_data['gid'] . '_status';
        }

        foreach ($gids as $gid) {
            $this->ci->pg_language->pages->set_string_langs("upload_gallery", $gid, get($langs_file[$gid], []), $langs_ids);
        }
    }

    /**
     * Export languages related data of upload_gallery types
     *
     * @param array $data      upload_gallery types data
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function export_langs($data, $langs_ids = null)
    {
        $gids = array();
        foreach ((array) $data as $type_data) {
            $gids[] = 'btn_' . $type_data['gid'] . '_reorder';
            $gids[] = 'error_' . $type_data['gid'] . '_max_reached';
            $gids[] = 'header_' . $type_data['gid'] . '_add';
            $gids[] = 'header_' . $type_data['gid'] . '_edit';

            foreach ((array) $type_data['thumbs'] as $thumb_gid) {
                $gids[] = 'header_' . $type_data['gid'] . '_size_' . $thumb_gid;
            }

            $gids[] = 'header_' . $type_data['gid'] . '_upload';
            $gids[] = 'success_' . $type_data['gid'] . '_deleted';
            $gids[] = 'success_' . $type_data['gid'] . '_recrop';
            $gids[] = 'success_' . $type_data['gid'] . '_rotate';
            $gids[] = 'success_' . $type_data['gid'] . '_saved';
            $gids[] = 'success_' . $type_data['gid'] . '_sorter';
            $gids[] = 'text_' . $type_data['gid'] . '_active';
            $gids[] = 'text_' . $type_data['gid'] . '_max_count';
            $gids[] = 'text_' . $type_data['gid'] . '_max_size';
            $gids[] = 'text_' . $type_data['gid'] . '_min_width_and_height';
            $gids[] = 'text_' . $type_data['gid'] . '_moderate';
            $gids[] = 'text_' . $type_data['gid'] . '_status';
        }

        $langs = array();

        return array_merge($langs, $this->ci->pg_language->export_langs("upload_gallery", $gids, $langs_ids));
    }

    /**
     * Validate upload gallery settings for saving to data source
     *
     * @param array $data module settings
     *
     * @return array
     */
    public function validate_settings($data)
    {
        $return = array('errors' => array(), 'data' => array());

        $use_aviary = $this->ci->pg_module->get_module_config('upload_gallery', 'use_aviary');

        if (isset($data["use_aviary"])) {
            $return["data"]["use_aviary"] = $use_aviary = $data['use_aviary'] ? 1 : 0;
        }

        if (isset($data["aviary_api_key"])) {
            $return["data"]["aviary_api_key"] = trim($data['aviary_api_key']);
            if (empty($return["data"]["aviary_api_key"]) && $use_aviary) {
                $return["errors"][] = l("error_aviary_api_key_empty", "upload_gallery");
            }
        }

        return $return;
    }

    /**
     * Save file from aviary editor
     *
     * @param integer $photo_id       photo identifier
     * @param array   $data           photo data
     * @param string  $file_url       upload url
     * @param boolean $use_moderation need mnoderation
     *
     * @return array
     */
    public function save_aviary_file($photo_id, $data, $file_url, $use_moderation)
    {
        $photo = $this->get_file_by_id($photo_id);
        if (!$photo) {
            return false;
        }

        if (function_exists('curl_init')) {
            $defaults = array(
                CURLOPT_URL => $file_url,
                CURLOPT_HEADER => 0,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_TIMEOUT => 4,
                CURLOPT_SSL_VERIFYPEER => false,
            );
            $ch = curl_init();
            curl_setopt_array($ch, $defaults);
            $image_data = curl_exec($ch);
            curl_close($ch);
        } else {
            $image_data = file_get_contents($file_url);
        }

        $path = $this->temp_path;
        if (!is_dir($path)) {
            @mkdir($path, 0777, true);
        }

        $filename = $photo['file_name'];
        file_put_contents($path . '/' . $filename, $image_data);

        $type_data = $this->get_type_by_id($photo["type_id"]);

        if (!empty($photo["file_name"])) {
            $this->ci->load->model("Uploads_model");
            $this->ci->Uploads_model->delete_upload($type_data["gid_upload_config"], $photo["postfix"],
                                                    $photo["file_name"]);
        }
        $this->ci->Uploads_model->delete_path($type_data["gid_upload_config"], $photo["postfix"]);

        $return = $this->save_local_file($photo_id, $data, $path . '/' . $filename);

        return $return;
    }

    /**
     * Save photo after aviary editor
     *
     * @param string $url  photo url
     * @param array  $data request data
     *
     * @return void
     */
    public function save_aviary($url, $data)
    {
        if (empty($data['type']) || empty($data['object_id']) || empty($data['photo_id'])) {
            show_404();
        }

        $data['type'] = intval($data['type']);
        if (!$data['type']) {
            show_404();
        }

        $data['object_id'] = intval($data['object_id']);
        if (!$data['object_id']) {
            show_404();
        }

        $data['photo_id'] = intval($data['photo_id']);
        if (!$data['photo_id']) {
            show_404();
        }

        $type_data = $this->get_type_by_id($data['type']);

        $use_moderation = !empty($data['user_id']);

        $save_data = array('type_id' => $type_data['id'], 'object_id' => $data['object_id']);
        $return = $this->save_aviary_file($data['photo_id'], $save_data, $url, $use_moderation);

        if (empty($return['errors'])) {
            if (strtolower($type_data["module"] . '_model') != strtolower($type_data["model"])) {
                $model_path = $type_data["module"] . "/models/" . $type_data["model"];
            } else {
                $model_path = $type_data["model"];
            }
            try {
                $this->ci->load->model($model_path);
                $this->ci->{$type_data["model"]}->{$type_data["callback"]}("upload_file", $data['object_id']);
            } catch (Exception $e) {
                show_404();
            }
        }
    }

}
