<?php

namespace Pg\Modules\Users\Models;

/**
 * Users module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * User auth model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Auth_model extends \Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Constructor
     *
     * @return Auth_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->ci->load->model("Users_model");
    }

    /**
     * Do sign in by user identifier
     *
     * @param integer $id user identifier
     *
     * @return array
     */
    public function login($id)
    {
        $data["errors"] = array();
        $data["user_data"] = array();
        $data["login"] = false;

        $this->ci->Users_model->set_format_settings('get_safe', false);
        $user_data = $this->ci->Users_model->get_user_by_id($id);
        $this->ci->Users_model->set_format_settings('get_safe', true);
        if (empty($user_data)) {
            $data["errors"][] = l("error_login_invalid_data", "users");
        } else if (!$this->ci->pg_module->get_module_config('users', $user_data['user_type'] . '_login_enabled')) {
            $data["errors"][] = l("error_login_disabled", "users");
        } else {
            if ($user_data["status"] == 0) {
                $data["errors"][] = l("error_disabled_user", "users");
            } elseif ($user_data["confirm"] == 0) {
                $data["errors"][] = l("error_unconfirmed_user", "users");
            } else {
                $data["user_data"] = $user_data;
                $data["login"] = true;

                $this->update_user_session_data($id);
            }
        }

        return $data;
    }

    /**
     * Update data of user session
     *
     * @param integer $id user identifer
     *
     * @return void
     */
    public function update_user_session_data($id)
    {
        $this->ci->Users_model->set_format_settings('get_safe', false);
        $user_data = $this->ci->Users_model->get_user_by_id($id);
        $this->ci->Users_model->set_format_settings('get_safe', true);
        
        if (!$user_data['lang_id']) {
            $user_data['lang_id'] = $this->ci->pg_language->current_lang_id;
            $this->ci->Users_model->save_user($id, ['lang_id' => $user_data['lang_id']]);            
        }       
        
        $session = array(
            'auth_type'   => 'user',
            'user_id'     => $user_data['id'],
            'user_type'   => $user_data['user_type'],
            'name'        => $user_data['output_name'],
            'email'       => $user_data['email'],
            'unique_name' => $user_data['unique_name'],
            'lang_id'     => $user_data['lang_id'],
            'logo'        => $user_data['media'][!empty($user_data['user_logo_moderation']) ? 'user_logo_moderation' : 'user_logo']['thumbs']['small'],
            'currency_id' => array_key_exists('id_currency', $user_data) ? $user_data['id_currency'] : intval($this->ci->session->userdata('currency_id')),
        );
        $this->ci->session->set_userdata($session);
        $this->ci->view->assign('user_session_data', $this->ci->session->all_userdata());

        return;
    }

    /**
     * Do sign in by e-mail and password
     *
     * @param string $email    user email
     * @param string $password user password
     *
     * @return array
     */
    public function login_by_email_password($email, $password)
    {
        $user_data = $this->ci->Users_model->get_user_by_email_password($email, $password);
        if (empty($user_data)) {
            $data["errors"][] = l("error_login_invalid_data", "users");
        } else {
            $data = $this->login($user_data["id"]);
        }

        return $data;
    }

    /**
     * Do log off
     *
     * @return void
     */
    public function logoff()
    {
        $this->ci->session->sess_destroy();

        $_SESSION = array();

        session_destroy();
        session_write_close();
    }

    /**
     * Validate data from login form
     *
     * @param array $data form data
     *
     * @return array
     */
    public function validate_login_data($data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["email"]) && isset($data["password"])) {
            $this->ci->config->load("reg_exps", true);

            $email_expr =  $this->ci->config->item("email", "reg_exps");
            $return["data"]["email"] = strip_tags($data["email"]);
            if (empty($return["data"]["email"]) || !preg_match($email_expr, $return["data"]["email"])) {
                $return["errors"][] = l("error_email_incorrect", "users");
            }

            $password_expr =  $this->ci->config->item("password", "reg_exps");
            $data["password"] = trim(strip_tags($data["password"]));
            if (!preg_match($password_expr, $data["password"])) {
                $return["errors"][] = l("error_password_incorrect", "users");
            } else {
                $return["data"]["password"] = $data["password"];
            }
        } else {
            $return["errors"][] = l("error_login_invalid_data", "users");
        }

        return $return;
    }
}
