<?php

/**
 * Social networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Social networking twitter service model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Social networking
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Twitter_service_model extends Model
{
    const HMAC_SHA1 = 'HMAC-SHA1';
    const RSA_SHA1 = 'RSA-SHA1';
    
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    public $CI;

    /**
     * Url scheme of twitter service api
     *
     * @var string
     */
    const SCHEME = 'https';

    /**
     * Url host of twitter service api
     *
     * @var string
     */
    const HOST = 'api.twitter.com';

    /**
     * URI of twitter authorize request
     *
     * @var string
     */
    const AUTHORIZE_URI = '/oauth/authorize';

    /**
     * URI of twitter token request
     *
     * @var string
     */
    const REQUEST_URI = '/oauth/request_token';

    /**
     * URI of twitter token access
     *
     * @var string
     */
    const ACCESS_URI = '/oauth/access_token';

    /**
     * Options of Authorize request
     *
     * @var integer/false
     */
    protected $_consumer = false;

    /**
     * Class constructor
     *
     * @return Twitter_service_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
    }

    /**
     * Form options of authorize request
     *
     * @param array $params request options
     *
     * @return array
     */
    public function oauth($params)
    {
        if (!array_key_exists('method', $params)) {
            $params['method'] = 'GET';
        }
        if (!array_key_exists('algorithm', $params)) {
            $params['algorithm'] = self::HMAC_SHA1;
        }
        $this->_consumer = $params;
    }

    /**
     * Return request token
     *
     * @param string $callback callback url
     *
     * @return array
     */
    public function get_request_token($callback)
    {
        $baseurl = self::SCHEME . '://' . self::HOST . self::REQUEST_URI;
        $auth = build_auth_array($baseurl, $this->_consumer['key'], $this->_consumer['secret'], array('oauth_callback' => urlencode($callback)), $this->_consumer['method'], $this->_consumer['algorithm']);
        $str = "";
        foreach ($auth as $key => $value) {
            $str .= ",{$key}=\"{$value}\"";
        }
        $str = 'Authorization: OAuth ' . substr($str, 1);
        $response = $this->_connect($baseurl, $str);
        parse_str($response, $resarray);
        $redirect = self::SCHEME . '://' . self::HOST . self::AUTHORIZE_URI . "?oauth_token={$resarray['oauth_token']}";
        if ($this->_consumer['algorithm'] == self::RSA_SHA1) {
            return $redirect;
        } else {
            return array('token_secret' => $resarray['oauth_token_secret'], 'redirect' => $redirect);
        }
    }

    /**
     * Return access token
     *
     * @param string $token    request token
     * @param string $secret   service secret
     * @param string $verifier verifier url
     *
     * @return array
     */
    public function get_access_token($token = false, $secret = false, $verifier = false)
    {
        if ($token === false && isset($_GET['oauth_token'])) {
            $token = $_GET['oauth_token'];
        }
        if ($verifier === false && isset($_GET['oauth_verifier'])) {
            $verifier = $_GET['oauth_verifier'];
        }
        if ($token === false && $verifier === false) {
            $uri = $_SERVER['REQUEST_URI'];
            $uriparts = explode('?', $uri);
            $authfields = array();
            parse_str($uriparts[1], $authfields);
            $token = $authfields['oauth_token'];
            $verifier = $authfields['oauth_verifier'];
        }
        $tokenddata = array('oauth_token' => urlencode($token), 'oauth_verifier' => urlencode($verifier));
        if ($secret !== false) {
            $tokenddata['oauth_token_secret'] = urlencode($secret);
        }
        $baseurl = self::SCHEME . '://' . self::HOST . self::ACCESS_URI;
        $auth = get_auth_header($baseurl, $this->_consumer['key'], $this->_consumer['secret'], $tokenddata, $this->_consumer['method'], $this->_consumer['algorithm']);
        $response = $this->_connect($baseurl, $auth);
        parse_str($response, $oauth);

        return $oauth;
    }

    /**
     * Send authorize request by CURL
     *
     * @param string $url  request url
     * @param array  $auth request options
     *
     * @return string
     */
    protected function _connect($url, $auth)
    {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($ch, CURLOPT_SSLVERSION, 3);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        //​curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1);
        curl_setopt($ch, CURLOPT_SSL_CIPHER_LIST, 'SSLv3');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array($auth));

        $response = curl_exec($ch);
        curl_close($ch);

        return $response;
    }
}
