<?php

/**
 * Users connections module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('USER_CONNECTIONS_TABLE')) {
    define('USER_CONNECTIONS_TABLE', DB_PREFIX . 'user_connections');
}

/**
 * User connections(social networking) model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users connections
 *
 * @category 	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_connections_model extends Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    public $CI;

    /**
     * Link to database object
     *
     * @var object
     */
    public $DB;

    /**
     * All connection object properties in data source
     *
     * @var array
     */
    public $fields_all = array(
        'id',
        'user_id',
        'service_id',
        'access_token',
        'access_token_secret',
        'data',
        'date_end',
    );

    /**
     * Class constructor
     *
     * @return Users_connections_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
        $this->DB = &$this->CI->db;
    }

    /**
     * Save connection object to data source
     *
     * @param integer $connection_id connection identifier
     * @param array   $data          connection data
     *
     * @return integer
     */
    public function save_connection($connection_id = false, $data = array())
    {
        if (!$connection_id) {
            $this->DB->insert(USER_CONNECTIONS_TABLE, $data);
            $connection_id = $this->DB->insert_id();
        } else {
            $this->DB->where('id', $connection_id);
            $this->DB->update(USER_CONNECTIONS_TABLE, $data);
        }

        return $connection_id;
    }

    /**
     * Return connection object by service and user identifiers
     *
     * @param integer $service_id service identifier
     * @param integer $user_id    user idnetifier
     *
     * @return array
     */
    public function get_connection_by_user_id($service_id = false, $user_id = false)
    {
        $data = array();
        $select_attrs = $this->fields_all;
        $this->DB->select(implode(", ", $select_attrs))->from(USER_CONNECTIONS_TABLE)->where(array('service_id' => $service_id, 'user_id' => $user_id))->order_by('id ASC');
        $result = $this->DB->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Return connection object by identifier
     *
     * @param integer $connection_id connection identifier
     *
     * @return array
     */
    public function get_connection_by_id($connection_id = false)
    {
        $data = array();
        $select_attrs = $this->fields_all;
        $this->DB->select(implode(", ", $select_attrs))->from(USER_CONNECTIONS_TABLE)->where(array('id' => $connection_id))->order_by('id ASC');
        $result = $this->DB->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Return connection object by service idnetifier and oauth data
     *
     * @param integer $service_id service identifier
     * @param array   $odata      oauth data
     *
     * @return array
     */
    public function get_connection_by_data($service_id = false, $odata = false)
    {
        $data = array();
        $select_attrs = $this->fields_all;
        $this->DB->select(implode(", ", $select_attrs))->from(USER_CONNECTIONS_TABLE)->where(array('service_id' => $service_id, 'data' => $odata))->order_by('id ASC');
        $result = $this->DB->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Remove connection object by identifier
     *
     * @param integer $connection_id connection identifier
     *
     * @return void
     */
    public function delete_connection($connection_id = false)
    {
        $data = $this->get_connection_by_id($connection_id);
        if (empty($data)) {
            return false;
        }
        $this->DB->where('id', $connection_id);
        $this->DB->delete(USER_CONNECTIONS_TABLE);

        return;
    }

    /**
     * Return connection object by service identifier and oauth data (deprecated)
     *
     * @param integer $service_id service identifier
     * @param array   $odata      oauth data
     *
     * @return array
     */
    public function get_connection_by_phone($service_id = false, $odata = false)
    {
        $data = array();
        $select_attrs = $this->fields_all;
        $this->DB->select(implode(", ", $select_attrs))->from(USER_CONNECTIONS_TABLE)->where(array('service_id' => $service_id, 'data' => $odata))->order_by('id ASC');
        $result = $this->DB->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }
}
