<?php

namespace Pg\Modules\Users_services\Controllers;

/**
 * Users services module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

use Pg\Libraries\View;
use Pg\Modules\Users_services\Models\Users_services_model;

/**
 * Users services user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users services
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_services extends \Controller
{
    /**
     * Constructor
     *
     * @return Users_services
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Users_services_model');
    }

    /**
     * Available paid services
     *
     * @param string $filter fileter by service guid
     *
     * @return void
     */
    public function index($filter = '')
    {
        $user_id = $this->session->userdata('user_id');

        $user_type = $this->session->userdata('user_type');
        $this->view->assign('usertype', $user_type);

        $available_filter = array('contact', 'post', 'combined');
        $filter = (in_array($filter, $available_filter) ? $filter : '');

        $this->load->model('Services_model');

        /* Contact services */
        if (!$filter || $filter == 'contact') {
            $service_template = array('contact_template');
            $contact_services = $this->Services_model->get_active_services($user_id, $service_template);
            foreach ($contact_services as $key => $contact_service) {
                if ($contact_service['type'] != 'tariff') {
                    unset($contact_services[$key]);
                }
            }
            $this->view->assign('contact_services', $contact_services);
        }

        /* Post services */
        if (!$filter || $filter == 'post') {
            $service_template = array('post_template');
            $post_services = $this->Services_model->get_active_services($user_id, $service_template);
            foreach ($post_services as $key => $post_service) {
                if ($post_service['type'] != 'tariff') {
                    unset($post_services[$key]);
                }
            }
            $this->view->assign('post_services', $post_services);

            $service_template = array('combined_template');
            $combined_services = $this->Services_model->get_active_services($user_id, $service_template);
            foreach ($combined_services as $key => $combined_service) {
                if ($combined_service['type'] != 'tariff') {
                    unset($combined_services[$key]);
                }
            }
            $this->view->assign('combined_services', $combined_services);
        }

        /* Combined services */
        if (!$filter || $filter == 'combined') {
            //$service_template = array($user_type.'_combined_template');
            $service_template = array('combined_template');
            $combined_services = $this->Services_model->get_active_services($user_id, $service_template);
            foreach ($combined_services as $key => $combined_service) {
                if ($combined_service['type'] != 'tariff') {
                    unset($combined_services[$key]);
                }
            }
            $this->view->assign('combined_services', $combined_services);
        }

        $this->load->model('Currencies_model');
        $base_currency = $this->Currencies_model->get_currency_default(true);
        $this->view->assign('base_currency', $base_currency);

        // breadcrumbs
        $this->load->model("Menu_model");
        $this->Menu_model->breadcrumbs_set_active(l("header_services", "users"));

        // filter
        switch ($filter) {
            case 'contact': {
                $this->view->assign('template_gids', array('contact_template'));
                break;
            }
            case 'post': {
                $this->view->assign('template_gids', array('post_template'));
                break;
            }
            case 'combined': {
                $this->view->assign('template_gids', array('contact_template', 'post_template'));
                break;
            }
        }

        // seo
        $this->load->model('Users_model');
        $user = $this->Users_model->get_user_by_id($user_id);
        $this->pg_seo->set_seo_data($user);

        $this->session->set_userdata(array('service_redirect' => 'payments/statistic'));

        $this->view->render('services');
    }

    /**
     * Paid services of me
     *
     * @return void
     */
    public function my_services()
    {
        $user_id = $this->session->userdata("user_id");

        $user_type = $this->session->userdata("user_type");
        $this->view->assign("usertype", $user_type);

        $order["id"] = "DESC";

        $params = array(
            "where" => array(
                "id_user"             => $user_id,
                "template_gid"        => "contact_template",
                "status"              => 1,
                "id_users_membership" => 0,
            ),
        );
        $current_services["contact"] = $this->Users_services_model->get_services_list($order, $params);

        $params = array(
            "where" => array(
                "id_user"             => $user_id,
                "template_gid"        => "post_template",
                "status"              => 1,
                "id_users_membership" => 0,
            ),
        );
        $current_services["post"] = $this->Users_services_model->get_services_list($order, $params);

        $params = array(
            "where" => array(
                "id_user"             => $user_id,
                "status"              => 1,
                "id_users_membership" => 0,
            ),
            "where_in" => array(
                "template_gid" => array("contact_template", "post_template"),
            ),
        );
        $current_services["combined"] = $this->Users_services_model->get_services_list($order, $params);

        $this->view->assign("current_services", $current_services);

        $this->load->model('Services_model');

        $service_template = array('contact_template');
        $contact_services = $this->Services_model->get_active_services($user_id, $service_template);
        foreach ($contact_services as $key => $contact_service) {
            if ($contact_service['type'] != 'tariff') {
                unset($contact_services[$key]);
            }
        }
        $this->view->assign('contact_services', $contact_services);

        $service_template = array('post_template');
        $post_services = $this->Services_model->get_active_services($user_id, $service_template);
        foreach ($post_services as $key => $post_service) {
            if ($post_service['type'] != 'tariff') {
                unset($post_services[$key]);
            }
        }
        $this->view->assign('post_services', $post_services);

        $service_template = array('combined_template');
        $combined_services = $this->Services_model->get_active_services($user_id, $service_template);
        foreach ($combined_services as $key => $combined_service) {
            if ($combined_service['type'] != 'tariff') {
                unset($combined_services[$key]);
            }
        }
        $this->view->assign('combined_services', $combined_services);

        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("page_data", $page_data);

        // breadcrumbs
        $this->load->model("Menu_model");
        $this->Menu_model->breadcrumbs_set_active(l("header_services", "users"));

        // seo
        $this->load->model("Users_model");
        $data = $this->Users_model->get_user_by_id($user_id);
        $this->pg_seo->set_seo_data($data);

        $this->view->render("my_services");
    }

    /**
     * Apply service to user
     *
     * @param integer $id          user identifier
     * @param string  $service_gid service guid
     *
     * @return void
     */
    public function apply_service($id, $service_gid = null)
    {
        $id = intval($id);
        if (!$id) {
            show_404();
        }
        $this->session->set_userdata(array('service_redirect' => 'users/account/'));
        $this->load->helper('payments');
        post_location_request(site_url() . 'services/form/' . $service_gid, array('user' => $id));
    }

    /**
     * Check contact is available
     *
     * The method checks the availability of contact for the user.
     *
     * @param integer $id_contact contact identifier
     *
     * @return void
     */
    public function ajax_available_contact($id_contact)
    {
        $return = array('available' => 0, 'content' => '', 'display_login' => 0);

        $id_user = $this->session->userdata('user_id');

        if ($this->session->userdata('auth_type') != 'user') {
            $return['display_login'] = 1;
        } else {
            $this->load->model('Users_services_model');
            $return = $this->Users_services_model->available_contact_action($id_user, $id_contact);
            if ($return["content_buy_block"] == true) {
                $return["content"] = $this->available_contact_block($id_user, $id_contact);
            }
        }
        
        $this->view->assign($return);
        
        $this->view->render();
    }

    /**
     * Load available contact service block
     *
     * @param integer $user_id    user identifier
     * @param integer $contact_id contact identifier
     *
     * @return
     */
    protected function available_contact_block($user_id, $contact_id)
    {
        $user_services = $this->Users_services_model->getUserContactServices($user_id);

        $date_format = $this->pg_date->get_format('date_literal', 'st');
        $date_time_literal = $this->pg_date->get_format('date_time_literal', 'st');

        $block_data = array(
            "id_contact"       => $contact_id,
            "user_services"    => $user_services,
            "date_format"      => $date_format,
            "date_time_format" => $date_time_literal,
        );

        $this->view->assign('block_data', $block_data);

        return $this->view->fetch('ajax_user_package_for_contact');
    }

    /**
     * Activate contact service by ajax
     *
     * @param integer $id_contact_user contact identifier
     * @param integer $id_user_service user service identifier
     *
     * @return void
     */
    public function ajax_activate_contact($id_contact_user, $id_user_service)
    {
        $user_data = array('contact' => $id_contact_user);

        $user_service = $this->Users_services_model->get_service($id_user_service);

        if (empty($user_service)) {
            show404();
        }

        $validate_data = $this->Users_services_model->validate_contact_activate($user_service, $user_data);

        if (!empty($validate_data['errors'])) {
            $status = 0;
            $message = l('error_service_activating', 'users_services');
        } else {
            $this->Users_services_model->service_contact_activate($user_service, $user_data);
            $status = 1;
            $message = l('success_service_activating', 'users_services');
        }

        $this->view->assign('status', $status);
        $this->view->assign('message', $message);
        
        $this->view->render();
    }

    /**
     * Activate featured service by ajax
     *
     * @param integer $id_user_service user service identifier
     *
     * @return void
     */
    public function activate_featured($id_user_service)
    {
        $user_id = $this->session->userdata('user_id');

        $user_data = array('user' => $user_id);

        $user_service = $this->Users_services_model->get_service($id_user_service);

        if (empty($user_service)) {
            show404();
        }

        $validate_data = $this->Users_services_model->validate_featured_activate($user_service, $user_data);
        if (!empty($validate_data['errors'])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
        } else {
            $this->Users_services_model->service_featured_activate($user_service, $user_data);
            $message = l('success_service_activating', 'users_services');
            $this->system_messages->addMessage(View::MSG_SUCCESS, $message);
        }

        $this->view->setRedirect($_SERVER['HTTP_REFERER']);
    }

    /**
     * Activate show logo service
     *
     * @param integer $id_user_service user service identifier
     *
     * @return void
     */
    public function activate_show_logo($id_user_service)
    {
        $user_id = $this->session->userdata('user_id');

        $user_data = array('user' => $user_id);

        $user_service = $this->Users_services_model->get_service($id_user_service);

        if (empty($user_service)) {
            show404();
        }

        $validate_data = $this->Users_services_model->validate_show_logo_activate($user_service, $user_data);
        if (!empty($validate_data['errors'])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
        } else {
            $this->Users_services_model->service_show_logo_activate($user_service, $user_data);
            $message = l('success_service_activating', 'users_services');
            $this->system_messages->addMessage(View::MSG_SUCCESS, $message);
        }

        $this->view->setRedirect($_SERVER['HTTP_REFERER']);
    }
}
