<?php

namespace Pg\Modules\Video_uploads\Models;

/**
 * Video uploads module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Video uploads main model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Video uploads
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Video_uploads_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Path to media files
     *
     * @var string
     */
    public $media_path = "";

    /**
     * Url to media files
     *
     * @var string
     */
    public $media_url = "";

    /**
     * Type of media files
     *
     * @var string
     */
    public $media_type = "";

    /**
     * Available file mimes
     *
     * @var array
     */
    public $mimes = array();

    /**
     * Video uploads configurations cache
     *
     * @var array
     */
    public $config_cache = array();

    /**
     * Constructor
     *
     * @return Video_uploads_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->media_path = SITE_PHYSICAL_PATH . UPLOAD_DIR . 'video/';
        $this->media_url = SITE_VIRTUAL_PATH . UPLOAD_DIR . 'video/';

        if (count($this->mimes) == 0) {
            $this->ci->load->library('upload');
            $this->ci->upload->mimes_types('');
            $this->mimes = $this->ci->upload->mimes;
        }
    }

    /**
     * Return path to media file
     *
     * @param string $config_gid configuration GUID
     * @param string $postfix    file path postfix
     *
     * @return string
     */
    public function get_media_path($config_gid, $postfix = '')
    {
        if (!empty($postfix)) {
            $postfix = $postfix . '/';
        }
        $path = $this->media_path . str_replace('//', '/', $config_gid . '/' . $postfix);

        return $path;
    }

    /**
     * Return url of media file
     *
     * @param string $config_gid configuration GUID
     * @param string $postfix    file path postfix
     *
     * @return string
     */
    public function get_media_url($config_gid, $postfix = '')
    {
        if (!empty($postfix)) {
            $postfix = $postfix . '/';
        }

        return $this->media_url . str_replace('//', '/', $config_gid . '/' . $postfix);
    }

    /**
     * Return configuration object by GUID
     *
     * @param string $config_gid configuration GUID
     *
     * @return array
     */
    public function get_config($config_gid)
    {
        if (empty($this->config_cache[$config_gid])) {
            $this->config_cache[$config_gid] = array();
            $this->ci->load->model('video_uploads/models/Video_uploads_config_model');
            $this->config_cache[$config_gid] = $this->ci->Video_uploads_config_model->get_config_by_gid($config_gid);
        }

        return $this->config_cache[$config_gid];
    }

    /**
     * Upload video file to server
     *
     * @param string  $config_gid configuration GUID
     * @param string  $postfix    path postfix
     * @param string  $upload_gid upload object GUID
     * @param integer $object_id  object identifier of owner video
     * @param array   $file_data  video data
     *
     * @return array
     */
    public function upload($config_gid, $postfix, $upload_gid, $object_id, $file_data)
    {
        $return["errors"] = array();
        $return["file"] = "";
        $file_return = array('error' => '');

        $config_data = $this->get_config($config_gid);

        $this->ci->load->helper('upload');
        $path = $this->get_media_path($config_gid, $postfix);
        if (!is_dir($path)) {
            @mkdir($path, 0777, true);
            @chmod($path, 0777);
        }

        //// upload src file
        $upload_config = array(
            'max_size' => $config_data["max_size"] / 1024, //// in kb
            'use_file_type_check' => false,
            'allowed_types' => $config_data["file_formats_str"],
            'overwrite' => true,
        );

        $type_check = $this->pre_check_file_type(
                $_FILES[$upload_gid]['name'], $_FILES[$upload_gid]['tmp_name'], $_FILES[$upload_gid]['type'],
                $config_data["file_formats"]
        );

        if (!$type_check || !isset($_FILES[$upload_gid]['name'])) {
            $file_return["error"] = "invalid filetype";
        } else {
            $file_name = $_FILES[$upload_gid]['name'];
            $file_name_ext = '.' . $this->get_extension($file_name);
            $new_file_name = $this->format_filename($file_name);
            $file_mime_type = $this->get_validate_file_type(
                    $_FILES[$upload_gid]['tmp_name'], $_FILES[$upload_gid]['type']);

            $_FILES[$upload_gid]['name'] = $new_file_name;
            $file_return = upload_file($upload_gid, $path, $upload_config);
        }
        if ($file_return["error"] != '') {
            $return["errors"] = $file_return["error"];
        } else {
            $file_name = $file_return["data"]["file_name"];
            $file_name_ext = $file_return["data"]["file_ext"];
            @chmod($path . $new_file_name, 0777);

            $return["file"] = $new_file_name;

            $file_data["ext"] = $this->get_extension($new_file_name);
            $file_data["type"] = $file_mime_type;
            $this->ci->load->model('video_uploads/models/Video_uploads_process_model');
            $this->ci->Video_uploads_process_model->prepare($new_file_name, $path, $file_data, $object_id, $config_gid);
        }

        return $return;
    }

    /**
     * Return available types of video file
     *
     * @param string $file           file name
     * @param string $post_file_type file type from post
     *
     * @return string
     */
    protected function get_validate_file_type($file, $post_file_type)
    {
        $file_type = $post_file_type;
        ///// следующим шагом пытаемся определить mime-Тип с поммощью fileinfo, если настройка включена
        $use_fileinfo = $this->pg_module->get_module_config('video_uploads', 'use_fileinfo');

        if ($use_fileinfo) {
            $file_type = '';

            if (function_exists('finfo_open')) {
                $_fileinfo = @finfo_open(FILEINFO_MIME);
                if ($_fileinfo) {
                    $finfo_data = finfo_file($_fileinfo, $file);
                    $t = explode(";", $finfo_data);
                    $file_type = trim($t[0]);
                }
            } elseif (function_exists('mime_content_type')) {
                $file_type = mime_content_type($file);
            }
        }

        return $file_type;
    }

    /**
     * Check file type before uploading
     *
     * @param string $file_name          new file name
     * @param string $file               file name
     * @param string $post_file_type     file type from post
     * @param array  $allowed_extensions allowed file extensions
     *
     * @return boolean
     */
    protected function pre_check_file_type($file_name, $file, $post_file_type, $allowed_extensions)
    {
        if (empty($allowed_extensions)) {
            return false;
        }
        $extension = strtolower($this->get_extension($file_name));

        if (!in_array($extension, $allowed_extensions)) {
            return false;
        }

        $ext_in_allowed = false;

        ///// пытаемся определить mime-Тип с поммощью fileinfo, если настройка включена
        $use_fileinfo = $this->pg_module->get_module_config('video_uploads', 'use_fileinfo');

        if ($use_fileinfo) {
            $file_type = '';

            if (function_exists('finfo_open')) {
                $_fileinfo = @finfo_open(FILEINFO_MIME);
                if ($_fileinfo) {
                    $finfo_data = finfo_file($_fileinfo, $file);
                    $t = explode(";", $finfo_data);
                    $file_type = trim($t[0]);
                }
            } elseif (function_exists('mime_content_type')) {
                $file_type = mime_content_type($file);
            }

            if ($file_type) {
                $ext_in_allowed = false;
                foreach ($allowed_extensions as $ext) {
                    $mime = $this->mimes[$ext];
                    if (is_array($mime)) {
                        if (in_array($file_type, $mime, true)) {
                            $ext_in_allowed = true;
                        }
                    } else {
                        if ($mime == $file_type) {
                            $ext_in_allowed = true;
                        }
                    }
                }
                if ($ext_in_allowed) {
                    return true;
                } else {
                    return false;
                }
            }
        } else {
            //// проверяем $post_file_type в $allowed_extensions
            foreach ($allowed_extensions as $ext) {
                $mime = $this->mimes[$ext];

                if (is_array($mime)) {
                    if (in_array($post_file_type, $mime, true)) {
                        $ext_in_allowed = true;
                    }
                } else {
                    if ($mime == $post_file_type) {
                        $ext_in_allowed = true;
                    }
                }
            }

            if (!$ext_in_allowed) {
                return false;
            }
        }

        return true;
    }

    /**
     * Upload local video file
     *
     * @param string  $config_gid configuration GUID
     * @param string  $postfix    path postfix
     * @param string  $file_path  path to local file
     * @param integer $object_id  object identifier of owner video
     * @param array   $file_data  video data
     *
     * @return array
     */
    public function upload_exists($config_gid, $postfix, $file_path, $object_id, $file_data)
    {
        $return["errors"] = array();
        $return["file"] = "";

        $config_data = $this->get_config($config_gid);

        $this->ci->load->helper('upload');
        $path = $this->get_media_path($config_gid, $postfix);
        if (!is_dir($path)) {
            @mkdir($path, 0777, true);
            @chmod($path, 0777);
        }

        $path_parts = pathinfo($file_path);
        $file_return = array(
            "error" => "",
            "data" => array(
                'file_name' => $path_parts["basename"],
                'file_ext' => "." . $path_parts["extension"],
            ),
        );
        @copy($file_path, $path . $path_parts["basename"]);

        if ($file_return["error"] != '') {
            $return["errors"] = $file_return["error"];
        } else {
            $file_name = $file_return["data"]["file_name"];
            $file_name_ext = $file_return["data"]["file_ext"];
            $new_file_name = $this->format_filename($file_name);

            if ($file_name != $new_file_name) {
                @copy($path . $file_name, $path . $new_file_name);
                @unlink($path . $file_name);
            }
            @chmod($path . $new_file_name, 0777);

            $file_mime_type = $this->get_validate_file_type($path . $new_file_name, 'video/avi');

            $return["file"] = $new_file_name;

            $file_data["ext"] = $this->get_extension($new_file_name);
            $file_data["type"] = $file_mime_type;
            $this->ci->load->model('video_uploads/models/Video_uploads_process_model');
            $this->ci->Video_uploads_process_model->prepare($new_file_name, $path, $file_data, $object_id, $config_gid);
        }

        return $return;
    }

    /**
     * Validate video object for uploading
     *
     * @param string $config_gid configuration GUID
     * @param string $upload_gid file name
     *
     * @return array
     */
    public function validate_upload($config_gid, $upload_gid)
    {
        $file_return = array('error' => array());

        if (empty($_FILES[$upload_gid]) || $_FILES[$upload_gid]['error'] == 4) {
            return $return;
        }

        $config_data = $this->get_config($config_gid);

        $this->ci->load->helper('upload');

        //// upload src file
        $upload_config = array(
            'max_size' => $config_data["max_size"] / 1024,
            'use_file_type_check' => false,
            'allowed_types' => $config_data["file_formats_str"],
            'overwrite' => true,
        );

        $type_check = $this->pre_check_file_type(
                $_FILES[$upload_gid]['name'], $_FILES[$upload_gid]['tmp_name'], $_FILES[$upload_gid]['type'],
                $config_data["file_formats"]
        );

        if (!$type_check) {
            $file_return["error"][] = "invalid filetype";
        } else {
            $file_return = validate_file($upload_gid, $upload_config);
        }

        return $file_return;
    }

    /**
     * Format video object
     *
     * Possible upload types: local or youtube
     *
     * @param string $config_gid  configuration GUID
     * @param string $postfix     path postfix
     * @param string $video       file name of video
     * @param string $image       file name of thumbnail
     * @param string $upload_type upload type
     * @param string $isHTML5     use html5 player
     *
     * @return array
     */
    public function format_upload($config_gid, $postfix, $video, $image, $upload_type = '', $isHTML5 = false)
    {
        $upload = array();
        if (!$video) {
            return array();
        }

        $config_data = $this->get_config($config_gid);
        $upload_type = (!$upload_type) ? $config_data["upload_type"] : $upload_type;

        $path = $this->get_media_path($config_gid, $postfix);
        $url = $this->get_media_url($config_gid, $postfix);

        switch ($upload_type) {
            case 'youtube':
                $this->ci->load->model('video_uploads/models/Video_uploads_youtube_model');
                $upload = array(
                    "file_name" => $video,
                    "width" => $config_data["youtube_settings"]["width"],
                    "height" => $config_data["youtube_settings"]["height"],
                    "embed" => $this->ci->Video_uploads_youtube_model->get_default_embed($video,
                                                                                         $config_data["youtube_settings"]["width"],
                                                                                         $config_data["youtube_settings"]["height"]),
                );
                break;
            case 'local':
                $this->ci->load->model('video_uploads/models/Video_uploads_local_model');
                $upload = array(
                    "path" => $path,
                    "url" => $url,
                    "file_name" => $video,
                    "file_path" => $path . $video,
                    "file_url" => $url . $video,
                    "width" => $config_data["local_settings"]["width"],
                    "height" => $config_data["local_settings"]["height"],
                    "embed" => $this->ci->Video_uploads_local_model->get_default_embed($url . $video,
                                                                                       $config_data["local_settings"]["width"],
                                                                                       $config_data["local_settings"]["height"],
                                                                                       $isHTML5),
                );
                break;
            case 'embed':
                $this->load->library('VideoEmbed');
                $video['width'] = $config_data['local_settings']['width'];
                $video['height'] = $config_data['local_settings']['height'];
                $upload = array(
                    'file_name' => $video,
                    'width' => $config_data['local_settings']['width'],
                    'height' => $config_data['local_settings']['height'],
                    'embed' => $this->videoembed->get_embed_code($video),
                );
                break;
        }

        if ($config_data["use_thumbs"]) {
            if (!empty($image)) {
                $thumbs = $this->format_thumbs($image, $url, $path, $config_data["thumbs_settings"]);
            } elseif ($config_data["default_img"]) {
                $thumbs = $this->format_thumbs($config_data["default_img"], $config_data["default_img_data"]["url"],
                                               $config_data["default_img_data"]["path"], $config_data["thumbs_settings"]);
            }
            if (!empty($thumbs)) {
                $upload = array_merge($upload, $thumbs);
            }
        }

        return $upload;
    }

    /**
     * Remove upload object
     *
     * Possible upload types: local or youtube
     *
     * @param string $config_gid  configuration GUID
     * @param string $postfix     path postfix
     * @param string $video       file name of video
     * @param string $image       file name of thumbnail
     * @param string $upload_type upload_type
     *
     * @return void
     */
    public function delete_upload($config_gid, $postfix, $video, $image = '', $upload_type = '')
    {
        $config_data = $this->get_config($config_gid);
        if (!empty($upload_type)) {
            $config_data["upload_type"] = $upload_type;
        }

        if ($config_data["upload_type"] == 'local') {
            $path = $this->get_media_path($config_gid, $postfix);
            $file = $path . $video;
            if (file_exists($file)) {
                @unlink($file);
            }

            $file = substr($file, 0, -4) . '.mp4';
            if (file_exists($file)) {
                @unlink($file);
            }

            $file = substr($file, 0, -4) . '.ogg';
            if (file_exists($file)) {
                @unlink($file);
            }

            $file = substr($file, 0, -4) . '.webm';
            if (file_exists($file)) {
                @unlink($file);
            }

            if ($image) {
                $this->delete_thumbs($image, $path, $config_data["thumbs_settings"]);
            }
        }
        if ($config_data["upload_type"] == 'youtube') {
            $this->ci->load->model('video_uploads/models/Video_uploads_youtube_model');
            $this->ci->Video_uploads_youtube_model->delete_method($video);
        }

        return;
    }

    /**
     * Return formatted file name of upload video
     *
     * @param string $filename file name of video
     *
     * @return string
     */
    protected function format_filename($filename)
    {
        $path_parts = pathinfo($filename);
        $extension = $path_parts["extension"];
        $str = substr($filename, 0, -(strlen($extension) + 1));
        $str = preg_replace('/\s+/', '-', $str);
        $str = preg_replace('/[^a-z0-9\-_]/i', '', $str);
        if (empty($str) || strlen($str) < 3) {
            $return = $this->generate_filename("." . $extension);
        } else {
            $return = $str . '.' . $extension;
        }

        return $return;
    }

    /**
     * Generate new file name for video object
     *
     * @param string $file_type file type
     *
     * @return string
     */
    protected function generate_filename($file_type)
    {
        srand();

        return substr(md5(date('Y-m-d H:i:s') . rand(1, 9999)), 0, 10) . $file_type;
    }

    /**
     * Return extensions of video file
     *
     * @param string $filename file name of video
     *
     * @return string
     */
    protected function get_extension($filename)
    {
        $x = explode('.', $filename);

        return end($x);
    }

    /**
     * Create thumbnails for video object
     *
     * @param string $file_name   name of video file
     * @param strign $file_path   path to video file
     * @param array  $thumbs_data thumbnails data
     *
     * @return void/false
     */
    public function create_thumbs($file_name, $file_path, $thumbs_data)
    {
        if (empty($thumbs_data)) {
            return false;
        }
        foreach ($thumbs_data as $thumb) {
            $thumb_file = $file_path . $thumb["gid"] . '-' . $file_name;
            if (file_exists($thumb_file)) {
                unlink($thumb_file);
            }
            copy($file_path . $file_name, $thumb_file);
            $this->action($thumb_file, $thumb);
        }
    }

    /**
     * Format thumbnails of video object
     *
     * @param string $file_name   name of video file
     * @param string $url         url to video files
     * @param strign $path        path to video files
     * @param array  $thumbs_data thumbnails data
     *
     * @return array
     */
    public function format_thumbs($file_name, $url, $path, $thumbs_data)
    {
        $upload = array();
        foreach ($thumbs_data as $thumb_data) {
            $upload["thumbs"][$thumb_data["gid"]] = $url . $thumb_data["gid"] . '-' . $file_name;
            $upload["thumbs_data"][$thumb_data["gid"]] = array(
                "file_name" => $thumb_data["gid"] . '-' . $file_name,
                "file_path" => $path . $thumb_data["gid"] . '-' . $file_name,
                "file_url" => $url . $thumb_data["gid"] . '-' . $file_name,
            );
        }

        return $upload;
    }

    /**
     * Remove thumbnails of video object
     *
     * @param string $file_name   name of video file
     * @param strign $file_path   path to video file
     * @param array  $thumbs_data thumbnails data
     *
     * @return void
     */
    public function delete_thumbs($file_name, $file_path, $thumbs_data)
    {
        $image = $file_path . $file_name;
        if (file_exists($image)) {
            @unlink($image);
        }

        if (isset($thumbs_data) && !empty($thumbs_data)) {
            foreach ($thumbs_data as $thumb) {
                $thumb = $file_path . $thumb["gid"] . '-' . $file_name;
                if (file_exists($thumb)) {
                    @unlink($thumb);
                }
            }
        }
    }

    ////// graph function

    /**
     * Do action over thumbnail
     *
     * @param string  $file           name of thumbnail file
     * @param array   $thumb          thumbnail settings
     * @param boolean $dynamic_output output file content
     *
     * @return array
     */
    protected function action($file, $thumb = array(), $dynamic_output = false)
    {
        @ini_set("memory_limit", "512M");
        $this->ci->load->library('image_lib');
        $this->ci->image_lib->clear();
        $error = $this->resize($file, $thumb["width"], $thumb["height"], 'in', true, '000000', $dynamic_output);

        return $error;
    }

    /**
     * Resize thumbnail
     *
     * Possible resize types:
     * in - width/height ratio after resize greater then before
     * out - width/height ratio after resize less then before
     *
     * @param string  $file           name of thumbnail file
     * @param integer $width          new width
     * @param integer $height         new height
     * @param string  $type           resize type
     * @param boolean $maintain       keep width/height ratio
     * @param boolean $color          background color
     * @param boolean $dynamic_output output file content
     *
     * @return array
     */
    protected function resize($file, $width, $height, $type = 'in', $maintain = false, $color = false, $dynamic_output = false)
    {
        $this->ci->load->library('image_lib');
        $this->ci->image_lib->clear();

        $resize_config['source_image'] = $file;
        $resize_config['dynamic_output'] = $dynamic_output;
        $resize_config['create_thumb'] = false;
        $resize_config['maintain_ratio'] = $maintain;
        $resize_config['width'] = $width;
        $resize_config['height'] = $height;

        if ($color !== false) {
            $resize_config['background'] = true;
            $resize_config['bgcolor'] = $color;
            $resize_config['bgwidth'] = $width;
            $resize_config['bgheight'] = $height;
        }

        list($twidth, $theight) = getimagesize($file);
        $k = $width / $height;
        $k1 = $twidth / $theight;

        switch ($type) {
            case "in": $master = ($k1 <= $k) ? "height" : "width";
                break;
            case "out": $master = ($k1 <= $k) ? "width" : "height";
                break;
        }
        $resize_config['master_dim'] = $master;

        $this->ci->image_lib->initialize($resize_config);
        $result = $this->ci->image_lib->resize();

        return $this->ci->image_lib->error_msg;
    }

}
